package mice.cdb.service;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import mice.cdb.service.beamline.BeamlineConstants;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * This is the head of the inheritance tree for CDB components.
 * 
 */
public class CDB {
    /** Reference to logging tool. */
    protected Logger m_log;

    /** Reference to the database. */
    protected DBAccess m_mdb;

    /** Reference to validator tool. */
    protected Validator m_validator = new Validator();

    /** CDB properties object. */
    private static ServerConfig m_config;

    /** Name of the server. */
    private String m_serverName;

    public CDB() {
        // MICE_HOME needs to be set in the CATALINA_OPTS not from the web.xml
        String miceHome = System.getProperty(ServerConstants.MICE_HOME_PROPERTY);
        if (miceHome == null || miceHome.isEmpty()) {
            miceHome = "/opt/mice";
        }
        String log4jFile = miceHome + ServerConstants.LOG4J_LOCATION;
        PropertyConfigurator.configureAndWatch(log4jFile, ServerConstants.LOG4J_INTERVAL_MILLIS);

        try {
            m_config = ServerConfig.getInstance();
            m_serverName = m_config.getString(ServerConstants.SERVER_NAME_PROPERTY);
        } catch (CDBException e) {
            m_serverName = "not set";
        }
        if (m_serverName.isEmpty()) {
            m_serverName = "not set";
        }
    }

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    public String getStatus() {
        Connection connection = null;
        try {
            // before saying we are ok test that we can connect to the database
            connection = m_mdb.getConnect();
        } catch (CDBException e) {
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            // close the connection to the database
            m_mdb.close(connection);
        }
        return "<ok>OK</ok>";
    }

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    public String getName() {
        return "<" + ServerConstants.SERVER_NAME + ">" + m_serverName + "</" + ServerConstants.SERVER_NAME + ">";
    }

    /**
     * Select the start time of the given run.
     * 
     * @param connection
     *            db connection
     * @param runNo
     *            the run number
     * @return the start time of the given run
     * @throws CDBException
     */
    protected Timestamp selectRunStartTime(Connection connection, int runNo) throws CDBException {
        m_log.debug("selectRunStartTime runNo:" + runNo);
        // There will only be one row returned
        String select = "SELECT " + BeamlineConstants.COLUMN_STARTTIME + " FROM " + BeamlineConstants.TABLE_RUN
                + " WHERE " + BeamlineConstants.COLUMN_RUNNUMBER + "=" + runNo + ";";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            if (rs.next()) {
                return rs.getTimestamp(BeamlineConstants.COLUMN_STARTTIME);
            } else {
                m_log.debug("Run " + runNo + " not found.");
                throw new CDBException("Run " + runNo + " not found.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

}
