package mice.cdb.service;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;

/**
 * Interface to the database.
 * 
 */
public class DBAccess {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /** The JDBC driver */
    private String m_jdbcDriver = "org.postgresql.Driver";

    /** The URL of the database. */
    private String m_dbUrl = "jdbc:postgresql://localhost/";

    /** The name of the database. */
    private String m_dbName = "cdb";

    /** The name of the database user. */
    private String m_dbUser = "";

    /** The password of the database user. */
    private String m_dbPwd = "";

    /** Reference to logging tool. */
    private Logger m_log;

    /**
     * Constructs a DBAccess object.
     */
    public DBAccess() {
        init();
        try {
            String dbUser = m_config.getString(ServerConstants.DB_USER);
            if (!dbUser.isEmpty()) {
                m_dbUser = dbUser;
            }
        } catch (CDBException e) {
            m_log.warn(e.getMessage());
        }

        try {
            String dbPwd = m_config.getString(ServerConstants.DB_PWD);
            if (!dbPwd.isEmpty()) {
                m_dbPwd = dbPwd;
            }
        } catch (CDBException e) {
            m_log.warn(e.getMessage());
        }
    }

    /**
     * Constructs a DBAccess object.
     * 
     * @param dbUser
     *            a String containing the database user name
     * @param dbPwd
     *            a String containing the database password
     */
    public DBAccess(String dbUser, String dbPwd) {
        init();
        m_dbUser = dbUser;
        m_dbPwd = dbPwd;
    }

    private void init() {
        m_log = Logger.getLogger(getClass().getName());

        try {
            m_config = ServerConfig.getInstance();

            try {
                String dbUrl = m_config.getString(ServerConstants.DB_URL);
                if (!dbUrl.isEmpty()) {
                    m_dbUrl = dbUrl;
                }
            } catch (CDBException e) {
                m_log.warn(e.getMessage());
            }

            try {
                String dbName = m_config.getString(ServerConstants.DB_NAME);
                if (!dbName.isEmpty()) {
                    m_dbName = dbName;
                }
            } catch (CDBException e) {
                m_log.warn(e.getMessage());
            }

            try {
                String jdbcDriver = m_config.getString(ServerConstants.DB_JDBC_DRIVER);
                if (!jdbcDriver.isEmpty()) {
                    m_jdbcDriver = jdbcDriver;
                }
            } catch (CDBException e) {
                m_log.info(e.getMessage());
            }

        } catch (CDBException e) {
            m_log.warn(e.getMessage());
        }
    }

    /**
     * Load the jdbc driver and connect to the database.
     * 
     * @throws CDBException
     */
    public Connection getConnect() throws CDBException {
        Connection connection = null;
        try {
            // Load jdbc driver
            Class.forName(m_jdbcDriver).newInstance();
            if (m_log.isDebugEnabled()) {
                m_log.debug("Driver found");
            }
        } catch (ClassNotFoundException e) { // driver not found
            m_log.error("Unable to load database driver. Contact Database Administrator. \nDetails : " + e);
            throw new CDBException("Unable to load database driver. Contact Database Administrator.", e);
        } catch (Exception e) {
            m_log.error("Unable to instantiate driver for db. Contact Database Administrator.");
            throw new CDBException("Unable to instantiate driver for db. Contact Database Administrator.", e);
        }

        try {
            connection = DriverManager.getConnection(m_dbUrl + m_dbName, m_dbUser, m_dbPwd);
            if (m_log.isDebugEnabled()) {
                m_log.debug("Should be connected to " + m_dbUrl + m_dbName + " with user " + m_dbUser);
            }
        } catch (SQLException e) {
            m_log.error("Could not get a connection with database " + m_dbName + " at " + m_dbUrl + " with user "
                    + m_dbUser + ". \n\t" + e.getMessage());
            throw new CDBException("Could not get a connection with database " + m_dbName + " at " + m_dbUrl + ". "
                    + e.getMessage());
        }

        return connection;
    }

    /**
     * Perform a query on the database.
     * 
     * @param connection
     *            db connection
     * @param query
     *            a String containing the query to perform
     * @return a ResultSet containing the results of the query
     * @throws CDBException
     */
    public ResultSet query(Connection connection, String query) throws CDBException {
        m_log.debug("query: " + query);
        if (connection == null) {
            m_log.error("Internal error, query called before database connect");
            throw new CDBException("Internal error, query called before database connect");
        }
        Statement statement = null;
        try {
            statement = connection.createStatement();
            return statement.executeQuery(query);
        } catch (SQLException e) {
            m_log.error("Problem issuing query " + query + "\n\t" + e.getMessage());
            throw new CDBException("Problem issuing query " + query, e);
        }
    }

    /**
     * Insert data into the database.
     * 
     * @param connection
     *            db connection
     * @param sql
     *            a String containing the insert statement
     * @throws CDBException
     */
    public void insert(Connection connection, String sql) throws CDBException {
        m_log.debug("insert: " + sql);
        if (connection == null) {
            m_log.error("Internal error, insert called before database connect");
            throw new CDBException("Internal error, insert called before database connect");
        }
        Statement statement = null;

        try {
            statement = connection.createStatement();
            statement.execute(sql);
            return;
        } catch (SQLException e) {
            StringBuilder message = new StringBuilder("Problem inserting data into the database. \n\t" + e.getMessage());
            if (e.getSQLState().equals("23505")) {
                m_log.warn(message);
                throw new CDBException("Problem inserting data into the database - duplicate record.");
            }
            m_log.error(message);
            throw new CDBException("Problem inserting data into the database.");

        } finally {
            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    // ignore
                }
            }
        }
    }

    /**
     * Insert batch data into the database. If there is an error with any of the
     * inserts the transaction will be rolled back.
     * 
     * @param connection
     *            db connection
     * @param insert
     *            a String containing the insert statements
     * @return the number of rows inserted
     * @throws CDBException
     */
    public int batchInsert(Connection connection, String insert) throws CDBException {
        m_log.debug("batchInsert: " + insert);
        if (connection == null) {
            m_log.error("Internal error, batch insert called before database connect");
            throw new CDBException("Internal error, batch insert called before database connect");
        }
        boolean roleback = true;
        Statement statement = null;
        String[] inserts = insert.split(";");

        try {
            connection.setAutoCommit(false);
            statement = connection.createStatement();
            for (int i = 0; i < inserts.length; i++) {
                statement.addBatch(inserts[i] + ";");
            }
            int[] results = statement.executeBatch();
            int inserted = 0;
            for (int i = 0; i < results.length; i++) {
                inserted = inserted + results[i];
            }
            connection.commit();
            connection.setAutoCommit(true);
            roleback = false;
            return inserted;

        } catch (SQLException e) {
            StringBuilder message = new StringBuilder(
                    "Problem with batch insert into the database. No data has been inserted.\n\t" + e.getMessage());
            SQLException eNext = e.getNextException();
            if (eNext != null) {
                message.append("\n\t" + eNext.getMessage());
                if (eNext.getSQLState().equals("23505")) {
                    m_log.warn(message);
                    throw new CDBException(
                            "Problem inserting data into the database - duplicate record. No data has been inserted.");
                }
            }
            m_log.error(message);
            throw new CDBException("Problem with batch insert into the database. No data has been inserted.");

        } finally {
            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    // ignore
                }
            }
            if (roleback) {
                try {
                    connection.rollback();
                } catch (SQLException se) {
                    // ignore
                }
            }
            try {
                connection.setAutoCommit(true);
            } catch (SQLException e) {
                // ignore
            }
        }
    }

    public PreparedStatement prepareStatement(Connection connection, String sql) throws SQLException {
        return connection.prepareStatement(sql);
    }

    /**
     * Close the connection to the database.
     * 
     * @param connection
     *            db connection
     */
    public void close(Connection connection) {
        if (connection != null) {
            try {
                connection.close();
                m_log.debug("Connection closed");
            } catch (SQLException e) { // Do nothing
                m_log.debug("Could not close connection normally.");
            }
        }
    }

}
