package mice.cdb.service.alarmhandler;

import java.sql.Timestamp;

/**
 * A representation of a single record of the alarm table.
 * 
 */
public class Alarm extends AlarmHandlerConstants {
    /** The name of the alarm. */
    private String m_name = "";

    /** The upper limit for a critical state. */
    private double m_hihi;

    /** The upper limit for a critical state has been set. */
    private boolean m_hihiSet = false;

    /** The upper limit for a warning state. */
    private double m_hi;

    /** The upper limit for a warning state has been set. */
    private boolean m_hiSet = false;

    /** The lower limit for a warning state. */
    private double m_lo;

    /** The lower limit for a warning state has been set. */
    private boolean m_loSet = false;

    /** The lower limit for a critical state. */
    private double m_lolo;

    /** The lower limit for a critical state has been set. */
    private boolean m_loloSet = false;

    /**
     * Constructs an Alarm object.
     */
    public Alarm() {
    }

    /**
     * Constructs an Alarm object.
     * 
     * @param name
     *            a string containing the name of the alarm
     * @param hihi
     *            the higher value which the alarm considers an error
     * @param hi
     *            the higher value which the alarm considers a warning
     * @param lo
     *            the lower value which the alarm considers a warning
     * @param lolo
     *            the lower value which the alarm considers an error
     */
    protected Alarm(String name, double hihi, double hi, double lo, double lolo) {
        m_name = name;
        m_hihi = hihi;
        m_hi = hi;
        m_lo = lo;
        m_lolo = lolo;
    }

    /**
     * Set the higher value which the alarm considers a warning.
     * 
     * @param hi
     *            the higher value which the alarm considers a warning
     */
    public void setHi(double hi) {
        m_hi = hi;
        m_hiSet = true;
    }

    /**
     * Set the higher value which the alarm considers an error.
     * 
     * @param hihi
     *            the higher value which the alarm considers an error
     */
    public void setHiHi(double hihi) {
        m_hihi = hihi;
        m_hihiSet = true;
    }

    /**
     * Set the lower value which the alarm considers a warning.
     * 
     * @param lo
     *            the lower value which the alarm considers a warning
     */
    public void setLo(double lo) {
        m_lo = lo;
        m_loSet = true;
    }

    /**
     * Set the lower value which the alarm considers an error.
     * 
     * @param lolo
     *            the lower value which the alarm considers an error
     */
    public void setLoLo(double lolo) {
        m_lolo = lolo;
        m_loloSet = true;
    }

    /**
     * Set the name of the Alarm.
     * 
     * @param name
     *            a string containing the name of the alarm
     * 
     * @see #getName
     */
    public void setName(String name) {
        m_name = name;
    }

    /**
     * Get the name of the Alarm.
     * 
     * @return the name of the alarm
     * 
     * @see #setName
     */
    public String getName() {
        return m_name;
    }

    /**
     * Validate that all values are set and make sense.
     * 
     * @return <code>true</code> if OK
     */
    public boolean validate() {
        if (m_name == null | m_name.isEmpty()) {
            return false;
        }

        if (!m_loloSet | !m_loSet | !m_hiSet | !m_hihiSet) {
            return false;
        }

        if (m_lolo > m_lo | m_lo > m_hi | m_hi > m_hihi) {
            return false;
        }
        return true;
    }

    /**
     * Return a string containing an SQL insert statement that encapsulates the
     * data in the Alarm object.
     * 
     * @param creationTime
     *            a timestamp of when the tag was created
     * @param tag
     *            a name used to group Alarms
     * @return a string containing an SQL insert statement
     */
    protected String getInsertQuery(Timestamp creationTime, String tag) {
        return "INSERT INTO " + TABLE_ALARM + " (" + COLUMN_NAME + ", " + COLUMN_HIHI + ", " + COLUMN_HI + ", "
                + COLUMN_LO + ", " + COLUMN_LOLO + ", " + COLUMN_CREATIONTIME + ", " + COLUMN_TAG + ") VALUES ('"
                + m_name + "', " + m_hihi + ", " + m_hi + ", " + m_lo + ", " + m_lolo + ", '" + creationTime + "', '"
                + tag + "');";
    }

    /**
     * Return an xml representation of the Alarm.
     * 
     * @return an xml representation of the Alarm
     */
    protected String toXMLString() {
        return "<alarm>" + "<name>" + m_name + "</name>" + "<hihi>" + m_hihi + "</hihi>" + "<hi>" + m_hi + "</hi>"
                + "<lo>" + m_lo + "</lo>" + "<lolo>" + m_lolo + "</lolo>" + "</alarm>";
    }

}
