package mice.cdb.service.alarmhandler;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>AlarmHandler</code> interface is used to retrieve Alarm Handlers
 * (ALH). To store ALH use the {@link AlarmHandlerSuperMouse} interface. It also
 * keeps track of when an ALH was in use. An ALH is associated with a tag,
 * {@link AlarmHandlerSuperMouse#setTaggedALH(String, String)
 * AlarmHandlerSuperMouse.setTaggedALH}, and retrieved via that tag,
 * {@link #getTaggedALH(String) getTaggedALH}. It is also possible to associated
 * a new ALH with an existing tag using <code>setTaggedALH</code>; the old ALH
 * is stored for diagnostic purposes.
 * </p>
 * 
 * <p>
 * A couple of methods are provided for diagnostics. It is possible to retrieve
 * a list of tags that were in use over a given time period,
 * {@link #getUsedTags(String, String) getUsedTags}, and to retrieve the values
 * of an ALH that was in use at a given time, {@link #getUsedALH(String)
 * getUsedALH}.
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 */
@WebService
public interface AlarmHandler {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the values of the AlarmHandler for the given tag with the greatest
     * creation time.
     * 
     * @param tag
     *            the name of the tag
     * @return a string containing XML representing the tagged AlarmHandler with
     *         the format of the XML as
     *         <dl>
     *         <dd>
     *         <code>&lt;alarmhandler creationtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' tag='xxx'&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;&lt;/name&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;&lt;/hihi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;&lt;/hi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;&lt;/lo&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;&lt;/lolo&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;&lt;/name&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;&lt;/hihi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;&lt;/hi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;&lt;/lo&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;&lt;/lolo&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *         <dd><code>&lt;/alarmhandler&gt;</code></dd>
     *         </dl>
     * 
     * @see mice.cdb.service.alarmhandler.AlarmHandlerSuperMouse#setTaggedALH
     */
    @WebMethod
    public abstract String getTaggedALH(String tag);

    /**
     * Get the AlarmHandler that was in use for the given timestamp.
     * 
     * @param timestamp
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * @return a string containing XML representing the selected AlarmHandler
     *         with the format of the XML as
     *         <dl>
     *         <dd>
     *         <code>&lt;alarmhandler creationtime='yyyy-mm-dd hh:mm:ss[.fffffffff]' tag='xxx'&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;&lt;/name&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;&lt;/hihi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;&lt;/hi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;&lt;/lo&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;&lt;/lolo&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;&lt;/name&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;&lt;/hihi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;&lt;/hi&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;&lt;/lo&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;&lt;/lolo&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *         <dd><code>&lt;/alarmhandler&gt;</code></dd>
     *         </dl>
     * 
     * @see mice.cdb.service.alarmhandler.AlarmHandlerSuperMouse#setTaggedALH
     */
    @WebMethod
    public abstract String getUsedALH(String timestamp);

    /**
     * Get a list of tags that were in use for the given time period. This will
     * include any tags that were in use at the start time. NB one start time
     * will be before that of the selected start time. The results contain the
     * name of the tag and the time that it was set as in use. Implicitly a tag
     * stopped being in use when the next tag was set as in use. It is possible
     * for the same tag to be repeated consecutively and point to different
     * AlarmHandlers. The details of an individual AlarmHandler can be found
     * using the <code>getUsedALH</code> method.
     * 
     * @param startTime
     *            the start of the time period in the format 'yyyy-mm-dd
     *            hh:mm:ss[.fffffffff]' in utc
     * @param endTime
     *            the end of the time period in the format 'yyyy-mm-dd
     *            hh:mm:ss[.fffffffff]' in utc
     * @return a string containing XML in the format
     *         <dl>
     *         <dd><code>&lt;tags&gt;</code></dd>
     *         <dd>&nbsp;&nbsp;&nbsp;&nbsp;
     *         <code>&lt;tag starttime='yyyy-mm-dd hh:mm:ss[.fffffffff]' name='xxx'/&gt;</code>
     *         </dd>
     *         <dd>&nbsp;&nbsp;&nbsp;&nbsp;
     *         <code>&lt;tag starttime='yyyy-mm-dd hh:mm:ss[.fffffffff]' name='yyy'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/tags&gt;</code></dd>
     *         </dl>
     * 
     * @see mice.cdb.service.alarmhandler.AlarmHandlerSuperMouse#setTagInUse
     */
    @WebMethod
    public abstract String getUsedTags(String startTime, String endTime);

    /**
     * Return a list of known tags. If there are no tags then no
     * <code>&lt;tag&gt;</code> elements will be returned.
     * 
     * @return a string containing XML in the format
     *         <dl>
     *         <dd><code>&lt;tags&gt;</code></dd>
     *         <dd>&nbsp;&nbsp;&nbsp;&nbsp;<code>&lt;tag name='xxx'/&gt;</code></dd>
     *         <dd>&nbsp;&nbsp;&nbsp;&nbsp;<code>&lt;tag name='yyy'/&gt;</code></dd>
     *         <dd><code>&lt;/tags&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String listTags();

}