package mice.cdb.service.alarmhandler;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>AlarmHandler</code>.
 * 
 * @see mice.cdb.service.alarmhandler.AlarmHandler
 */
@WebService(endpointInterface = "mice.cdb.service.alarmhandler.AlarmHandler")
public class AlarmHandlerImpl extends CDB implements AlarmHandler {

    /**
     * Constructs an AlarmHandler object.
     */
    public AlarmHandlerImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public String getTaggedALH(String tag) {
        AlarmList alarmList;
        Connection connection = null;

        try {
            m_validator.validateString(tag);
            connection = m_mdb.getConnect();
            // get the timestamp of the most recent version of the tag
            Timestamp creationTime = selectMaxCreationTime(connection, tag);
            // get the list of alarms for the latest version of the tag
            alarmList = selectAlarmList(connection, creationTime, tag);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return alarmList.toXMLString();
    }

    @Override
    public String getUsedALH(String timestamp) {
        AlarmList alarmList;
        Connection connection = null;

        try {
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);
            connection = m_mdb.getConnect();
            // get the list of alarms that were in use at the given timestamp
            alarmList = selectAlarmList(connection, usersTimestamp);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return alarmList.toXMLString();
    }

    @Override
    public String getUsedTags(String startTime, String endTime) {
        String xml;
        Connection connection = null;

        try {
            Timestamp start = m_validator.getTimeStamp(startTime);
            Timestamp end = m_validator.getTimeStamp(endTime);
            connection = m_mdb.getConnect();
            // get the tags that were in use for the given time period
            xml = selectTags(connection, start, end);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return xml;
    }

    @Override
    public String listTags() {
        String xml;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            // get a list of tags
            xml = selectTags(connection);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return xml;
    }

    /**
     * Get the latest creation time for the given tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of the tag
     * @return the latest creation time for the given tag
     * @throws CDBException
     */
    private Timestamp selectMaxCreationTime(Connection connection, String tag) throws CDBException {
        m_log.debug("selectMaxCreationTime tag: " + tag);

        // There will only be one row returned
        String select = "SELECT max(" + AlarmHandlerConstants.COLUMN_CREATIONTIME + ") FROM "
                + AlarmHandlerConstants.TABLE_ALARM + " WHERE " + AlarmHandlerConstants.COLUMN_TAG + "='" + tag + "';";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            rs.next();
            Timestamp creationTime = rs.getTimestamp("max");
            if (creationTime != null) {
                return creationTime;
            } else {
                m_log.debug("No match found for tag = " + tag + ". \n\t" + select);
                throw new CDBException("No match found for tag = " + tag + ".");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Get the list of Alarms that have the given creation time and tag.
     * 
     * @param connection
     *            db connection
     * @param creationTime
     *            a timestamp containing the creation time of the AlarmHandler
     * @param tag
     *            the name of the tag
     * @return a list of Alarms
     * @throws CDBException
     */
    private AlarmList selectAlarmList(Connection connection, Timestamp creationTime, String tag) throws CDBException {
        m_log.debug("selectAlarmList creationTime: " + creationTime + ", tag: " + tag);
        AlarmList alarmList = new AlarmList(creationTime, tag);
        String select = "SELECT " + AlarmHandlerConstants.COLUMN_NAME + ", " + AlarmHandlerConstants.COLUMN_HIHI + ", "
                + AlarmHandlerConstants.COLUMN_HI + ", " + AlarmHandlerConstants.COLUMN_LO + ", "
                + AlarmHandlerConstants.COLUMN_LOLO + " FROM " + AlarmHandlerConstants.TABLE_ALARM + " WHERE "
                + AlarmHandlerConstants.COLUMN_CREATIONTIME + "='" + creationTime + "' and "
                + AlarmHandlerConstants.COLUMN_TAG + "='" + tag + "';";
        m_log.debug(select);
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                Alarm alarm = new Alarm(rs.getString(AlarmHandlerConstants.COLUMN_NAME),
                        rs.getDouble(AlarmHandlerConstants.COLUMN_HIHI), rs.getDouble(AlarmHandlerConstants.COLUMN_HI),
                        rs.getDouble(AlarmHandlerConstants.COLUMN_LO), rs.getDouble(AlarmHandlerConstants.COLUMN_LOLO));
                alarmList.add(alarm);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return alarmList;
    }

    /**
     * Get the list of Alarms that were in use at a given time.
     * 
     * @param connection
     *            db connection
     * @param timestamp
     *            a timestamp containing the time of interest
     * @return a list of Alarms
     * @throws CDBException
     */
    private AlarmList selectAlarmList(Connection connection, Timestamp timestamp) throws CDBException {
        m_log.debug("selectAlarmList timestamp: " + timestamp);
        AlarmList alarmList;
        Timestamp startTime = selectStartTime(connection, timestamp);
        if (startTime == null) {
            m_log.debug("Starttime: " + timestamp + " out of range of values in the database.");
            throw new CDBException("Starttime: " + timestamp + " out of range of values in the database.");
        }

        // The WHERE clause includes the primary key so there will be a maximum
        // of one row returned
        String select = "SELECT " + AlarmHandlerConstants.COLUMN_ALARM_CREATIONTIME + ", "
                + AlarmHandlerConstants.COLUMN_ALARM_TAG + " FROM " + AlarmHandlerConstants.TABLE_ALARM_HISTORY
                + " WHERE " + AlarmHandlerConstants.COLUMN_START_TIME + "='" + startTime + "';";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            if (rs.next()) {
                alarmList = selectAlarmList(connection,
                        rs.getTimestamp(AlarmHandlerConstants.COLUMN_ALARM_CREATIONTIME),
                        rs.getString(AlarmHandlerConstants.COLUMN_ALARM_TAG));
            } else {
                m_log.debug("No data returned from the database.\n\t" + select);
                throw new CDBException("No data returned from the database.\n" + select);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return alarmList;
    }

    /**
     * Get the start time that an AlarmHandler was in use from, where the
     * AlarmHandler was in use at the given time.
     * 
     * @param connection
     *            db connection
     * @param timestamp
     *            a timestamp containing the time of interest
     * @return the start timestamp for the period of use. NB maybe
     *         <code>null</code> if the given timestamp is earlier than the
     *         earliest recorded starttime.
     * @throws CDBException
     */
    private Timestamp selectStartTime(Connection connection, Timestamp timestamp) throws CDBException {
        m_log.debug("selectStartTime timestamp: " + timestamp);
        // There will only be one row returned
        String select = "SELECT max(" + AlarmHandlerConstants.COLUMN_START_TIME + ") FROM "
                + AlarmHandlerConstants.TABLE_ALARM_HISTORY + " WHERE " + AlarmHandlerConstants.COLUMN_START_TIME
                + "<='" + timestamp + "';";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            rs.next();
            return rs.getTimestamp("max");
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Get a list of tags that were in use for the given time period.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time period in the format 'yyyy-mm-dd
     *            hh:mm:ss.sss' in UTC
     * @param endTime
     *            the end of the time period in the format 'yyyy-mm-dd
     *            hh:mm:ss.sss' in UTC
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTags(Connection connection, Timestamp startTime, Timestamp endTime) throws CDBException {
        m_log.debug("selectTags startTime: " + startTime + ", endTime: " + endTime);
        Timestamp tagStartTime = selectStartTime(connection, startTime);
        String select;
        // tagStartTime will be null if the startTime was earlier than the
        // earliest recorded startTime, so we want everything up to the endTime
        if (tagStartTime == null) {
            select = "SELECT " + AlarmHandlerConstants.COLUMN_START_TIME + ", "
                    + AlarmHandlerConstants.COLUMN_ALARM_TAG + " FROM " + AlarmHandlerConstants.TABLE_ALARM_HISTORY
                    + " WHERE " + AlarmHandlerConstants.COLUMN_START_TIME + "<='" + endTime + "' ORDER BY "
                    + AlarmHandlerConstants.COLUMN_START_TIME + ";";
        } else {
            select = "SELECT " + AlarmHandlerConstants.COLUMN_START_TIME + ", "
                    + AlarmHandlerConstants.COLUMN_ALARM_TAG + " FROM " + AlarmHandlerConstants.TABLE_ALARM_HISTORY
                    + " WHERE " + AlarmHandlerConstants.COLUMN_START_TIME + ">='" + tagStartTime + "' and "
                    + AlarmHandlerConstants.COLUMN_START_TIME + "<='" + endTime + "' ORDER BY "
                    + AlarmHandlerConstants.COLUMN_START_TIME + ";";
        }
        StringBuilder xml = new StringBuilder();
        xml = xml.append("<tags>");
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                xml = xml.append("<tag starttime='").append(rs.getString(AlarmHandlerConstants.COLUMN_START_TIME))
                        .append("' name='").append(rs.getString(AlarmHandlerConstants.COLUMN_ALARM_TAG)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml = xml.append("</tags>");
        return xml.toString();
    }

    /**
     * Get a list of all known tags.
     * 
     * @param connection
     *            db connection
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTags(Connection connection) throws CDBException {
        m_log.debug("selectTags");
        String select = "SELECT DISTINCT(" + AlarmHandlerConstants.COLUMN_TAG + ") FROM "
                + AlarmHandlerConstants.TABLE_ALARM + ";";
        StringBuilder xml = new StringBuilder();
        xml = xml.append("<tags>");
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                xml = xml.append("<tag name='").append(rs.getString(AlarmHandlerConstants.COLUMN_TAG)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + select + "\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml = xml.append("</tags>");
        return xml.toString();
    }

}
