package mice.cdb.service.alarmhandler;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>AlarmHandlerSuperMouse</code> interface is used to store and
 * retrieve Alarm Handlers (ALH). It also keeps track of when an ALH was in use.
 * An ALH is associated with a tag, {@link #setTaggedALH(String, String)
 * setTaggedALH} , and retrieved via that tag, {@link #getTaggedALH(String)
 * getTaggedALH}. It is also possible to associated a new ALH with an existing
 * tag using <code>setTaggedALH</code> ; the old ALH is stored for diagnostic
 * purposes.
 * </p>
 * 
 * <p>
 * A couple of methods are provided for diagnostics. It is possible to retrieve
 * a list of tags that were in use over a given time period,
 * {@link #getUsedTags(String, String) getUsedTags}, and to retrieve the values
 * of an ALH that was in use at a given time, {@link #getUsedALH(String)
 * getUsedALH}.
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * The returned XML will contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface AlarmHandlerSuperMouse extends AlarmHandler {

    /**
     * Set the values of the AlarmHandler with the given tag. If an AlarmHandler
     * with the tag already exists then the passed in AlarmHandler will become
     * the new AlarmHandler for that tag. If there is an old AlarmHandler that
     * has been used, it will be accessible via the <code>getUsedTags</code> and
     * <code>getUsedALH</code> methods.
     * 
     * @param tag
     *            the name of the tag
     * @param xml
     *            a string containing XML representing the AlarmHandler with the
     *            format
     *            <dl>
     *            <dd><code>&lt;alarmhandler&gt;</code></dd>
     *            <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;alarm1&lt;/name&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;10.0&lt;/hihi&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;9.5&lt;/hi&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;8.5&lt;/lo&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;8.0&lt;/lolo&gt;</code>
     *            </dd>
     *            <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *            <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;alarm&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;name&gt;alarm2&lt;/name&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hihi&gt;1000&lt;/hihi&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;hi&gt;950&lt;/hi&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lo&gt;700&lt;/lo&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;lolo&gt;600&lt;/lolo&gt;</code>
     *            </dd>
     *            <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/alarm&gt;</code></dd>
     *            <dd><code>&lt;/alarmhandler&gt;</code></dd>
     *            </dl>
     *            where <code>hihi =&gt; hi =&gt; lo =&gt; lolo</code>
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     * 
     * @see #getTaggedALH
     */
    @WebMethod
    public abstract String setTaggedALH(String tag, String xml);

    /**
     * Record the fact that the AlarmHandler with the given tag and creation
     * time is now in use. Only one AlarmHandler maybe in use at a time. A call
     * to this method will supersede all previous calls to this method.
     * 
     * @param tag
     *            the name of the tagged AlarmHandler
     * @param creationTime
     *            the timestamp of when the AlarmHandler was created in the
     *            format yyyy-mm-dd hh:mm:ss[.fffffffff]
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     * 
     * @see #getUsedTags(String, String)
     */
    @WebMethod
    public abstract String setTagInUse(String tag, String creationTime);

}