package mice.cdb.service.alarmhandler;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.parser.AlarmHandlerParser;
import mice.cdb.service.parser.ParseException;

/**
 * An implementation of the <code>AlarmHandlerSuperMouse</code>.
 * 
 * @see mice.cdb.service.alarmhandler.AlarmHandlerSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.alarmhandler.AlarmHandlerSuperMouse")
public class AlarmHandlerSuperMouseImpl extends AlarmHandlerImpl implements AlarmHandlerSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Constructs an AlarmHandler object with super mouse powers.
     */
    public AlarmHandlerSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String setTaggedALH(String tag, String xml) {
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        int inserted = 0;
        Connection connection = null;

        try {
            m_validator.validateString(tag);
            connection = m_mdb.getConnect();
            // test to see if tag + creation time already exists - Extremely
            // unlikely
            if (!newEntry(connection, creationTime, tag)) {
                return "<warning>AlarmHandler with tag = " + tag + " and creation time = " + creationTime
                        + " already exists. Please try again.</warning>";
            }

            // parse the XML
            AlarmHandlerParser ahParser = new AlarmHandlerParser();
            try {
                ahParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse AlarmHandler XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            AlarmList alarmList = ahParser.getAlarmList();
            String insert = alarmList.getInsertQuery(creationTime, tag);
            inserted = m_mdb.batchInsert(connection, insert);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        String message = "";
        if (inserted == 1) {
            message = "Inserted " + inserted + " alarm for tag " + tag;
        } else {
            message = "Inserted " + inserted + " alarms for tag " + tag;
        }
        m_log.info(message);
        return "<ok>" + message + "</ok>";
    }

    @Override
    public synchronized String setTagInUse(String tag, String creationTime) {
        // set the startTime to now
        Timestamp startTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;

        try {
            m_validator.validateString(tag);
            Timestamp creationTimestamp = m_validator.getTimeStamp(creationTime);
            connection = m_mdb.getConnect();

            // the user data must point to an existing data entry
            if (newEntry(connection, creationTimestamp, tag)) {
                return "<error>AlarmHandler with tag = " + tag + " and creation time = " + creationTime
                        + " does not exist.</error>";
            }
            // insert the data
            String insert = "INSERT into " + AlarmHandlerConstants.TABLE_ALARM_HISTORY + " ("
                    + AlarmHandlerConstants.COLUMN_START_TIME + ", " + AlarmHandlerConstants.COLUMN_ALARM_CREATIONTIME
                    + ", " + AlarmHandlerConstants.COLUMN_ALARM_TAG + ") VALUES ('" + startTime + "', '"
                    + creationTimestamp + "', '" + tag + "');";
            m_mdb.batchInsert(connection, insert);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        m_log.info("Tag " + tag + " set in use at " + startTime);
        return "<ok>Tag " + tag + " set in use at " + startTime + "</ok>";
    }

    /**
     * Check to see if the primary key of creationTime, tag exists in the alarm
     * table.
     * 
     * @param connection
     *            db connection
     * @param creationTime
     *            a timestamp containing the creation time of the AlarmHandler
     * @param tag
     *            the name of the tag
     * @return <code>true</code> if primary key does not exist
     * @throws CDBException
     */
    private boolean newEntry(Connection connection, Timestamp creationTime, String tag) throws CDBException {
        // There will only be one row returned
        String select = "SELECT count(*) FROM " + AlarmHandlerConstants.TABLE_ALARM + " WHERE "
                + AlarmHandlerConstants.COLUMN_CREATIONTIME + "='" + creationTime + "' and "
                + AlarmHandlerConstants.COLUMN_TAG + "='" + tag + "';";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            rs.next();
            if (rs.getInt("count") > 0) {
                return false;
            } else {
                return true;
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

}
