package mice.cdb.service.beamline;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>Beamline</code> interface is used to retrieve data about beam line
 * settings. Beam line data is stored on a per run basis. To store beam line
 * data use the {@link BeamlineSuperMouse} interface. All available data can be
 * retrieved using {@link #getAllBeamlines() getAllBeamlines}. A number of
 * methods are available to retrieve subsets of the data,
 * {@link #getBeamlineForRun(String) getBeamlineForRun},
 * {@link #getBeamlinesForDates(String, String) getBeamlinesForDates} and
 * {@link #getBeamlinesForPulses(String, String) getBeamlinesForPulses}.
 * </p>
 * <p>
 * In addition there is the concept of tagged data. It is possible to tag a set
 * of parameters using the {@link BeamlineSuperMouse} interface. Tagged
 * parameters can then be retrieved using {@link #getBeamlineForTag(String)
 * getBeamlineForTag}.
 * </p>
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 * 
 */
@WebService
public interface Beamline {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the beam line data for all runs.
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;runs&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;run runNumber='nnn'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;notes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endNotes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;optics='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseStart='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseEnd='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;step='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;status='true'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;runType='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqTrigger='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqGateWidth='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqVersion='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;gdcHostName='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/run&gt;</code></dd>
     *         <dd><code>&lt;/runs&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getAllBeamlines();

    /**
     * Get the beam line data for the given run.
     * 
     * @param runNo
     *            the run number
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;runs&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;run runNumber='nnn'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;notes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endNotes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;optics='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseStart='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseEnd='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;step='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;status='true'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;runType='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqTrigger='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqGateWidth='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqVersion='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;gdcHostName='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/run&gt;</code></dd>
     *         <dd><code>&lt;/runs&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getBeamlineForRun(String runNo);

    /**
     * Get the beam line data for the given time period.
     * 
     * @param startTime
     *            : a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in
     *            UTC
     * @param stopTime
     *            : a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in
     *            UTC
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;runs&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;run runNumber='nnn'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;notes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endNotes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;optics='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseStart='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseEnd='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;step='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;status='true'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;runType='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqTrigger='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqGateWidth='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqVersion='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;gdcHostName='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/run&gt;</code></dd>
     *         <dd><code>&lt;/runs&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getBeamlinesForDates(String startTime, String stopTime);

    /**
     * Get the beam line data for the given pulse range.
     * 
     * @param startPulse
     *            : pulse number
     * @param endPulse
     *            : pulse number
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;runs&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;run runNumber='nnn'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;notes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endNotes='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;optics='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseStart='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;pulseEnd='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;step='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;status='true'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;runType='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqTrigger='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqGateWidth='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqVersion='n.n'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;gdcHostName='xxx'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/run&gt;</code></dd>
     *         <dd><code>&lt;/runs&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getBeamlinesForPulses(String startPulse, String endPulse);

    /**
     * Get the beam line data for the given tag.
     * 
     * @param tag
     *            : tag name
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;tags&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tag name='xxx'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *         </dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/tag&gt;</code></dd>
     *         <dd><code>&lt;/tags&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getBeamlineForTag(String tag);

    /**
     * List the available tags
     * 
     * @return a string containing XML representing the beam lines with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;tagNames&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tag name='xxx'/&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tag name='xxx'/&gt;</code></dd>
     *         <dd><code>&lt;/tagNames&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String listTags();

    /**
     * Get the run numbers and start and end times.
     * 
     * @return a string containing XML representing the run numbers with the
     *         format of the XML as
     *         <dl>
     *         <dd><code>&lt;runs&gt;</code></dd>
     *         <dd><code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;run runNumber='nnn'</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]/run&gt;'</code>
     *         </dd>
     *         <dd><code>&lt;/runs&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getRunNumbers();

}
