package mice.cdb.service.beamline;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Beamline</code>.
 * 
 * @see mice.cdb.service.beamline.Beamline
 */
@WebService(endpointInterface = "mice.cdb.service.beamline.Beamline")
public class BeamlineImpl extends CDB implements Beamline {

    /**
     * Construct a Beamline.
     */
    public BeamlineImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public String getRunNumbers() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getRunNumbers()");
        }
        String xml = null;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            xml = selectRunNumbers(connection);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getAllBeamlines() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getAllBeamlines()");
        }
        String xml = null;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            xml = selectBeamlines(connection, "");
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getBeamlineForRun(String runNo) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBeamlineForRun(" + runNo + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_RUNNUMBER)
                    .append(" = ").append(runNumber);
            xml = selectBeamlines(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getBeamlinesForDates(String startDate, String stopDate) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBeamlinesForDates(" + startDate + ", " + stopDate + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            Timestamp start = m_validator.getTimeStamp(startDate);
            Timestamp stop = m_validator.getTimeStamp(stopDate);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append("((").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_STARTTIME).append(" <= '").append(start).append("' AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_ENDTIME)
                    .append(" >= '").append(start).append("') OR (").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_STARTTIME).append(" <= '").append(stop).append("' AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_ENDTIME)
                    .append(" >= '").append(stop).append("') OR (").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_STARTTIME).append(" >= '").append(start).append("' AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_ENDTIME)
                    .append(" <= '").append(stop).append("'))");
            xml = selectBeamlines(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getBeamlinesForPulses(String startPulse, String endPulse) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBeamlinesForPulses(" + startPulse + ", " + endPulse + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            int start = m_validator.getInt(startPulse);
            int end = m_validator.getInt(endPulse);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append("((").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_PULSE_START).append(" <= ").append(start).append(" AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_PULSE_END)
                    .append(" >= ").append(start).append(") OR (").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_PULSE_START).append(" <= ").append(end).append(" AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_PULSE_END)
                    .append(" >= ").append(end).append(") OR (").append(BeamlineConstants.TABLE_RUN).append(".")
                    .append(BeamlineConstants.COLUMN_PULSE_START).append(" >= ").append(start).append(" AND ")
                    .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_PULSE_END)
                    .append(" <= ").append(end).append("))");
            xml = selectBeamlines(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getBeamlineForTag(String tag) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBeamlineForTag(" + tag + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateNonBlank(tag);
            connection = m_mdb.getConnect();
            xml = selectTaggedBeamline(connection, tag);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String listTags() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("listTags()");
        }
        String xml = null;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            xml = selectTags(connection);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    /**
     * Select the run numers and associated dates.
     * 
     * @param connection
     *            db connection
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectRunNumbers(Connection connection) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectRunNumbers()");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ")
                .append(BeamlineConstants.COLUMN_STARTTIME).append(", ").append(BeamlineConstants.COLUMN_ENDTIME);
        select.append(" FROM ").append(BeamlineConstants.TABLE_RUN);
        select.append(" ORDER BY ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + BeamlineConstants.RUNS + ">");

        // The schema is evolving so columns may have been added after the first
        // data was taken. rs.getInt, rs.getDouble etc return 0 if the column is
        // empty, this is not what we want.
        try {
            while (rs.next()) {
                xml.append("<").append(BeamlineConstants.RUNNUMBER).append(" ");
                xml.append(BeamlineConstants.RUNNUMBER).append("='")
                        .append(rs.getLong(BeamlineConstants.COLUMN_RUNNUMBER)).append("' ");
                xml.append(BeamlineConstants.STARTTIME).append("='")
                        .append(rs.getTimestamp(BeamlineConstants.COLUMN_STARTTIME)).append("' ");
                xml.append(BeamlineConstants.ENDTIME).append("='")
                        .append(rs.getTimestamp(BeamlineConstants.COLUMN_ENDTIME)).append("' ");
                xml.append("/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        xml.append("</").append(BeamlineConstants.RUNS).append(">");

        return xml.toString();
    }

    /**
     * Select the values for all beamlines matching the clause.
     * 
     * @param connection
     *            db connection
     * @param clause
     *            a SQL where clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectBeamlines(Connection connection, String clause) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectBeamlines(" + clause + ")");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(BeamlineConstants.TABLE_RUN).append(".")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.TABLE_RUN)
                .append(".").append(BeamlineConstants.COLUMN_STARTTIME).append(", ")
                .append(BeamlineConstants.TABLE_RUN).append(".").append(BeamlineConstants.COLUMN_ENDTIME).append(", ")
                .append(BeamlineConstants.COLUMN_START_NOTES).append(", ").append(BeamlineConstants.COLUMN_END_NOTES)
                .append(", ").append(BeamlineConstants.COLUMN_OPTICS).append(", ")
                .append(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS).append(", ")
                .append(BeamlineConstants.COLUMN_PULSE_START).append(", ").append(BeamlineConstants.COLUMN_PULSE_END)
                .append(", ").append(BeamlineConstants.COLUMN_STEP).append(", ")
                .append(BeamlineConstants.COLUMN_STATUS).append(", ").append(BeamlineConstants.COLUMN_RUN_TYPE)
                .append(", ").append(BeamlineConstants.COLUMN_TRIGGER).append(", ")
                .append(BeamlineConstants.COLUMN_GATE_WIDTH).append(", ").append(BeamlineConstants.COLUMN_VERSION)
                .append(", ").append(BeamlineConstants.COLUMN_BEAM_STOP).append(", ")
                .append(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS).append(", ")
                .append(BeamlineConstants.COLUMN_GDC_HOST_NAME).append(", ").append(BeamlineConstants.COLUMN_DELAY)
                .append(", ").append(BeamlineConstants.COLUMN_DEPTH).append(", ")
                .append(BeamlineConstants.COLUMN_DRIVE_VOLTAGE);
        select.append(" FROM ").append(BeamlineConstants.TABLE_RUN);
        select.append(" LEFT JOIN ").append(BeamlineConstants.TABLE_DAQ);
        select.append(" ON ").append(BeamlineConstants.TABLE_RUN).append(".")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(BeamlineConstants.TABLE_DAQ)
                .append(".").append(BeamlineConstants.COLUMN_RUNNUMBER);
        select.append(" LEFT JOIN ").append(BeamlineConstants.TABLE_BEAMLINE);
        select.append(" ON ").append(BeamlineConstants.TABLE_RUN).append(".")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(BeamlineConstants.TABLE_BEAMLINE)
                .append(".").append(BeamlineConstants.COLUMN_RUNNUMBER);
        select.append(" LEFT JOIN ").append(BeamlineConstants.TABLE_TARGET);
        select.append(" ON ").append(BeamlineConstants.TABLE_RUN).append(".")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(BeamlineConstants.TABLE_TARGET)
                .append(".").append(BeamlineConstants.COLUMN_RUNNUMBER);
        if (!clause.isEmpty()) {
            select.append(" WHERE ");
            select.append(clause);
        }
        select.append(" ORDER BY ").append(BeamlineConstants.TABLE_RUN).append(".")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + BeamlineConstants.RUNS + ">");

        // The schema is evolving so columns may have been added after the first
        // data was taken. rs.getInt, rs.getDouble etc return 0 if the column is
        // empty, this is not what we want.
        try {
            while (rs.next()) {
                xml.append("<").append(BeamlineConstants.RUN).append(" ");
                xml.append(BeamlineConstants.RUNNUMBER).append("='")
                        .append(rs.getLong(BeamlineConstants.COLUMN_RUNNUMBER)).append("' ");
                xml.append(BeamlineConstants.STARTTIME).append("='")
                        .append(rs.getTimestamp(BeamlineConstants.COLUMN_STARTTIME)).append("' ");
                xml.append(BeamlineConstants.ENDTIME).append("='")
                        .append(rs.getTimestamp(BeamlineConstants.COLUMN_ENDTIME)).append("' ");
                xml.append(BeamlineConstants.START_NOTES).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_START_NOTES)).append("' ");
                xml.append(BeamlineConstants.END_NOTES).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_END_NOTES)).append("' ");
                xml.append(BeamlineConstants.OPTICS).append("='").append(rs.getString(BeamlineConstants.COLUMN_OPTICS))
                        .append("' ");
                xml.append(BeamlineConstants.PROTON_ABSORBOR_THICKNESS).append("='")
                        .append(getInt(rs.getString(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS))).append("' ");
                xml.append(BeamlineConstants.START_PULSE).append("='")
                        .append(getLong(rs.getString(BeamlineConstants.COLUMN_PULSE_START))).append("' ");
                xml.append(BeamlineConstants.END_PULSE).append("='")
                        .append(getLong(rs.getString(BeamlineConstants.COLUMN_PULSE_END))).append("' ");
                xml.append(BeamlineConstants.STEP).append("='")
                        .append(getDouble(rs.getString(BeamlineConstants.COLUMN_STEP))).append("' ");
                xml.append(BeamlineConstants.STATUS).append("='")
                        .append(rs.getBoolean(BeamlineConstants.COLUMN_STATUS)).append("' ");
                xml.append(BeamlineConstants.RUN_TYPE).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_RUN_TYPE)).append("' ");
                xml.append(BeamlineConstants.TRIGGER).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_TRIGGER)).append("' ");
                xml.append(BeamlineConstants.GATE_WIDTH).append("='")
                        .append(getDouble(rs.getString(BeamlineConstants.COLUMN_GATE_WIDTH))).append("' ");
                xml.append(BeamlineConstants.VERSION).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_VERSION)).append("' ");
                xml.append(BeamlineConstants.BEAM_STOP).append("='")
                        .append(rs.getBoolean(BeamlineConstants.COLUMN_BEAM_STOP)).append("' ");
                xml.append(BeamlineConstants.DIFFUSER_THICKNESS).append("='")
                        .append(getInt(rs.getString(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS))).append("' ");
                xml.append(BeamlineConstants.GDC_HOST_NAME).append("='")
                        .append(rs.getString(BeamlineConstants.COLUMN_GDC_HOST_NAME)).append("' ");
                xml.append(BeamlineConstants.DELAY).append("='")
                        .append(getDouble(rs.getString(BeamlineConstants.COLUMN_DELAY))).append("' ");
                xml.append(BeamlineConstants.DEPTH).append("='")
                        .append(getDouble(rs.getString(BeamlineConstants.COLUMN_DEPTH))).append("' ");
                xml.append(BeamlineConstants.DRIVE_VOLTAGE).append("='")
                        .append(getDouble(rs.getString(BeamlineConstants.COLUMN_DRIVE_VOLTAGE))).append("'");
                xml.append(">");

                select = new StringBuilder();
                select.append("SELECT ").append(BeamlineConstants.COLUMN_NAME).append(" FROM ")
                        .append(BeamlineConstants.TABLE_LDC_HOSTS).append(" WHERE ")
                        .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                        .append(rs.getString(BeamlineConstants.COLUMN_RUNNUMBER)).append(";");
                ResultSet ldcRs = m_mdb.query(connection, select.toString());
                while (ldcRs.next()) {
                    xml.append("<").append(BeamlineConstants.LDC_HOST).append(" ");
                    xml.append(BeamlineConstants.NAME).append("='")
                            .append(ldcRs.getString(BeamlineConstants.COLUMN_NAME)).append("'/>");
                }

                select = new StringBuilder();
                select.append("SELECT ").append(BeamlineConstants.COLUMN_NAME).append(", ")
                        .append(BeamlineConstants.COLUMN_SET_CURRENT).append(", ")
                        .append(BeamlineConstants.COLUMN_POLARITY).append(" FROM ")
                        .append(BeamlineConstants.TABLE_MAGNET).append(" WHERE ")
                        .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                        .append(rs.getString(BeamlineConstants.COLUMN_RUNNUMBER)).append(";");
                ResultSet magnetRs = m_mdb.query(connection, select.toString());
                while (magnetRs.next()) {
                    xml.append("<").append(BeamlineConstants.MAGNET).append(" ");
                    xml.append(BeamlineConstants.NAME).append("='")
                            .append(magnetRs.getString(BeamlineConstants.COLUMN_NAME)).append("' ");
                    xml.append(BeamlineConstants.SET_CURRENT).append("='")
                            .append(getDouble(magnetRs.getString(BeamlineConstants.COLUMN_SET_CURRENT))).append("' ");
                    xml.append(BeamlineConstants.POLARITY).append("='")
                            .append(getInt(magnetRs.getString(BeamlineConstants.COLUMN_POLARITY))).append("'");
                    xml.append("/>");
                }

                select = new StringBuilder();
                select.append("SELECT ").append(BeamlineConstants.COLUMN_NAME).append(", ")
                        .append(BeamlineConstants.COLUMN_VALUE).append(" FROM ").append(BeamlineConstants.TABLE_SCALAR)
                        .append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                        .append(rs.getString(BeamlineConstants.COLUMN_RUNNUMBER)).append(";");
                ResultSet scalarRs = m_mdb.query(connection, select.toString());
                while (scalarRs.next()) {
                    xml.append("<").append(BeamlineConstants.SCALAR).append(" ");
                    xml.append(BeamlineConstants.SCALAR_NAME).append("='")
                            .append(scalarRs.getString(BeamlineConstants.COLUMN_NAME)).append("' ");
                    xml.append(BeamlineConstants.SCALAR_VALUE).append("='")
                            .append(getInt(scalarRs.getString(BeamlineConstants.COLUMN_VALUE))).append("'");
                    xml.append("/>");
                }

                select = new StringBuilder();
                select.append("SELECT ").append(BeamlineConstants.COLUMN_NAME).append(", ")
                        .append(BeamlineConstants.COLUMN_MEAN).append(", ").append(BeamlineConstants.COLUMN_SIGMA)
                        .append(" FROM ").append(BeamlineConstants.TABLE_ISIS_BEAM).append(" WHERE ")
                        .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                        .append(rs.getString(BeamlineConstants.COLUMN_RUNNUMBER)).append(";");
                ResultSet isisBeamlineRs = m_mdb.query(connection, select.toString());
                while (isisBeamlineRs.next()) {
                    xml.append("<").append(BeamlineConstants.ISIS_BEAM).append(" ");
                    xml.append(BeamlineConstants.ISIS_BEAM_NAME).append("='")
                            .append(isisBeamlineRs.getString(BeamlineConstants.COLUMN_NAME)).append("' ");
                    xml.append(BeamlineConstants.MEAN).append("='")
                            .append(isisBeamlineRs.getDouble(BeamlineConstants.COLUMN_MEAN)).append("' ");
                    xml.append(BeamlineConstants.SIGMA).append("='")
                            .append(isisBeamlineRs.getDouble(BeamlineConstants.COLUMN_SIGMA)).append("'");
                    xml.append("/>");
                }

                xml.append("</").append(BeamlineConstants.RUN).append(">");

            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        xml.append("</").append(BeamlineConstants.RUNS).append(">");

        return xml.toString();
    }

    /**
     * Select the values for the beamline matching the tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of a tag
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTaggedBeamline(Connection connection, String tag) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectTaggedBeamline(" + tag + ")");
        }
        int id = getId(connection, tag);
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS).append(", ")
                .append(BeamlineConstants.COLUMN_BEAM_STOP).append(", ")
                .append(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS);
        select.append(" FROM ").append(BeamlineConstants.TABLE_BEAMLINE_TAGGED);
        select.append(" WHERE ").append(BeamlineConstants.COLUMN_ID).append(" = ").append(id).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + BeamlineConstants.TAGS + ">");
        try {
            // There will only be one row returned
            while (rs.next()) {
                xml.append("<" + BeamlineConstants.TAG + " " + BeamlineConstants.COLUMN_NAME + "='" + tag + "' ");
                xml.append(BeamlineConstants.BEAM_STOP).append("='")
                        .append(rs.getBoolean(BeamlineConstants.COLUMN_BEAM_STOP)).append("' ");
                xml.append(BeamlineConstants.DIFFUSER_THICKNESS).append("='")
                        .append(rs.getInt(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS)).append("' ");
                xml.append(BeamlineConstants.PROTON_ABSORBOR_THICKNESS).append("='")
                        .append(rs.getInt(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS)).append("'");
                xml.append(">");
            }
            select = new StringBuilder();
            select.append("SELECT ").append(BeamlineConstants.COLUMN_NAME).append(", ")
                    .append(BeamlineConstants.COLUMN_SET_CURRENT);
            select.append(" FROM ").append(BeamlineConstants.TABLE_MAGNET_TAGGED);
            select.append(" WHERE ").append(BeamlineConstants.COLUMN_ID).append(" = ").append(id).append(";");
            ResultSet magnetRs = m_mdb.query(connection, select.toString());
            while (magnetRs.next()) {
                xml.append("<").append(BeamlineConstants.MAGNET).append(" ");
                xml.append(BeamlineConstants.NAME).append("='")
                        .append(magnetRs.getString(BeamlineConstants.COLUMN_NAME)).append("' ");
                xml.append(BeamlineConstants.SET_CURRENT).append("='")
                        .append(magnetRs.getDouble(BeamlineConstants.COLUMN_SET_CURRENT)).append("'");
                xml.append("/>");
            }
            xml.append("</").append(BeamlineConstants.TAG).append(">");
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        xml.append("</").append(BeamlineConstants.TAGS).append(">");

        return xml.toString();
    }

    /**
     * Select the list of available tag names.
     * 
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTags(Connection connection) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectTags()");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT DISTINCT ").append(BeamlineConstants.COLUMN_TAG).append(" FROM ")
                .append(BeamlineConstants.TABLE_BEAMLINE_TAG).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + BeamlineConstants.TAG_NAMES + ">");
        try {
            while (rs.next()) {

                xml.append("<").append(BeamlineConstants.TAG).append(" ");
                xml.append(BeamlineConstants.NAME).append("='").append(rs.getString(BeamlineConstants.COLUMN_TAG))
                        .append("' ");
                xml.append("/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml.append("</" + BeamlineConstants.TAG_NAMES + ">");
        return xml.toString();
    }

    /**
     * Get the id of the latest version of the given tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of a tag
     * @return id of the tag
     * @throws CDBException
     */
    protected int getId(Connection connection, String tag) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getId(connection" + tag + ")");
        }
        // There will only be one row returned
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(BeamlineConstants.COLUMN_ID).append(" FROM ")
                .append(BeamlineConstants.TABLE_BEAMLINE_TAG).append(" WHERE ").append(BeamlineConstants.COLUMN_TAG)
                .append(" = '").append(tag).append("' AND ").append(BeamlineConstants.COLUMN_VALID_UNTIL_TIME)
                .append(" is Null;");

        ResultSet rs = m_mdb.query(connection, select.toString());
        int id = 0;
        try {
            while (rs.next()) {
                id = rs.getInt(BeamlineConstants.COLUMN_ID);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        if (id == 0) {
            throw new CDBException("Tag " + tag + " not found.");
        }
        return id;
    }

    /**
     * Check value of string is a double.
     * 
     * @param s
     *            the string to check
     * @return a string containing the double or null
     */
    private String getDouble(String s) {
        try {
            return String.valueOf(Double.parseDouble(s));
        } catch (NumberFormatException e) {
            return "null";
        } catch (NullPointerException e) {
            return "null";
        }
    }

    /**
     * Check value of string is an int.
     * 
     * @param s
     *            the string to check
     * @return a string containing the int or null
     */
    private String getInt(String s) {
        try {
            return String.valueOf(Integer.parseInt(s));
        } catch (NumberFormatException e) {
            return "null";
        } catch (NullPointerException e) {
            return "null";
        }
    }

    /**
     * Check value of string is a long.
     * 
     * @param s
     *            the string to check
     * @return a string containing the long or null
     */
    private String getLong(String s) {
        try {
            return String.valueOf(Long.parseLong(s));
        } catch (NumberFormatException e) {
            return "null";
        } catch (NullPointerException e) {
            return "null";
        }
    }

}
