package mice.cdb.service.beamline;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>BeamlineSuperMouse</code> interface is used to store and retrieve
 * data about beam line settings. Prior to a run commencing the
 * {@link #setStartRun(String) setStartRun} and {@link #setBeamline(String)
 * setBeamline} methods should be called to record the initial parameters. At
 * the end of the run {@link #setEndRun(String) setEndRun} should be called to
 * record the final parameters. All available data can be retrieved using
 * {@link #getAllBeamlines() getAllBeamlines}. A number of methods are available
 * to retrieve subsets of the data, {@link #getBeamlineForRun(String)
 * getBeamlineForRun}, {@link #getBeamlinesForDates(String, String)
 * getBeamlinesForDates} and {@link #getBeamlinesForPulses(String, String)
 * getBeamlinesForPulses}
 * </p>
 * <p>
 * In addition there is the concept of tagged data. It is possible to tag a set
 * of parameters using the {@link #setBeamlineTag(String) setBeamlineTag}
 * method. Tagged parameters can then be retrieved using
 * {@link #getBeamlineForTag(String) getBeamlineForTag}.The use of
 * setTaggedBeamline with an existing tag name will replace the existing
 * parameters, although the original parameters will still be stored in the
 * database.
 * </p>
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * The returned XML will contain a message enclosed in one of three tags: ok,
 * warning or error. A tag of warning indicates that the problem maybe transient
 * and retrying the request MAY succeed. A tag of error maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 * 
 */
@WebService
public interface BeamlineSuperMouse extends Beamline {

    /**
     * Add the initial parameters associated with a run.
     * 
     * @param xml
     *            a string containing XML representing the beam line with the
     *            format of the XML as
     *            <dl>
     *            <dd><code>&lt;run runNumber='nnn'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;notes='xxx'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;optics='xxx'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;startTargetPulse='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;step='n.n'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;runType='xxx'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqTrigger='xxx'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqGateWidth='n.n'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;daqVersion='n.n'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;gdcHostName='xxx'/&gt;</code>
     *            </dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setStartRun(String xml);

    /**
     * Add the parameters associated with the end of a run.
     * 
     * @param xml
     *            a string containing XML representing the beam line with the
     *            format of the XML as
     *            <dl>
     *            <dd><code>&lt;endRun runNumber='nnn'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endTime='yyyy-mm-dd hh:mm:ss[.fffffffff]'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endPulse='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;status='true'&gt;</code>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;scalar name='xxx' value='nnn'/&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/endRun&gt;</code></dd>
     *            </dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setEndRun(String xml);

    /**
     * Add the parameters associated with the beam line.
     * 
     * @param xml
     *            a string containing XML representing the beam line with the
     *            format of the XML as
     *            <dl>
     *            <dd><code>&lt;beamline runNumber='nnn'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn';</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;endNotes='xxx'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='nnn'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='nnn'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n' polarity='nnn'/&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/beamline&gt;</code></dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setBeamline(String xml);

    /**
     * Set the parameters associated with a beam line tag.
     * 
     * @param xml
     *            a string containing XML representing the beam line with the
     *            format of the XML as
     *            <dl>
     *            <dd><code>&lt;tag name='xxx'</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;beamStop='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;diffuserThickness='nnn'</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;protonAbsorberThickness='nnn'&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;magnet name='xxx' setCurrent='n.n'/&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/tag&gt;</code></dd>
     *            </dl>
     */
    @WebMethod
    public abstract String setBeamlineTag(String xml);

}
