package mice.cdb.service.beamline;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.parser.BeamlineParser;
import mice.cdb.service.parser.ParseException;

/**
 * An implementation of the <code>BeamlineSuperMouse</code>.
 * 
 * @see mice.cdb.service.beamline.BeamlineSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.beamline.BeamlineSuperMouse")
public class BeamlineSuperMouseImpl extends BeamlineImpl implements BeamlineSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Constructs a Beamline object with super mouse powers.
     */
    public BeamlineSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String setStartRun(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setStartRun(" + xml + ")");
        }
        Connection connection = null;
        BeamlineData blData = null;

        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            BeamlineParser blParser = new BeamlineParser();
            try {
                blParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Beamline XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            blData = blParser.getData();
            if (m_log.isInfoEnabled() && blData.getOverwrite()) {
                m_log.info("overwrite flag set for setStartRun");
            }
            StringBuilder insert = new StringBuilder();

            insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_RUN).append(" (")
                    .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_STARTTIME)
                    .append(", ").append(BeamlineConstants.COLUMN_START_NOTES).append(", ")
                    .append(BeamlineConstants.COLUMN_OPTICS).append(", ").append(BeamlineConstants.COLUMN_PULSE_START)
                    .append(", ").append(BeamlineConstants.COLUMN_STEP).append(", ")
                    .append(BeamlineConstants.COLUMN_RUN_TYPE).append(", ")
                    .append(BeamlineConstants.COLUMN_GDC_HOST_NAME).append(") VALUES (").append(blData.getRunNumber())
                    .append(", '").append(blData.getStartTime()).append("', '").append(blData.getNotes())
                    .append("', '").append(blData.getOptics()).append("', ").append(blData.getPulseStart())
                    .append(", ").append(blData.getStep()).append(", '").append(blData.getRunType()).append("', '")
                    .append(blData.getGdcHostName()).append("'); ");

            insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_DAQ).append(" (")
                    .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_TRIGGER)
                    .append(", ").append(BeamlineConstants.COLUMN_GATE_WIDTH).append(", ")
                    .append(BeamlineConstants.COLUMN_VERSION).append(") VALUES (").append(blData.getRunNumber())
                    .append(", '").append(blData.getDaqTrigger()).append("', ").append(blData.getDaqGateWidth())
                    .append(", '").append(blData.getDaqVersion()).append("'); ");

            insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_TARGET).append(" (")
                    .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_DEPTH)
                    .append(", ").append(BeamlineConstants.COLUMN_DELAY).append(", ")
                    .append(BeamlineConstants.COLUMN_DRIVE_VOLTAGE).append(") VALUES (").append(blData.getRunNumber())
                    .append(", ").append(blData.getTargetDepth()).append(", ").append(blData.getTargetDelay())
                    .append(", ").append(blData.getTargetDriveVoltage()).append("); ");
            for (String host : blData.getLcdHosts()) {
                insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_LDC_HOSTS).append(" (");
                insert.append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_NAME);
                insert.append(") VALUES (").append(blData.getRunNumber()).append(", '").append(host).append("'); ");
            }
            m_mdb.batchInsert(connection, insert.toString());

        } catch (CDBException e) {
            if (blData.getOverwrite()) {
                try {
                    m_mdb.batchInsert(connection, setStartRunOverwrite(blData));
                    if (m_log.isInfoEnabled()) {
                        m_log.info("startRun data overwritten");
                    }
                } catch (CDBException e1) {
                    if (m_log.isDebugEnabled()) {
                        m_log.debug(e.getMessage());
                    }
                    return "<error>" + e.getMessage() + "</error>";
                }
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug(e.getMessage());
                }
                return "<error>" + e.getMessage() + "</error>";
            }
        } finally {
            m_mdb.close(connection);
        }

        return "<ok>Run data added for run " + blData.getRunNumber() + "</ok>";
    }

    @Override
    public synchronized String setEndRun(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setEndRun(" + xml + ")");
        }
        Connection connection = null;
        BeamlineData blData = null;
        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            BeamlineParser blParser = new BeamlineParser();
            try {
                blParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Beamline XML. Please check input. " + e.getMessage() + "</error>";
            }

            // update the data
            blData = blParser.getData();
            if (m_log.isInfoEnabled() && blData.getOverwrite()) {
                m_log.info("overwrite flag set for setEndRun");
            }
            if (!updatable(connection, blData.getRunNumber()) && !blData.getOverwrite()) {
                return "<error>Entry for run " + blData.getRunNumber()
                        + " does not exist or already has the end data set</error>";
            }
            StringBuilder update = new StringBuilder();

            update.append("UPDATE ").append(BeamlineConstants.TABLE_RUN).append(" SET ");
            update.append(BeamlineConstants.COLUMN_ENDTIME).append(" = '").append(blData.getEndTime()).append("', ");
            update.append(BeamlineConstants.COLUMN_PULSE_END).append(" = '").append(blData.getPulseEnd()).append("', ");
            update.append(BeamlineConstants.COLUMN_END_NOTES).append(" = '").append(blData.getEndNotes()).append("', ");
            update.append(BeamlineConstants.COLUMN_STATUS).append(" = '").append(blData.getStatus()).append("'");
            update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                    .append(blData.getRunNumber()).append(";");
            m_mdb.insert(connection, update.toString());

            try {
                StringBuilder insert = new StringBuilder();
                Map<String, Integer> scalarData = blData.getScalar();
                Set<String> keys = scalarData.keySet();
                for (String key : keys) {
                    insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_SCALAR).append("(");
                    insert.append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ");
                    insert.append(BeamlineConstants.COLUMN_NAME).append(", ");
                    insert.append(BeamlineConstants.COLUMN_VALUE).append(")");
                    insert.append(" VALUES (").append(blData.getRunNumber()).append(", '");
                    insert.append(key).append("', ");
                    insert.append(scalarData.get(key)).append(");");
                }
                Map<String, IsisData> isisBeamData = blData.getIsisBeam();
                keys = isisBeamData.keySet();
                for (String key : keys) {
                    insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_ISIS_BEAM).append("(");
                    insert.append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ");
                    insert.append(BeamlineConstants.COLUMN_NAME).append(", ");
                    insert.append(BeamlineConstants.COLUMN_MEAN).append(", ");
                    insert.append(BeamlineConstants.COLUMN_SIGMA).append(")");
                    insert.append(" VALUES (").append(blData.getRunNumber()).append(", '");
                    insert.append(key).append("', ");
                    insert.append(isisBeamData.get(key).getMean()).append(", ");
                    insert.append(isisBeamData.get(key).getSigma()).append(");");
                }
                m_mdb.insert(connection, insert.toString());
            } catch (CDBException e) {
                if (blData.getOverwrite()) {
                    try {
                        m_mdb.batchInsert(connection, setEndRunOverwrite(blData));
                        if (m_log.isInfoEnabled()) {
                            m_log.info("endRun data overwritten");
                        }
                    } catch (CDBException e1) {
                        if (m_log.isDebugEnabled()) {
                            m_log.debug(e.getMessage());
                        }
                        return "<error>" + e.getMessage() + "</error>";
                    }
                } else {
                    if (m_log.isDebugEnabled()) {
                        m_log.debug(e.getMessage());
                    }
                    return "<error>" + e.getMessage() + "</error>";
                }
            }
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        if (m_log.isInfoEnabled() && blData.getOverwrite()) {
            m_log.info("endRun data overwritten");
        }
        return "<ok>Run data updated for run " + blData.getRunNumber() + "</ok>";
    }

    @Override
    public synchronized String setBeamline(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setBeamline(" + xml + ")");
        }
        Connection connection = null;
        BeamlineData blData = null;
        List<Map<String, String>> magnets = null;
        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            BeamlineParser blParser = new BeamlineParser();
            try {
                blParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Beamline XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            blData = blParser.getData();
            if (m_log.isInfoEnabled() && blData.getOverwrite()) {
                m_log.info("overwrite flag set for setBeamline");
            }
            StringBuilder insert = new StringBuilder();

            insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_BEAMLINE).append(" (")
                    .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_BEAM_STOP)
                    .append(", ").append(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS).append(", ")
                    .append(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS).append(") VALUES (")
                    .append(blData.getRunNumber()).append(", ").append(blData.getBeamStop()).append(", ")
                    .append(blData.getDiffuserThickness()).append(", ").append(blData.getProtonAbsorberThickness())
                    .append("); ");

            magnets = blParser.getMagnets();
            for (Map<String, String> magnet : magnets) {
                insert.append("INSERT INTO ").append(BeamlineConstants.TABLE_MAGNET).append(" (")
                        .append(BeamlineConstants.COLUMN_RUNNUMBER).append(", ").append(BeamlineConstants.COLUMN_NAME)
                        .append(", ").append(BeamlineConstants.COLUMN_SET_CURRENT).append(", ")
                        .append(BeamlineConstants.COLUMN_POLARITY).append(") VALUES (").append(blData.getRunNumber())
                        .append(", '").append(magnet.get(BeamlineConstants.NAME)).append("', ")
                        .append(magnet.get(BeamlineConstants.SET_CURRENT)).append(", '")
                        .append(magnet.get(BeamlineConstants.POLARITY)).append("'); ");
            }

            m_mdb.batchInsert(connection, insert.toString());

        } catch (CDBException e) {
            if (blData.getOverwrite()) {
                try {
                    m_mdb.batchInsert(connection, setBeamlineOverwrite(blData, magnets));
                    if (m_log.isInfoEnabled()) {
                        m_log.info("beamline data overwritten");
                    }
                } catch (CDBException e1) {
                    if (m_log.isDebugEnabled()) {
                        m_log.debug(e.getMessage());
                    }
                    return "<error>" + e.getMessage() + "</error>";
                }
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug(e.getMessage());
                }
                return "<error>" + e.getMessage() + "</error>";
            }
        } finally {
            m_mdb.close(connection);
        }

        return "<ok>Beamline data added for run " + blData.getRunNumber() + "</ok>";
    }

    @Override
    public synchronized String setBeamlineTag(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setBeamlineTag(" + xml + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        BeamlineData blData = null;

        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            BeamlineParser blParser = new BeamlineParser();
            try {
                blParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Beamline XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            blData = blParser.getData();
            List<Map<String, String>> magnets = blParser.getMagnets();
            int id = updateBeamlineTag(connection, blData.getTag(), creationTime);
            writeTaggedBeamline(connection, id, blData, magnets);

        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return "<ok>Beamline tag " + blData.getTag() + " set</ok>";
    }

    /**
     * Check if an entry for the given run number exists in the database.
     * 
     * @param connection
     *            db connection
     * @param runNumber
     *            the run number
     * @return <code>true</code> if entry for run number exists
     * @throws CDBException
     */
    private boolean updatable(Connection connection, int runNumber) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("updatable(" + runNumber + ")");
        }
        // There will only be one row returned
        StringBuilder select = new StringBuilder();
        select.append("SELECT COUNT(*) FROM ").append(BeamlineConstants.TABLE_RUN).append(" WHERE ")
                .append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(runNumber).append(" AND ")
                .append(BeamlineConstants.COLUMN_ENDTIME).append(" is Null;");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            rs.next();
            if (rs.getInt("count") == 1) {
                return true;
            } else {
                return false;
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Overwrite the start run data.
     * 
     * @param blData
     * @return a SQL update statement
     */
    private String setStartRunOverwrite(BeamlineData blData) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setStartRunOverwrite(blData)");
        }
        StringBuilder update = new StringBuilder();
        update.append("UPDATE ").append(BeamlineConstants.TABLE_RUN).append(" SET ");
        update.append(BeamlineConstants.COLUMN_STARTTIME).append(" = '").append(blData.getStartTime()).append("', ");
        update.append(BeamlineConstants.COLUMN_START_NOTES).append(" = '").append(blData.getNotes()).append("', ");
        update.append(BeamlineConstants.COLUMN_OPTICS).append(" = '").append(blData.getOptics()).append("', ");
        update.append(BeamlineConstants.COLUMN_PULSE_START).append(" = '").append(blData.getPulseStart()).append("', ");
        update.append(BeamlineConstants.COLUMN_STEP).append(" = '").append(blData.getStep()).append("', ");
        update.append(BeamlineConstants.COLUMN_RUN_TYPE).append(" = '").append(blData.getRunType()).append("', ");
        update.append(BeamlineConstants.COLUMN_GDC_HOST_NAME).append(" = '").append(blData.getGdcHostName());
        update.append("' WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                .append(blData.getRunNumber()).append(";");

        update.append("UPDATE ").append(BeamlineConstants.TABLE_DAQ).append(" SET ");
        update.append(BeamlineConstants.COLUMN_TRIGGER).append(" = '").append(blData.getDaqTrigger()).append("', ");
        update.append(BeamlineConstants.COLUMN_GATE_WIDTH).append(" = '").append(blData.getDaqGateWidth())
                .append("', ");
        update.append(BeamlineConstants.COLUMN_VERSION).append(" = '").append(blData.getDaqVersion()).append("' ");
        update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(blData.getRunNumber())
                .append(";");

        update.append("UPDATE ").append(BeamlineConstants.TABLE_TARGET).append(" SET ");
        update.append(BeamlineConstants.COLUMN_DEPTH).append(" = ").append(blData.getTargetDepth()).append(", ");
        update.append(BeamlineConstants.COLUMN_DELAY).append(" = ").append(blData.getTargetDelay()).append(", ");
        update.append(BeamlineConstants.COLUMN_DRIVE_VOLTAGE).append(" = ").append(blData.getTargetDriveVoltage());
        update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(blData.getRunNumber())
                .append(";");

        return update.toString();
    }

    private String setEndRunOverwrite(BeamlineData blData) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setEndRunOverwrite(blData)");
        }
        Map<String, Integer> scalarData = blData.getScalar();

        StringBuilder update = new StringBuilder();
        Set<String> keys = scalarData.keySet();
        for (String key : keys) {
            update.append("UPDATE ").append(BeamlineConstants.TABLE_SCALAR).append(" SET ");
            update.append(BeamlineConstants.COLUMN_VALUE).append(" = ").append(scalarData.get(key));
            update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                    .append(blData.getRunNumber());
            update.append(" AND ").append(BeamlineConstants.COLUMN_NAME).append(" = '").append(key).append("';");
        }
        return update.toString();
    }

    /**
     * Overwrite the beamline data.
     * 
     * @param blData
     * @param magnets
     * @return a SQL update statement
     */
    private String setBeamlineOverwrite(BeamlineData blData, List<Map<String, String>> magnets) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setBeamlineOverwrite(blData, magnets)");
        }
        StringBuilder update = new StringBuilder();
        update.append("UPDATE ").append(BeamlineConstants.TABLE_BEAMLINE).append(" SET ");
        update.append(BeamlineConstants.COLUMN_BEAM_STOP).append(" = '").append(blData.getBeamStop()).append("', ");
        update.append(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS).append(" = '").append(blData.getDiffuserThickness())
                .append("', ");
        update.append(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS).append(" = '")
                .append(blData.getProtonAbsorberThickness()).append("' ");
        update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ").append(blData.getRunNumber())
                .append(";");

        for (Map<String, String> magnet : magnets) {
            update.append("UPDATE ").append(BeamlineConstants.TABLE_MAGNET).append(" SET ");
            update.append(BeamlineConstants.COLUMN_SET_CURRENT).append(" = '")
                    .append(magnet.get(BeamlineConstants.SET_CURRENT)).append("', ");
            update.append(BeamlineConstants.COLUMN_POLARITY).append(" = ")
                    .append(magnet.get(BeamlineConstants.POLARITY));
            update.append(" WHERE ").append(BeamlineConstants.COLUMN_RUNNUMBER).append(" = ")
                    .append(blData.getRunNumber()).append(" AND ").append(BeamlineConstants.COLUMN_NAME).append(" = '")
                    .append(magnet.get(BeamlineConstants.NAME)).append("';");
        }
        return update.toString();
    }

    /**
     * Get a new id for the given tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of the tag
     * @param creationTime
     *            the creation time of the tag
     * @return the new id
     * @throws CDBException
     */
    private int updateBeamlineTag(Connection connection, String tag, Timestamp creationTime) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("updateBeamlineTag(" + tag + ", " + creationTime + ")");
        }
        StringBuilder sql = new StringBuilder();
        sql.append("UPDATE ").append(BeamlineConstants.TABLE_BEAMLINE_TAG).append(" SET ")
                .append(BeamlineConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime).append("' WHERE ")
                .append(BeamlineConstants.COLUMN_TAG).append("='").append(tag).append("' AND ")
                .append(BeamlineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        try {
            m_mdb.insert(connection, sql.toString());
        } catch (CDBException e) {
            // no tag to update
        }
        sql = new StringBuilder();
        sql.append("INSERT INTO ").append(BeamlineConstants.TABLE_BEAMLINE_TAG).append(" (")
                .append(BeamlineConstants.COLUMN_TAG).append(", ").append(BeamlineConstants.COLUMN_CREATIONTIME)
                .append(") VALUES ('").append(tag).append("', '").append(creationTime).append("'); ");
        m_mdb.insert(connection, sql.toString());
        return getId(connection, tag);
    }

    /**
     * Write the tagged data.
     * 
     * @param connection
     *            db connection
     * @param id
     *            the id of the tag
     * @param blData
     *            the beam line data
     * @param magnets
     *            the magnets data
     * @throws CDBException
     */
    private void writeTaggedBeamline(Connection connection, int id, BeamlineData blData,
            List<Map<String, String>> magnets) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("writeTaggedBeamline(" + id + ")");
        }
        StringBuilder sql = new StringBuilder();
        sql.append("INSERT INTO ").append(BeamlineConstants.TABLE_BEAMLINE_TAGGED).append(" (")
                .append(BeamlineConstants.COLUMN_ID).append(", ").append(BeamlineConstants.COLUMN_BEAM_STOP)
                .append(", ").append(BeamlineConstants.COLUMN_DIFFUSER_THICKNESS).append(", ")
                .append(BeamlineConstants.COLUMN_PROTON_ABSORBOR_THICKNESS).append(") VALUES (").append(id)
                .append(", ").append(blData.getBeamStop()).append(", ").append(blData.getDiffuserThickness())
                .append(", ").append(blData.getProtonAbsorberThickness()).append("); ");
        for (Map<String, String> magnet : magnets) {
            sql.append("INSERT INTO ").append(BeamlineConstants.TABLE_MAGNET_TAGGED).append(" (")
                    .append(BeamlineConstants.COLUMN_ID).append(", ").append(BeamlineConstants.COLUMN_NAME)
                    .append(", ").append(BeamlineConstants.COLUMN_SET_CURRENT).append(") VALUES (").append(id)
                    .append(", '").append(magnet.get(BeamlineConstants.NAME)).append("', ")
                    .append(magnet.get(BeamlineConstants.SET_CURRENT)).append("); ");
        }
        m_mdb.batchInsert(connection, sql.toString());
    }

}
