package mice.cdb.service.cabling;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Cabling</code>.
 * 
 * @see mice.cdb.service.cabling.Cabling
 */
@WebService(endpointInterface = "mice.cdb.service.cabling.Cabling")
public class CablingImpl extends CDB implements Cabling {

    protected List<String> m_detectors = new LinkedList<String>();

    protected List<String> m_trackers = new LinkedList<String>();

    /**
     * Construct a Cabling.
     */
    public CablingImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
        for (String detector : CablingConstants.DETECTORS) {
            m_detectors.add(detector.toUpperCase());
        }
        for (String tracker : CablingConstants.TRACKERS) {
            m_trackers.add(tracker.toUpperCase());
        }
    }

    @Override
    public String getCurrentCabling(String name) {
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateStringNonBlank(name);
            String upperName = name.toUpperCase().trim();

            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();

            clause.append("WHERE ").append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL");
            if (CablingConstants.CONTROL.equals(upperName)) {
                xml = selectCablingControl(connection, clause.toString());
            } else if (m_trackers.contains(upperName)) {
                clause.append(" AND ").append(CablingConstants.COLUMN_DEVICE).append("='").append(upperName)
                        .append("'");
                xml = selectCablingTracker(connection, clause.toString());
            } else {
                m_log.debug("Unknown device:" + name);
                return "<error>" + "Unknown device:" + name + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCablingForDate(String name, String timestamp) {
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateStringNonBlank(name);
            String upperName = name.toUpperCase().trim();
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);

            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append("WHERE ").append(CablingConstants.COLUMN_CREATIONTIME).append("<='").append(usersTimestamp)
                    .append("' AND (").append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(">'")
                    .append(usersTimestamp).append("' OR ").append(CablingConstants.COLUMN_VALID_UNTIL_TIME)
                    .append(" is NULL)");
            if (CablingConstants.CONTROL.equalsIgnoreCase(upperName)) {
                xml = selectCablingControl(connection, clause.toString());
            } else if (m_trackers.contains(upperName)) {
                clause.append(" AND ").append(CablingConstants.COLUMN_DEVICE).append("='").append(upperName)
                        .append("'");
                xml = selectCablingTracker(connection, clause.toString());
            } else {
                m_log.debug("Unknown device:" + name);
                return "<error>" + "Unknown device:" + name + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCablingForRun(String name, String runNo) {
        String xml = null;
        Connection connection = null;

        try {
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runStartTime = selectRunStartTime(connection, runNumber);
            xml = getCablingForDate(name, runStartTime.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String listDevices() {
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(CablingConstants.CABLING).append(">");
        xml.append("<").append(CablingConstants.DEVICE_LIST).append(">");
        xml.append("<").append(CablingConstants.DEVICE).append(" ").append(CablingConstants.NAME).append("='")
                .append(CablingConstants.CONTROL).append("'/>");
        for (String detector : m_detectors) {
            xml.append("<").append(CablingConstants.DEVICE).append(" ").append(CablingConstants.NAME).append("='")
                    .append(detector).append("'/>");
        }
        for (String tracker : m_trackers) {
            xml.append("<").append(CablingConstants.DEVICE).append(" ").append(CablingConstants.NAME).append("='")
                    .append(tracker).append("'/>");
        }
        xml.append("</").append(CablingConstants.DEVICE_LIST).append(">");
        xml.append("</").append(CablingConstants.CABLING).append(">");
        return xml.toString();
    }

    @Override
    public byte[] getDetectorCablingForDate(String device, String timestamp) {
        byte[] data = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            Timestamp time = m_validator.getTimeStamp(timestamp);
            validateDevice(device);
            StringBuilder deviceType = new StringBuilder();
            deviceType.append(" AND ").append(CablingConstants.COLUMN_DEVICE).append("='").append(device).append("'");
            StringBuilder maxValidFromTime = new StringBuilder();
            maxValidFromTime.append("SELECT max(").append(CablingConstants.COLUMN_VALID_FROM_TIME);
            maxValidFromTime.append(") FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR);
            maxValidFromTime.append(" WHERE ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= '")
                    .append(time).append("'").append(deviceType);
            StringBuilder maxCreationTime = new StringBuilder();
            maxCreationTime.append("SELECT max(").append(CablingConstants.COLUMN_CREATIONTIME);
            maxCreationTime.append(") FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR);
            maxCreationTime.append(" WHERE ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" = (")
                    .append(maxValidFromTime).append(")").append(deviceType);
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CablingConstants.COLUMN_CREATIONTIME).append(" = (")
                    .append(maxCreationTime);
            clause.append(") AND ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" = (")
                    .append(maxValidFromTime).append(")").append(deviceType);
            data = selectCablingDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public byte[] getCurrentDetectorCabling(String device) {
        Timestamp now = new Timestamp(System.currentTimeMillis());
        return getDetectorCablingForDate(device, now.toString());
    }

    @Override
    public byte[] getDetectorCablingForId(String id) {
        byte[] data = null;
        Connection connection = null;
        try {
            int numericId = m_validator.getInt(id);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CablingConstants.COLUMN_ID).append(" = ").append(numericId);
            data = selectCablingDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public byte[] getDetectorCablingForRun(String device, String runNo) {
        byte[] data = null;
        Connection connection = null;
        try {
            validateDevice(device);
            m_validator.validateStringNonBlank(runNo);
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runTime = selectRunStartTime(connection, runNumber);
            StringBuilder maxCreationTime = new StringBuilder();
            maxCreationTime.append("SELECT max(").append(CablingConstants.COLUMN_CREATIONTIME);
            maxCreationTime.append(") FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR);
            maxCreationTime.append(" WHERE ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= ('")
                    .append(runTime).append("')");
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CablingConstants.COLUMN_CREATIONTIME).append(" = (")
                    .append(maxCreationTime);
            clause.append(") AND ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= ('").append(runTime)
                    .append("')");
            data = selectCablingDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public String getCablingIds(String startTime, String stopTime) {
        String data = null;
        Connection connection = null;
        try {
            Timestamp start = m_validator.getTimeStamp(startTime);
            connection = m_mdb.getConnect();
            if (stopTime == null) {
                data = selectIds(connection, start);
            } else {
                Timestamp stop = m_validator.getTimeStamp(stopTime);
                if (start.after(stop)) {
                    String message = "<error>Start time must be before stop time.</error>";
                    return message;
                }
                data = selectIds(connection, start, stop);
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    /**
     * Select the cabling for control.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectCablingControl(Connection connection, String clause) throws CDBException {
        m_log.debug("selectCablingControl(" + clause + ")");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CablingConstants.COLUMN_CRATE).append(", ")
                .append(CablingConstants.COLUMN_MODULE).append(", ").append(CablingConstants.COLUMN_CHANNEL)
                .append(", ").append(CablingConstants.COLUMN_NAME).append(" FROM ")
                .append(CablingConstants.TABLE_CABLING_CONTROL).append(" ").append(clause).append(" ORDER BY ")
                .append(CablingConstants.COLUMN_CRATE).append(", ").append(CablingConstants.COLUMN_MODULE).append(", ")
                .append(CablingConstants.COLUMN_CHANNEL).append(";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(CablingConstants.CABLING).append(" ").append(CablingConstants.TYPE).append("='")
                .append(CablingConstants.CONTROL).append("'>");
        String crate = "";
        String module = "";

        try {
            while (rs.next()) {

                if (!rs.getString(CablingConstants.COLUMN_CRATE).equals(crate)) {
                    if (!crate.isEmpty()) {
                        xml.append("</").append(CablingConstants.MODULE).append(">");
                        xml.append("</").append(CablingConstants.CRATE).append(">");
                        module = "";
                    }
                    crate = rs.getString(CablingConstants.COLUMN_CRATE);
                    xml.append("<").append(CablingConstants.CRATE).append(" ").append(CablingConstants.NUMBER)
                            .append("='").append(crate).append("'>");
                }

                if (!rs.getString(CablingConstants.COLUMN_MODULE).equals(module)) {
                    if (!module.isEmpty()) {
                        xml.append("</").append(CablingConstants.MODULE).append(">");
                    }
                    module = rs.getString(CablingConstants.COLUMN_MODULE);
                    xml.append("<").append(CablingConstants.MODULE).append(" ").append(CablingConstants.NUMBER)
                            .append("='").append(module).append("'>");
                }

                xml.append("<").append(CablingConstants.CHANNEL).append(" ").append(CablingConstants.NUMBER)
                        .append("='").append(rs.getString(CablingConstants.COLUMN_CHANNEL)).append("' ")
                        .append(CablingConstants.NAME).append("='").append(rs.getString(CablingConstants.COLUMN_NAME))
                        .append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        if (!crate.isEmpty()) {
            xml.append("</").append(CablingConstants.MODULE).append(">");
            xml.append("</").append(CablingConstants.CRATE).append(">");
        }
        xml.append("</").append(CablingConstants.CABLING).append(">");

        return xml.toString();
    }

    /**
     * Get the cabling data from the database for a detector.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return the data
     * @throws CDBException
     */
    private byte[] selectCablingDetector(Connection connection, StringBuilder clause) throws CDBException {
        m_log.debug("selectCablingDetector(" + clause + ")");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CablingConstants.COLUMN_DATA);
        select.append(" FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR);
        select.append(clause).append(";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getBytes(CablingConstants.COLUMN_DATA);
            } else {
                m_log.debug("No data returned from the database.\n\t" + select);
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select the cabling for a tracker.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectCablingTracker(Connection connection, String clause) throws CDBException {
        m_log.debug("selectCablingTracker(" + clause + ")");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CablingConstants.COLUMN_VLSB_COMPUTER_ID).append(", ")
                .append(CablingConstants.COLUMN_VLSB_GEO_NUMBER).append(", ")
                .append(CablingConstants.COLUMN_VLSB_CHANNEL).append(", ").append(CablingConstants.COLUMN_TRACKER_NO)
                .append(", ").append(CablingConstants.COLUMN_STATION).append(", ")
                .append(CablingConstants.COLUMN_PLANE).append(", ").append(CablingConstants.COLUMN_CHANNEL)
                .append(" FROM ").append(CablingConstants.TABLE_CABLING_TRACKER).append(" ").append(clause)
                .append(" ORDER BY ").append(CablingConstants.COLUMN_VLSB_COMPUTER_ID).append(", ")
                .append(CablingConstants.COLUMN_VLSB_GEO_NUMBER).append(", ")
                .append(CablingConstants.COLUMN_VLSB_CHANNEL).append(";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(CablingConstants.CABLING).append(" ").append(CablingConstants.TYPE).append("='")
                .append(CablingConstants.TRACKER).append("'>");
        String vlsbComputerId = "";
        String vlsbGeoNumber = "";

        try {
            while (rs.next()) {

                if (!rs.getString(CablingConstants.COLUMN_VLSB_COMPUTER_ID).equals(vlsbComputerId)) {
                    if (!vlsbComputerId.isEmpty()) {
                        xml.append("</").append(CablingConstants.VLSB_GEO_NUMBER).append(">");
                        xml.append("</").append(CablingConstants.VLSB_COMPUTER_ID).append(">");
                        vlsbGeoNumber = "";
                    }
                    vlsbComputerId = rs.getString(CablingConstants.COLUMN_VLSB_COMPUTER_ID);
                    xml.append("<").append(CablingConstants.VLSB_COMPUTER_ID).append(" ")
                            .append(CablingConstants.NUMBER).append("='").append(vlsbComputerId).append("'>");
                }

                if (!rs.getString(CablingConstants.COLUMN_VLSB_GEO_NUMBER).equals(vlsbGeoNumber)) {
                    if (!vlsbGeoNumber.isEmpty()) {
                        xml.append("</").append(CablingConstants.VLSB_GEO_NUMBER).append(">");
                    }
                    vlsbGeoNumber = rs.getString(CablingConstants.COLUMN_VLSB_GEO_NUMBER);
                    xml.append("<").append(CablingConstants.VLSB_GEO_NUMBER).append(" ")
                            .append(CablingConstants.NUMBER).append("='").append(vlsbGeoNumber).append("'>");
                }

                xml.append("<").append(CablingConstants.VLSB_CHANNEL).append(" ").append(CablingConstants.NUMBER)
                        .append("='").append(rs.getString(CablingConstants.COLUMN_VLSB_CHANNEL)).append("' ")
                        .append(CablingConstants.TRACK_NO).append("='")
                        .append(rs.getString(CablingConstants.COLUMN_TRACKER_NO)).append("' ")
                        .append(CablingConstants.STATION).append("='")
                        .append(rs.getString(CablingConstants.COLUMN_STATION)).append("' ")
                        .append(CablingConstants.PLANE).append("='")
                        .append(rs.getString(CablingConstants.COLUMN_PLANE)).append("' ")
                        .append(CablingConstants.CHANNEL).append("='")
                        .append(rs.getString(CablingConstants.COLUMN_CHANNEL)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        if (!vlsbComputerId.isEmpty()) {
            xml.append("</").append(CablingConstants.VLSB_GEO_NUMBER).append(">");
            xml.append("</").append(CablingConstants.VLSB_COMPUTER_ID).append(">");
        }
        xml.append("</").append(CablingConstants.CABLING).append(">");

        return xml.toString();
    }

    /**
     * Select all ids that would have been valid at the given time.
     * 
     * @param connection
     *            db connection
     * @param time
     *            the time of interest
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp time) throws CDBException {
        m_log.debug("selectIds startTime:" + time);
        StringBuilder maxValidFromTime = new StringBuilder();
        maxValidFromTime.append("SELECT max(").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(") FROM ")
                .append(CablingConstants.TABLE_CABLING_DETECTOR).append(" WHERE ")
                .append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= '").append(time).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CablingConstants.COLUMN_ID).append(", ")
                .append(CablingConstants.COLUMN_VALID_FROM_TIME).append(", ")
                .append(CablingConstants.COLUMN_CREATIONTIME).append(", ").append(CablingConstants.COLUMN_DEVICE)
                .append(" FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR).append(" WHERE ")
                .append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" = (").append(maxValidFromTime)
                .append(") ORDER BY ").append(CablingConstants.COLUMN_CREATIONTIME).append(";");
        return selectIds(connection, select.toString());
    }

    /**
     * Select all ids that would have been valid during the given time range.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @param stopTime
     *            the end of the time range
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp startTime, Timestamp stopTime) throws CDBException {
        m_log.debug("selectIds startTime:" + startTime + " stopTime:" + stopTime);
        String startValidFromTime = selectStartValidFromTime(connection, startTime);
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CablingConstants.COLUMN_ID).append(", ")
                .append(CablingConstants.COLUMN_VALID_FROM_TIME).append(", ")
                .append(CablingConstants.COLUMN_CREATIONTIME).append(", ").append(CablingConstants.COLUMN_DEVICE)
                .append(" FROM ").append(CablingConstants.TABLE_CABLING_DETECTOR).append(" WHERE ")
                .append(startValidFromTime).append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= '")
                .append(stopTime).append("' ORDER BY ").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(", ")
                .append(CablingConstants.COLUMN_CREATIONTIME).append(";");
        return selectIds(connection, select.toString());
    }

    /**
     * Get an SQL expression that includes the ValidFromTime that immediately
     * precedes the given time. If the given time is before the earliest time
     * then use the earliest ValidFromTime.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @return a string containing an SQL snippet
     * @throws CDBException
     */
    private String selectStartValidFromTime(Connection connection, Timestamp startTime) throws CDBException {
        m_log.debug("selectStartValidFromTime startTime:" + startTime);
        StringBuilder select = new StringBuilder();
        select.append("SELECT max(").append(CablingConstants.COLUMN_VALID_FROM_TIME).append(") FROM ")
                .append(CablingConstants.TABLE_CABLING_DETECTOR).append(" WHERE ")
                .append(CablingConstants.COLUMN_VALID_FROM_TIME).append(" <= '").append(startTime).append("'");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            rs.next();
            Timestamp startValidFromTime = rs.getTimestamp("max");
            if (startValidFromTime == null) {
                return "";
            } else {
                return CablingConstants.COLUMN_VALID_FROM_TIME + " >= '" + startValidFromTime + "' AND ";
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select ids
     * 
     * @param connection
     *            db connection
     * @param select
     *            a string containing the select statement.
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, String select) throws CDBException {
        m_log.debug("selectIds select:" + select);
        StringBuilder xml = new StringBuilder();
        xml = xml.append("<ids>");
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                xml = xml.append("<id name='").append(rs.getString(CablingConstants.COLUMN_ID)).append("' validFrom='")
                        .append(rs.getString(CablingConstants.COLUMN_VALID_FROM_TIME)).append("' created='")
                        .append(rs.getString(CablingConstants.COLUMN_CREATIONTIME)).append("' device='")
                        .append(rs.getString(CablingConstants.COLUMN_DEVICE)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml = xml.append("</ids>");
        return xml.toString();
    }

    /**
     * Check that the device is a known device.
     * 
     * @param device
     *            name of device
     * @throws CDBException
     */
    private void validateDevice(String device) throws CDBException {
        m_validator.validateStringNonBlank(device);
        String upperName = device.toUpperCase().trim();
        if (!m_detectors.contains(upperName)) {
            m_log.debug("Unknown device:" + device);
            throw new CDBException("Unknown device:" + device);
        }
    }
}
