package mice.cdb.service.cabling;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>CablingSuperMouse</code> interface is used to store and retrieve
 * cabling data. Data can be stored and updated. Old versions of the channel
 * data are stored for diagnostic purposes.
 * </p>
 * 
 * <p>
 * The returned XML may contain a message enclosed in one of three tags:
 * <code>ok</code>, <code>warning</code> or <code>error</code>. A tag of
 * <code>warning</code> indicates that the problem maybe transient and retrying
 * the request MAY succeed. A tag of <code>error</code> maybe due to to bad data
 * being passed in or an unexpected internal error.
 * </p>
 */
@WebService
public interface CablingSuperMouse extends Cabling {

    /**
     * Add the cabling data. A timestamp is associated with the data. <b>
     * N.B.</b> There must NOT already be an entry for the crate, module,
     * channel otherwise an error will be reported.
     * 
     * @param crate
     *            the number of the crate
     * @param module
     *            the number of the module
     * @param channel
     *            the number of the channel
     * @param name
     *            the name of the device that is connected to the channel
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String addControl(String crate, String module, String channel, String name);

    /**
     * Update the name for the given crate, module, channel. A timestamp is
     * associated with the data. <b>N.B.</b> There must already be an entry for
     * the crate, module, channel otherwise an error will be reported.
     * 
     * @param crate
     *            the number of the crate
     * @param module
     *            the number of the module
     * @param channel
     *            the number of the channel
     * @param name
     *            the name of the device that is connected to the channel
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String updateControl(String crate, String module, String channel, String name);

    /**
     * Update the data for the given board, bank, channel. A timestamp is
     * associated with the data.
     * 
     * @param device
     *            the name of the tracker
     * @param xml
     *            a string containing XML representing the channel
     *            <dl>
     *            <dd><code>&lt;cabling&gt;</code></dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tracker vlsbComputerId='n' vlsbGeoNumber='n' vlsbChannel='n' trackerNo='n' station='n' plane='n.n' channel='n'/&gt;</code>
     *            </dd>
     *            <dd>
     *            <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;tracker vlsbComputerId='n' vlsbGeoNumber='n' vlsbChannel='n' trackerNo='n' station='n' plane='n.n' channel='n'/&gt;</code>
     *            </dd>
     *            <dd><code>&lt;/cabling&gt;</code></dd>
     *            </dl>
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setTracker(String device, String xml);

    /**
     * Update the data for the given device. A timestamp is associated with the
     * data. Using a byte array enables the user to pass in the data in a zipped
     * format if so required.
     * 
     * @param device
     *            the name of the detector
     * @param validFromTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC.
     *            This should represent the time from which this data set is
     *            considered to be valid.
     * @param data
     *            a byte array containing a calibration data set
     * 
     * @return a string containing XML with one element of type <code>ok</code>,
     *         <code>warning</code> or <code>error</code>
     */
    @WebMethod
    public abstract String setDetector(String device, String validFromTime, byte[] data);

}
