package mice.cdb.service.cabling;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.parser.CablingParser;
import mice.cdb.service.parser.ParseException;

/**
 * An implementation of the <code>CablingSuperMouse</code>.
 * 
 * @see mice.cdb.service.cabling.CablingSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.cabling.CablingSuperMouse")
public class CablingSuperMouseImpl extends CablingImpl implements CablingSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Construct a Cabling with super mouse powers.
     */
    public CablingSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public String addControl(String crate, String module, String channel, String name) {
        m_log.debug("addControl(" + crate + ", " + module + ", " + channel + ", " + name + ")");
        return control(CablingConstants.ADD, crate, module, channel, name);
    }

    @Override
    public String updateControl(String crate, String module, String channel, String name) {
        m_log.debug("updateControl(" + crate + ", " + module + ", " + channel + ", " + name + ")");
        return control(CablingConstants.UPDATE, crate, module, channel, name);
    }

    @Override
    public String setTracker(String device, String xml) {
        m_log.debug("setTracker(" + device + ", " + xml + ")");
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        String message = "";
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            m_validator.validateStringNonBlank(device);
            String upperDevice = device.toUpperCase().trim();
            if (m_trackers.contains(upperDevice)) {

                // parse the XML
                CablingParser cablingParser = new CablingParser();
                try {
                    cablingParser.parse(xml);
                } catch (ParseException e) {
                    return "<error>Could not parse Cabling XML. Please check input. " + e.getMessage() + "</error>";
                }

                // insert the data
                List<TrackerData> trackerData = cablingParser.getTrackerData();
                message = updateTracker(connection, creationTime, upperDevice, trackerData);
            } else {
                m_log.debug("Unknown tracker:" + upperDevice);
                return "<error>" + "Unknown tracker:" + upperDevice + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return message;
    }

    @Override
    public synchronized String setDetector(String device, String validFromTime, byte[] data) {
        m_log.debug("setDetector(" + device + ", " + validFromTime + ")");
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        int inserted = 0;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            m_validator.validateStringNonBlank(device);
            String upperDevice = device.toUpperCase().trim();
            Timestamp validFrom = m_validator.getTimeStamp(validFromTime);
            if (m_detectors.contains(upperDevice)) {

                // insert the data
                StringBuilder insert = new StringBuilder();
                insert.append("INSERT INTO ").append(CablingConstants.TABLE_CABLING_DETECTOR);
                insert.append(" (").append(CablingConstants.COLUMN_DEVICE);
                insert.append(", ").append(CablingConstants.COLUMN_CREATIONTIME);
                insert.append(", ").append(CablingConstants.COLUMN_VALID_FROM_TIME);
                insert.append(", ").append(CablingConstants.COLUMN_DATA).append(")");
                insert.append(" VALUES ('").append(upperDevice).append("'");
                insert.append(", '").append(creationTime).append("'");
                insert.append(", '").append(validFrom).append("'");
                insert.append(", ?);");
                m_log.debug("insert:" + insert);
                PreparedStatement ps = m_mdb.prepareStatement(connection, insert.toString());
                ps.setBytes(1, data);
                inserted = ps.executeUpdate();
                ps.close();
            } else {
                m_log.debug("Unknown detector:" + upperDevice);
                return "<error>" + "Unknown detector:" + upperDevice + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } catch (SQLException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        if (inserted == 1) {
            m_log.info("Inserted cabling, " + data.length + " bytes");
            return "<ok>Inserted cabling, " + data.length + " bytes</ok>";
        } else {
            m_log.info("Cabling not inserted");
            return "<error>Cabling not inserted</error>";
        }
    }

    private synchronized String control(String operation, String crateString, String moduleString,
            String channelString, String name) {
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        String message = "";
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            int crate = m_validator.getInt(crateString);
            int module = m_validator.getInt(moduleString);
            int channel = m_validator.getInt(channelString);
            m_validator.validateStringNonBlank(name);
            StringBuilder sql = new StringBuilder();
            sql.append("SELECT COUNT(*) FROM ").append(CablingConstants.TABLE_CABLING_CONTROL).append(" WHERE ")
                    .append(CablingConstants.COLUMN_CRATE).append("=").append(crate).append(" AND ")
                    .append(CablingConstants.COLUMN_MODULE).append("=").append(module).append(" AND ")
                    .append(CablingConstants.COLUMN_CHANNEL).append("=").append(channel).append(";");
            if (operation.equals(CablingConstants.ADD)) {
                message = addControl(connection, newEntry(connection, sql.toString()), crate, module, channel, name,
                        creationTime);
            } else {
                message = updateControl(connection, newEntry(connection, sql.toString()), crate, module, channel, name,
                        creationTime);
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return message;
    }

    private String addControl(Connection connection, boolean newEntry, int crate, int module, int channel, String name,
            Timestamp creationTime) {
        m_log.debug("addControl(" + newEntry + ", " + crate + ", " + module + ", " + channel + ", " + name + ", "
                + creationTime + ")");
        try {
            if (!newEntry) {
                return "<error>Entry for crate:" + crate + "' module:" + module + ", channel:" + channel
                        + " already exists</error>";
            }
            StringBuilder sql = new StringBuilder();
            sql.append("INSERT INTO ").append(CablingConstants.TABLE_CABLING_CONTROL).append(" (")
                    .append(CablingConstants.COLUMN_CRATE).append(", ").append(CablingConstants.COLUMN_MODULE)
                    .append(", ").append(CablingConstants.COLUMN_CHANNEL).append(", ")
                    .append(CablingConstants.COLUMN_NAME).append(", ").append(CablingConstants.COLUMN_CREATIONTIME)
                    .append(") VALUES (").append(crate).append(", ").append(module).append(", ").append(channel)
                    .append(", '").append(name).append("', '").append(creationTime).append("');");
            m_mdb.insert(connection, sql.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        }
        m_log.info("Control cabling added");
        return "<ok>Control cabling added</ok>";
    }

    private String updateControl(Connection connection, boolean newEntry, int crate, int module, int channel,
            String name, Timestamp creationTime) {
        m_log.debug("updateControl(" + newEntry + ", " + crate + ", " + module + ", " + channel + ", " + name + ", "
                + creationTime + ")");
        try {
            if (newEntry) {
                return "<error>Entry for crate:" + crate + "' module:" + module + ", channel:" + channel
                        + " does not exist</error>";
            }
            StringBuilder sql = new StringBuilder();
            sql.append("UPDATE ").append(CablingConstants.TABLE_CABLING_CONTROL).append(" SET ")
                    .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                    .append("' WHERE ").append(CablingConstants.COLUMN_CRATE).append("=").append(crate).append(" AND ")
                    .append(CablingConstants.COLUMN_MODULE).append("=").append(module).append(" AND ")
                    .append(CablingConstants.COLUMN_CHANNEL).append("=").append(channel).append(" AND ")
                    .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
            m_mdb.insert(connection, sql.toString());
            sql = new StringBuilder();
            sql.append("INSERT INTO ").append(CablingConstants.TABLE_CABLING_CONTROL).append(" (")
                    .append(CablingConstants.COLUMN_CRATE).append(", ").append(CablingConstants.COLUMN_MODULE)
                    .append(", ").append(CablingConstants.COLUMN_CHANNEL).append(", ")
                    .append(CablingConstants.COLUMN_NAME).append(", ").append(CablingConstants.COLUMN_CREATIONTIME)
                    .append(") VALUES (").append(crate).append(", ").append(module).append(", ").append(channel)
                    .append(", '").append(name).append("', '").append(creationTime).append("');");
            m_mdb.insert(connection, sql.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        }
        m_log.info("Control cabling updated");
        return "<ok>Control cabling updated</ok>";
    }

    /**
     * Update any existing entries then add new entries.
     * 
     * @param connection
     *            db connection
     * @param creationTime
     *            creation time
     * @param device
     *            name of device
     * @param trackerData
     *            tracker data
     * @return a string containing xml
     */
    private synchronized String updateTracker(Connection connection, Timestamp creationTime, String device,
            List<TrackerData> trackerData) {
        try {
            StringBuilder sqlUpdate = new StringBuilder();
            StringBuilder sqlAdd = new StringBuilder();
            for (TrackerData data : trackerData) {
                sqlUpdate.append("UPDATE ").append(CablingConstants.TABLE_CABLING_TRACKER).append(" SET ")
                        .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                        .append("' WHERE ").append(CablingConstants.COLUMN_DEVICE).append("='").append(device)
                        .append("' AND ").append(CablingConstants.COLUMN_VLSB_COMPUTER_ID).append("=")
                        .append(data.getVlsbComputerId()).append(" AND ")
                        .append(CablingConstants.COLUMN_VLSB_GEO_NUMBER).append("=").append(data.getVlsbGeoNumber())
                        .append(" AND ").append(CablingConstants.COLUMN_VLSB_CHANNEL).append("=")
                        .append(data.getVlsbChannel()).append(" AND ").append(CablingConstants.COLUMN_VALID_UNTIL_TIME)
                        .append(" is NULL;");
                sqlAdd.append("INSERT INTO ").append(CablingConstants.TABLE_CABLING_TRACKER).append(" (")
                        .append(CablingConstants.COLUMN_DEVICE).append(", ")
                        .append(CablingConstants.COLUMN_VLSB_COMPUTER_ID).append(", ")
                        .append(CablingConstants.COLUMN_VLSB_GEO_NUMBER).append(", ")
                        .append(CablingConstants.COLUMN_VLSB_CHANNEL).append(", ")
                        .append(CablingConstants.COLUMN_TRACKER_NO).append(", ")
                        .append(CablingConstants.COLUMN_STATION).append(", ").append(CablingConstants.COLUMN_PLANE)
                        .append(", ").append(CablingConstants.COLUMN_CHANNEL).append(", ")
                        .append(CablingConstants.COLUMN_CREATIONTIME).append(") VALUES ('").append(device)
                        .append("', ").append(data.getVlsbComputerId()).append(", ").append(data.getVlsbGeoNumber())
                        .append(", ").append(data.getVlsbChannel()).append(", ").append(data.getTrackerNo())
                        .append(", ").append(data.getStation()).append(", ").append(data.getPlane()).append(", ")
                        .append(data.getChannel()).append(", '").append(creationTime).append("');");
            }
            m_mdb.batchInsert(connection, sqlUpdate.toString());
            m_mdb.batchInsert(connection, sqlAdd.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        }
        m_log.info("Traker cabling set");
        return "<ok>Tracker cabling set</ok>";
    }

    /**
     * Check to see if a record for the crate, module, channel exists.
     * 
     * @param connection
     *            db connection
     * @param sql
     *            a string containing the select statement
     * @param module
     *            the name of the module
     * @param channel
     *            the name of the channel
     * @return <code>true</code> if does not exist
     * @throws CDBException
     */
    private boolean newEntry(Connection connection, String sql) throws CDBException {
        m_log.debug("newEntry(" + sql + ")");

        // There will only be one row returned
        ResultSet rs = m_mdb.query(connection, sql);
        try {
            rs.next();
            if (rs.getInt("count") > 0) {
                return false;
            } else {
                return true;
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

}
