package mice.cdb.service.calibration;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>Calibration</code> interface is used to retrieve calibration data.
 * To store data use the {@link CalibrationSuperMouse} interface. For the
 * trackers data is described by its location with reference to bank, board and
 * channel. Old versions of the data are stored for diagnostic purposes. Data
 * can be retrieved for a given time using
 * {@link #getCalibrationForDate(String, String) getCalibrationForDate} and for
 * a given run using {@link #getCalibrationForRun(String, String)
 * getCalibrationForRun}
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 * 
 */
@WebService
public interface Calibration {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a tracker. <b>N.B.</b> The xml returned is dependent on
     * type of device.
     * 
     * @param device
     *            the name of the tracker
     * @return a string containing XML representing the channel
     * 
     *         <dl>
     *         <dd><code>&lt;calibration type='x'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;board number='n'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/board&gt;</code></dd>
     *         <dd><code>&lt;/calibration&gt;</code></dd>
     *         </dl>
     * 
     */
    @WebMethod
    public abstract String getCurrentCalibration(String device);

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a tracker, that was valid at the given timestamp.
     * <b>N.B.</b> The xml returned is dependent on type of device.
     * 
     * @param device
     *            the name of the tracker
     * @param timestamp
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * @return a string containing XML representing the channel
     * 
     *         <dl>
     *         <dd><code>&lt;calibration type='x'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;board number='n'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/board&gt;</code></dd>
     *         <dd><code>&lt;/calibration&gt;</code></dd>
     *         </dl>
     * 
     */
    @WebMethod
    public abstract String getCalibrationForDate(String device, String timestamp);

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a tracker, that was valid for the given run number.
     * <b>N.B.</b> The xml returned is dependent on type of device.
     * 
     * @param device
     *            the name of the tracker
     * @param runNo
     *            the number of a run
     * @return a string containing XML representing the channel
     * 
     *         <dl>
     *         <dd><code>&lt;calibration type='x'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;board number='n'&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;bank number='n'&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;channel number='n' adcPedestal='n' adcGain='n' tdcPedestal='n' tdcSlope='n'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/bank&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/board&gt;</code></dd>
     *         <dd><code>&lt;/calibration&gt;</code></dd>
     *         </dl>
     * 
     */
    @WebMethod
    public abstract String getCalibrationForRun(String device, String runNo);

    /**
     * Get a list of known devices. These are the device names that are
     * recognised by this API.
     * 
     * @return a string containing XML representing the list of known devices
     * 
     *         <dl>
     *         <dd><code>&lt;calibration&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;deviceList&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;device name='x'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;device name='x'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/deviceList&gt;</code></dd>
     *         <dd><code>&lt;/calibration&gt;</code></dd>
     *         </dl>
     * 
     */
    @WebMethod
    public abstract String listDevices();

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a detector.
     * 
     * @param device
     *            the name of the detector
     * @param type
     *            the type of the calibration
     * @return a byte array containing the calibration data set
     * 
     */
    @WebMethod
    public abstract byte[] getCurrentDetectorCalibration(String device, String type);

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a detector, that was valid at the given timestamp.
     * 
     * @param device
     *            the name of the detector
     * @param type
     *            the type of the calibration
     * @param timestamp
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * 
     * @return a byte array containing the calibration data set
     */
    @WebMethod
    public abstract byte[] getDetectorCalibrationForDate(String device, String type, String timestamp);

    /**
     * Get the calibration data of the given device, where the name of the
     * device is that of a detector, that was valid for the given run number.
     * 
     * @param device
     *            the name of the detector
     * @param type
     *            the type of the calibration
     * @param runNo
     *            the number of a run
     * 
     * @return a byte array containing the calibration data set
     */
    @WebMethod
    public abstract byte[] getDetectorCalibrationForRun(String device, String type, String runNo);

    /**
     * Get the calibration data that was valid for the given id.
     * 
     * @param id
     *            the id of a data set
     * 
     * @return a byte array containing the calibration data set
     */
    @WebMethod
    public abstract byte[] getDetectorCalibrationForId(String id);

    /**
     * Return a list of ids of the calibration data sets that were valid during
     * the given time period. <b>N.B.</b> if the stopTime is <code>null</code>
     * then only the ids that were valid at the startTime will be returned.
     * 
     * @param startTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * @param stopTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC.
     *            May be <code>null</code>.
     * @return a string containing XML representing the selected data sets with
     *         the format of the XML as
     *         <dl>
     *         <dd><code>&lt;ids&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='1' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' device='xxx' type='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='5' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' device='xxx' type='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='3' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' device='xxx' type='xxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/ids&gt;</code></dd>
     *         <dl>
     */
    @WebMethod
    public abstract String getCalibrationIds(String startTime, String stopTime);

}
