package mice.cdb.service.calibration;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.LinkedList;
import java.util.List;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Calibration</code>.
 * 
 * @see mice.cdb.service.calibration.Calibration
 */
@WebService(endpointInterface = "mice.cdb.service.calibration.Calibration")
public class CalibrationImpl extends CDB implements Calibration {

    protected List<String> m_detectors = new LinkedList<String>();

    protected List<String> m_trackers = new LinkedList<String>();

    /**
     * Construct a Calibration.
     */
    public CalibrationImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
        for (String detector : CalibrationConstants.DETECTORS) {
            m_detectors.add(detector.toUpperCase());
        }
        for (String tracker : CalibrationConstants.TRACKERS) {
            m_trackers.add(tracker.toUpperCase());
        }
    }

    @Override
    public String getCurrentCalibration(String name) {
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateStringNonBlank(name);
            String upperName = name.toUpperCase().trim();

            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();

            clause.append("WHERE ").append(CalibrationConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL");
            if (m_trackers.contains(upperName)) {
                clause.append(" AND ").append(CalibrationConstants.COLUMN_DEVICE).append("='").append(upperName)
                        .append("'");
                xml = selectCalibrationTracker(connection, clause.toString());
            } else {
                m_log.debug("Unknown device:" + name);
                return "<error>" + "Unknown device:" + name + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCalibrationForDate(String name, String timestamp) {
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateStringNonBlank(name);
            String upperName = name.toUpperCase().trim();
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);

            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append("WHERE ").append(CalibrationConstants.COLUMN_CREATIONTIME).append("<='")
                    .append(usersTimestamp).append("' AND (").append(CalibrationConstants.COLUMN_VALID_UNTIL_TIME)
                    .append(">'").append(usersTimestamp).append("' OR ")
                    .append(CalibrationConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL)");
            if (m_trackers.contains(upperName)) {
                clause.append(" AND ").append(CalibrationConstants.COLUMN_DEVICE).append("='").append(upperName)
                        .append("'");
                xml = selectCalibrationTracker(connection, clause.toString());
            } else {
                m_log.debug("Unknown device:" + name);
                return "<error>" + "Unknown device:" + name + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCalibrationForRun(String name, String runNo) {
        String xml = null;
        Connection connection = null;

        try {
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runStartTime = selectRunStartTime(connection, runNumber);
            xml = getCalibrationForDate(name, runStartTime.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String listDevices() {
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(CalibrationConstants.CALIBRATION).append(">");
        xml.append("<").append(CalibrationConstants.DEVICE_LIST).append(">");
        for (String detector : m_detectors) {
            xml.append("<").append(CalibrationConstants.DEVICE).append(" ").append(CalibrationConstants.NAME)
                    .append("='").append(detector).append("'/>");
        }
        for (String tracker : m_trackers) {
            xml.append("<").append(CalibrationConstants.DEVICE).append(" ").append(CalibrationConstants.NAME)
                    .append("='").append(tracker).append("'/>");
        }
        xml.append("</").append(CalibrationConstants.DEVICE_LIST).append(">");
        xml.append("</").append(CalibrationConstants.CALIBRATION).append(">");
        return xml.toString();
    }

    @Override
    public byte[] getDetectorCalibrationForDate(String device, String type, String timestamp) {
        byte[] data = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            Timestamp time = m_validator.getTimeStamp(timestamp);
            validateDevice(device);
            StringBuilder deviceType = new StringBuilder();
            deviceType.append(" AND ").append(CalibrationConstants.COLUMN_DEVICE).append("='").append(device)
                    .append("'");
            deviceType.append(" AND ").append(CalibrationConstants.COLUMN_TYPE).append("='").append(type).append("'");
            StringBuilder maxValidFromTime = new StringBuilder();
            maxValidFromTime.append("SELECT max(").append(CalibrationConstants.COLUMN_VALID_FROM_TIME);
            maxValidFromTime.append(") FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR);
            maxValidFromTime.append(" WHERE ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= '")
                    .append(time).append("'").append(deviceType);
            StringBuilder maxCreationTime = new StringBuilder();
            maxCreationTime.append("SELECT max(").append(CalibrationConstants.COLUMN_CREATIONTIME);
            maxCreationTime.append(") FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR);
            maxCreationTime.append(" WHERE ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" = (")
                    .append(maxValidFromTime).append(")").append(deviceType);
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CalibrationConstants.COLUMN_CREATIONTIME).append(" = (")
                    .append(maxCreationTime);
            clause.append(") AND ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" = (")
                    .append(maxValidFromTime).append(")").append(deviceType);
            data = selectCalibrationDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public byte[] getCurrentDetectorCalibration(String device, String type) {
        Timestamp now = new Timestamp(System.currentTimeMillis());
        return getDetectorCalibrationForDate(device, type, now.toString());
    }

    @Override
    public byte[] getDetectorCalibrationForId(String id) {
        byte[] data = null;
        Connection connection = null;
        try {
            int numericId = m_validator.getInt(id);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CalibrationConstants.COLUMN_ID).append(" = ").append(numericId);
            data = selectCalibrationDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public byte[] getDetectorCalibrationForRun(String device, String type, String runNo) {
        byte[] data = null;
        Connection connection = null;
        try {
            validateDevice(device);
            m_validator.validateStringNonBlank(runNo);
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runTime = selectRunStartTime(connection, runNumber);
            StringBuilder maxCreationTime = new StringBuilder();
            maxCreationTime.append("SELECT max(").append(CalibrationConstants.COLUMN_CREATIONTIME);
            maxCreationTime.append(") FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR);
            maxCreationTime.append(" WHERE ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= ('")
                    .append(runTime).append("')");
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CalibrationConstants.COLUMN_CREATIONTIME).append(" = (")
                    .append(maxCreationTime);
            clause.append(") AND ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= ('")
                    .append(runTime).append("')");
            data = selectCalibrationDetector(connection, clause);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public String getCalibrationIds(String startTime, String stopTime) {
        String data = null;
        Connection connection = null;
        try {
            Timestamp start = m_validator.getTimeStamp(startTime);
            connection = m_mdb.getConnect();
            if (stopTime == null) {
                data = selectIds(connection, start);
            } else {
                Timestamp stop = m_validator.getTimeStamp(stopTime);
                if (start.after(stop)) {
                    String message = "<error>Start time must be before stop time.</error>";
                    return message;
                }
                data = selectIds(connection, start, stop);
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    /**
     * Get the calibration data from the database for a detector.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return the data
     * @throws CDBException
     */
    private byte[] selectCalibrationDetector(Connection connection, StringBuilder clause) throws CDBException {
        m_log.debug("selectCalibrationDetector(" + clause + ")");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CalibrationConstants.COLUMN_DATA);
        select.append(" FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR);
        select.append(clause).append(";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getBytes(CalibrationConstants.COLUMN_DATA);
            } else {
                m_log.debug("No data returned from the database.\n\t" + select);
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select the calibration for a tracker.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectCalibrationTracker(Connection connection, String clause) throws CDBException {
        m_log.debug("selectCalibrationTracker(" + clause + ")");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CalibrationConstants.COLUMN_BOARD).append(", ")
                .append(CalibrationConstants.COLUMN_BANK).append(", ").append(CalibrationConstants.COLUMN_CHANNEL)
                .append(", ").append(CalibrationConstants.COLUMN_ADC_PEDESTAL).append(", ")
                .append(CalibrationConstants.COLUMN_ADC_GAIN).append(", ")
                .append(CalibrationConstants.COLUMN_TDC_PEDESTAL).append(", ")
                .append(CalibrationConstants.COLUMN_TDC_SLOPE).append(" FROM ")
                .append(CalibrationConstants.TABLE_CALIBRATION_TRACKER).append(" ").append(clause).append(" ORDER BY ")
                .append(CalibrationConstants.COLUMN_BOARD).append(", ").append(CalibrationConstants.COLUMN_BANK)
                .append(", ").append(CalibrationConstants.COLUMN_CHANNEL).append(";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(CalibrationConstants.CALIBRATION).append(" ").append(CalibrationConstants.TYPE)
                .append("='").append(CalibrationConstants.TRACKER).append("'>");
        String board = "";
        String bank = "";

        try {
            while (rs.next()) {

                if (!rs.getString(CalibrationConstants.COLUMN_BOARD).equals(board)) {
                    if (!board.isEmpty()) {
                        xml.append("</").append(CalibrationConstants.BANK).append(">");
                        xml.append("</").append(CalibrationConstants.BOARD).append(">");
                        bank = "";
                    }
                    board = rs.getString(CalibrationConstants.COLUMN_BOARD);
                    xml.append("<").append(CalibrationConstants.BOARD).append(" ").append(CalibrationConstants.NUMBER)
                            .append("='").append(board).append("'>");
                }

                if (!rs.getString(CalibrationConstants.COLUMN_BANK).equals(bank)) {
                    if (!bank.isEmpty()) {
                        xml.append("</").append(CalibrationConstants.BANK).append(">");
                    }
                    bank = rs.getString(CalibrationConstants.COLUMN_BANK);
                    xml.append("<").append(CalibrationConstants.BANK).append(" ").append(CalibrationConstants.NUMBER)
                            .append("='").append(bank).append("'>");
                }

                xml.append("<").append(CalibrationConstants.CHANNEL).append(" ").append(CalibrationConstants.NUMBER)
                        .append("='").append(rs.getString(CalibrationConstants.COLUMN_CHANNEL)).append("' ")
                        .append(CalibrationConstants.ADC_PEDESTAL).append("='")
                        .append(rs.getString(CalibrationConstants.COLUMN_ADC_PEDESTAL)).append("' ")
                        .append(CalibrationConstants.ADC_GAIN).append("='")
                        .append(rs.getString(CalibrationConstants.COLUMN_ADC_GAIN)).append("' ")
                        .append(CalibrationConstants.TDC_PEDESTAL).append("='")
                        .append(rs.getString(CalibrationConstants.COLUMN_TDC_PEDESTAL)).append("' ")
                        .append(CalibrationConstants.TDC_SLOPE).append("='")
                        .append(rs.getString(CalibrationConstants.COLUMN_TDC_SLOPE)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        if (!board.isEmpty()) {
            xml.append("</").append(CalibrationConstants.BANK).append(">");
            xml.append("</").append(CalibrationConstants.BOARD).append(">");
        }
        xml.append("</").append(CalibrationConstants.CALIBRATION).append(">");

        return xml.toString();
    }

    /**
     * Select all ids that would have been valid at the given time.
     * 
     * @param connection
     *            db connection
     * @param time
     *            the time of interest
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp time) throws CDBException {
        m_log.debug("selectIds startTime:" + time);
        StringBuilder maxValidFromTime = new StringBuilder();
        maxValidFromTime.append("SELECT max(").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(") FROM ")
                .append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR).append(" WHERE ")
                .append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= '").append(time).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CalibrationConstants.COLUMN_ID).append(", ")
                .append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(", ")
                .append(CalibrationConstants.COLUMN_CREATIONTIME).append(", ")
                .append(CalibrationConstants.COLUMN_DEVICE).append(", ").append(CalibrationConstants.COLUMN_TYPE)
                .append(" FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR).append(" WHERE ")
                .append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" = (").append(maxValidFromTime)
                .append(") ORDER BY ").append(CalibrationConstants.COLUMN_CREATIONTIME).append(";");
        return selectIds(connection, select.toString());
    }

    /**
     * Select all ids that would have been valid during the given time range.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @param stopTime
     *            the end of the time range
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp startTime, Timestamp stopTime) throws CDBException {
        m_log.debug("selectIds startTime:" + startTime + " stopTime:" + stopTime);
        String startValidFromTime = selectStartValidFromTime(connection, startTime);
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CalibrationConstants.COLUMN_ID).append(", ")
                .append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(", ")
                .append(CalibrationConstants.COLUMN_CREATIONTIME).append(", ")
                .append(CalibrationConstants.COLUMN_DEVICE).append(", ").append(CalibrationConstants.COLUMN_TYPE)
                .append(" FROM ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR).append(" WHERE ")
                .append(startValidFromTime).append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= '")
                .append(stopTime).append("' ORDER BY ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME)
                .append(", ").append(CalibrationConstants.COLUMN_CREATIONTIME).append(";");
        return selectIds(connection, select.toString());
    }

    /**
     * Get an SQL expression that includes the ValidFromTime that immediately
     * precedes the given time. If the given time is before the earliest time
     * then use the earliest ValidFromTime.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @return a string containing an SQL snippet
     * @throws CDBException
     */
    private String selectStartValidFromTime(Connection connection, Timestamp startTime) throws CDBException {
        m_log.debug("selectStartValidFromTime startTime:" + startTime);
        StringBuilder select = new StringBuilder();
        select.append("SELECT max(").append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(") FROM ")
                .append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR).append(" WHERE ")
                .append(CalibrationConstants.COLUMN_VALID_FROM_TIME).append(" <= '").append(startTime).append("'");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            rs.next();
            Timestamp startValidFromTime = rs.getTimestamp("max");
            if (startValidFromTime == null) {
                return "";
            } else {
                return CalibrationConstants.COLUMN_VALID_FROM_TIME + " >= '" + startValidFromTime + "' AND ";
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select ids
     * 
     * @param connection
     *            db connection
     * @param select
     *            a string containing the select statement.
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, String select) throws CDBException {
        m_log.debug("selectIds select:" + select);
        StringBuilder xml = new StringBuilder();
        xml = xml.append("<ids>");
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                xml = xml.append("<id name='").append(rs.getString(CalibrationConstants.COLUMN_ID))
                        .append("' validFrom='").append(rs.getString(CalibrationConstants.COLUMN_VALID_FROM_TIME))
                        .append("' created='").append(rs.getString(CalibrationConstants.COLUMN_CREATIONTIME))
                        .append("' device='").append(rs.getString(CalibrationConstants.COLUMN_DEVICE))
                        .append("' type='").append(rs.getString(CalibrationConstants.COLUMN_TYPE)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml = xml.append("</ids>");
        return xml.toString();
    }

    /**
     * Check that the device is a known device.
     * 
     * @param device
     *            name of device
     * @throws CDBException
     */
    private void validateDevice(String device) throws CDBException {
        m_validator.validateStringNonBlank(device);
        String upperName = device.toUpperCase().trim();
        if (!m_detectors.contains(upperName)) {
            m_log.debug("Unknown device:" + device);
            throw new CDBException("Unknown device:" + device);
        }
    }

}
