package mice.cdb.service.calibration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.parser.CalibrationParser;
import mice.cdb.service.parser.ParseException;

/**
 * An implementation of the <code>CalibrationSuperMouse</code>.
 * 
 * @see mice.cdb.service.calibration.CalibrationSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.calibration.CalibrationSuperMouse")
public class CalibrationSuperMouseImpl extends CalibrationImpl implements CalibrationSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Construct a Calibration with super mouse powers.
     */
    public CalibrationSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String setTracker(String device, String xml) {
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        String message = "";
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            m_validator.validateStringNonBlank(device);
            String upperDevice = device.toUpperCase().trim();
            if (m_trackers.contains(upperDevice)) {

                // parse the XML
                CalibrationParser caliParser = new CalibrationParser();
                try {
                    caliParser.parse(xml);
                } catch (ParseException e) {
                    return "<error>Could not parse Calibration XML. Please check input. " + e.getMessage() + "</error>";
                }

                // insert the data
                List<TrackerData> trackerData = caliParser.getTrackerData();
                message = updateTracker(connection, creationTime, upperDevice, trackerData);
            } else {
                m_log.debug("Unknown tracker:" + upperDevice);
                return "<error>" + "Unknown tracker:" + upperDevice + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return message;
    }

    @Override
    public synchronized String setDetector(String device, String type, String validFromTime, byte[] data) {
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        int inserted = 0;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            m_validator.validateStringNonBlank(device);
            String upperDevice = device.toUpperCase().trim();
            m_validator.validateStringNonBlank(type);
            Timestamp validFrom = m_validator.getTimeStamp(validFromTime);
            if (m_detectors.contains(upperDevice)) {

                // insert the data
                StringBuilder insert = new StringBuilder();
                insert.append("INSERT INTO ").append(CalibrationConstants.TABLE_CALIBRATION_DETECTOR);
                insert.append(" (").append(CalibrationConstants.COLUMN_DEVICE);
                insert.append(", ").append(CalibrationConstants.COLUMN_TYPE);
                insert.append(", ").append(CalibrationConstants.COLUMN_CREATIONTIME);
                insert.append(", ").append(CalibrationConstants.COLUMN_VALID_FROM_TIME);
                insert.append(", ").append(CalibrationConstants.COLUMN_DATA).append(")");
                insert.append(" VALUES ('").append(upperDevice).append("'");
                insert.append(", '").append(type).append("'");
                insert.append(", '").append(creationTime).append("'");
                insert.append(", '").append(validFrom).append("'");
                insert.append(", ?);");
                m_log.debug("insert:" + insert);
                PreparedStatement ps = m_mdb.prepareStatement(connection, insert.toString());
                ps.setBytes(1, data);
                inserted = ps.executeUpdate();
                ps.close();
            } else {
                m_log.debug("Unknown detector:" + upperDevice);
                return "<error>" + "Unknown detector:" + upperDevice + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } catch (SQLException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        if (inserted == 1) {
            m_log.info("Inserted calibration, " + data.length + " bytes");
            return "<ok>Inserted calibration, " + data.length + " bytes</ok>";
        } else {
            m_log.info("Calibration not inserted");
            return "<error>Calibration not inserted</error>";
        }
    }

    /**
     * Update any existing entries then add new entries.
     * 
     * @param connection
     *            db connection
     * @param creationTime
     *            creation time
     * @param device
     *            name of device
     * @param trackerData
     *            tracker data
     * @return a string containing xml
     */
    private synchronized String updateTracker(Connection connection, Timestamp creationTime, String device,
            List<TrackerData> trackerData) {
        try {
            StringBuilder sqlUpdate = new StringBuilder();
            StringBuilder sqlAdd = new StringBuilder();
            for (TrackerData data : trackerData) {
                sqlUpdate.append("UPDATE ").append(CalibrationConstants.TABLE_CALIBRATION_TRACKER).append(" SET ")
                        .append(CalibrationConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                        .append("' WHERE ").append(CalibrationConstants.COLUMN_DEVICE).append("='").append(device)
                        .append("' AND ").append(CalibrationConstants.COLUMN_BOARD).append("=").append(data.getBoard())
                        .append(" AND ").append(CalibrationConstants.COLUMN_BANK).append("=").append(data.getBank())
                        .append(" AND ").append(CalibrationConstants.COLUMN_CHANNEL).append("=")
                        .append(data.getChannel()).append(" AND ").append(CalibrationConstants.COLUMN_VALID_UNTIL_TIME)
                        .append(" is NULL;");
                sqlAdd.append("INSERT INTO ").append(CalibrationConstants.TABLE_CALIBRATION_TRACKER).append(" (")
                        .append(CalibrationConstants.COLUMN_DEVICE).append(", ")
                        .append(CalibrationConstants.COLUMN_BOARD).append(", ")
                        .append(CalibrationConstants.COLUMN_BANK).append(", ")
                        .append(CalibrationConstants.COLUMN_CHANNEL).append(", ")
                        .append(CalibrationConstants.COLUMN_ADC_PEDESTAL).append(", ")
                        .append(CalibrationConstants.COLUMN_ADC_GAIN).append(", ")
                        .append(CalibrationConstants.COLUMN_TDC_PEDESTAL).append(", ")
                        .append(CalibrationConstants.COLUMN_TDC_SLOPE).append(", ")
                        .append(CalibrationConstants.COLUMN_CREATIONTIME).append(") VALUES ('").append(device)
                        .append("', ").append(data.getBoard()).append(", ").append(data.getBank()).append(", ")
                        .append(data.getChannel()).append(", ").append(data.getAdcPedestal()).append(", ")
                        .append(data.getAdcGain()).append(", ").append(data.getTdcPedestal()).append(", ")
                        .append(data.getTdcSlope()).append(", '").append(creationTime).append("');");
            }
            m_mdb.batchInsert(connection, sqlUpdate.toString());
            m_mdb.batchInsert(connection, sqlAdd.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        }
        m_log.info("Traker calibration set");
        return "<ok>Tracker calibration set</ok>";
    }

}
