package mice.cdb.service.control;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.cabling.CablingConstants;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Control</code>.
 * 
 * @see mice.cdb.service.control.Control
 */
@WebService(endpointInterface = "mice.cdb.service.control.Control")
public class ControlImpl extends CDB implements Control {

    /**
     * Construct a Control.
     */
    public ControlImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public String getControls() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getControls()");
        }
        String xml = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            xml = selectLatestControls(connection, "");
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getControlsForCrate(String crateString) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getControlsForCrate(" + crateString + ")");
        }
        String xml = null;
        Connection connection = null;
        try {
            int crate = m_validator.getInt(crateString);
            StringBuilder clause = new StringBuilder();
            clause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_CRATE).append("=").append(crate);
            connection = m_mdb.getConnect();
            xml = selectLatestControls(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getControlsForChannel(String crateString, String slotString, String channelString) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getControlsForChannel(" + crateString + ", " + slotString + ", " + channelString + ")");
        }
        String xml = null;
        Connection connection = null;
        try {
            int crate = m_validator.getInt(crateString);
            int slot = m_validator.getInt(slotString);
            int channel = m_validator.getInt(channelString);
            StringBuilder clause = new StringBuilder();
            clause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_CRATE).append("=").append(crate);
            clause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_MODULE).append("=").append(slot);
            clause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_CHANNEL).append("=").append(channel);
            connection = m_mdb.getConnect();
            xml = selectLatestControls(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getPreviousSettings(String timestamp) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getPreviousSettings(" + timestamp + ")");
        }
        String xml = null;
        Connection connection = null;
        try {
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);
            StringBuilder clause = new StringBuilder();
            clause.append(" AND ").append(CablingConstants.TABLE_CABLING_CONTROL).append(".")
                    .append(CablingConstants.COLUMN_CREATIONTIME).append("<='").append(usersTimestamp).append("'");
            clause.append(" AND (").append(CablingConstants.TABLE_CABLING_CONTROL).append(".")
                    .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(">'").append(usersTimestamp)
                    .append("' OR ").append(CablingConstants.TABLE_CABLING_CONTROL).append(".")
                    .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL)");
            clause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_CREATIONTIME).append("<='").append(usersTimestamp).append("'");
            clause.append(" AND (").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_VALID_UNTIL_TIME).append(">'").append(usersTimestamp)
                    .append("' OR ").append(ControlConstants.TABLE_CONTROLS).append(".")
                    .append(ControlConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL)");
            connection = m_mdb.getConnect();
            xml = selectControls(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    /**
     * Select the latest values for all controls matching the clause.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectLatestControls(Connection connection, String clause) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectLatestControls(" + clause + ")");

        }
        StringBuilder latestClause = new StringBuilder();
        latestClause.append(clause);
        latestClause.append(" AND ").append(CablingConstants.TABLE_CABLING_CONTROL).append(".")
                .append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL");
        latestClause.append(" AND ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL ");
        return selectControls(connection, latestClause.toString());
    }

    /**
     * Select the latest values for all controls matching the clause.
     * 
     * @param connection
     *            db connection
     * @param clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectControls(Connection connection, String clause) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectControls(" + clause + ")");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_CRATE).append(", ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_MODULE).append(", ").append(ControlConstants.TABLE_CONTROLS)
                .append(".").append(ControlConstants.COLUMN_CHANNEL).append(", ").append(CablingConstants.COLUMN_NAME)
                .append(", ").append(ControlConstants.COLUMN_PARAMETER_NAME).append(", ")
                .append(ControlConstants.COLUMN_PARAMETER_VALUE).append(" FROM ")
                .append(CablingConstants.TABLE_CABLING_CONTROL).append(", ").append(ControlConstants.TABLE_CONTROLS)
                .append(" WHERE ").append(CablingConstants.TABLE_CABLING_CONTROL).append(".")
                .append(CablingConstants.COLUMN_CRATE).append(" = ").append(ControlConstants.TABLE_CONTROLS)
                .append(".").append(ControlConstants.COLUMN_CRATE).append(" AND ")
                .append(CablingConstants.TABLE_CABLING_CONTROL).append(".").append(CablingConstants.COLUMN_MODULE)
                .append(" = ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_MODULE).append(" AND ").append(CablingConstants.TABLE_CABLING_CONTROL)
                .append(".").append(CablingConstants.COLUMN_CHANNEL).append(" = ")
                .append(ControlConstants.TABLE_CONTROLS).append(".").append(ControlConstants.COLUMN_CHANNEL)
                .append(clause).append(" ORDER BY ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_CRATE).append(", ").append(ControlConstants.TABLE_CONTROLS).append(".")
                .append(ControlConstants.COLUMN_MODULE).append(", ").append(ControlConstants.TABLE_CONTROLS)
                .append(".").append(ControlConstants.COLUMN_CHANNEL).append(", ")
                .append(ControlConstants.COLUMN_PARAMETER_NAME + ";");
        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(ControlConstants.CONTROLS).append(">");
        String crate = "";
        String channel = "";

        try {
            while (rs.next()) {

                if (!rs.getString(ControlConstants.COLUMN_CRATE).equals(crate)) {
                    if (!crate.isEmpty()) {
                        xml.append("</").append(ControlConstants.CHANNEL).append(">");
                        xml.append("</").append(ControlConstants.CRATE).append(">");
                        channel = "";
                    }
                    xml.append("<").append(ControlConstants.CRATE).append(" ").append(ControlConstants.CRATE_NAME)
                            .append("='").append(rs.getInt(ControlConstants.COLUMN_CRATE)).append("'>");
                    crate = rs.getString(ControlConstants.COLUMN_CRATE);
                }

                if (!rs.getString(ControlConstants.COLUMN_CHANNEL).equals(channel)) {
                    if (!channel.isEmpty()) {
                        xml.append("</").append(ControlConstants.CHANNEL).append(">");
                    }
                    xml.append("<").append(ControlConstants.CHANNEL).append(" ").append(ControlConstants.CHANNEL_NAME)
                            .append("='").append(rs.getInt(ControlConstants.COLUMN_CHANNEL)).append("' ")
                            .append(ControlConstants.MODULE).append("='")
                            .append(rs.getInt(ControlConstants.COLUMN_MODULE)).append("' ")
                            .append(ControlConstants.REMOTE_CHANNEL_NAME).append("='")
                            .append(rs.getString(CablingConstants.COLUMN_NAME)).append("'>");
                    channel = rs.getString(ControlConstants.COLUMN_CHANNEL);
                }

                xml.append("<").append(ControlConstants.PARAMETER).append(" ").append(ControlConstants.PARAMETER_NAME)
                        .append("='").append(rs.getString(ControlConstants.COLUMN_PARAMETER_NAME)).append("' ")
                        .append(ControlConstants.PARAMETER_VALUE).append("='")
                        .append(rs.getString(ControlConstants.COLUMN_PARAMETER_VALUE) + "'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        if (!crate.isEmpty()) {
            xml.append("</").append(ControlConstants.CHANNEL).append(">");
            xml.append("</").append(ControlConstants.CRATE).append(">");
        }
        xml.append("</").append(ControlConstants.CONTROLS).append(">");

        return xml.toString();
    }
}
