package mice.cdb.service.control;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.cabling.CablingSuperMouseImpl;

/**
 * An implementation of the <code>ControlSuperMouse</code>.
 * 
 * @see mice.cdb.service.control.ControlSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.control.ControlSuperMouse")
public class ControlSuperMouseImpl extends ControlImpl implements ControlSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    private CablingSuperMouseImpl m_cabling = new CablingSuperMouseImpl();

    /**
     * Construct a Control with super mouse powers.
     */
    public ControlSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public String setParameter(String crateString, String moduleString, String channelString, String parameterName,
            String parameterValue) {
        m_log.debug("setParameter(" + crateString + ", " + moduleString + ", " + channelString + ", " + parameterName
                + ", " + parameterValue + ")");
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;

        try {
            int crate = m_validator.getInt(crateString);
            int module = m_validator.getInt(moduleString);
            int channel = m_validator.getInt(channelString);
            m_validator.validateStringNonBlank(parameterName);
            m_validator.validateStringNonBlank(parameterValue);

            connection = m_mdb.getConnect();

            if (!newEntry(connection, crate, module, channel, parameterName)) {
                m_log.debug("Entry for " + crate + ", " + module + ", " + channel + ", " + parameterName
                        + " already exists");
                return "<error>Entry for " + crate + ", " + module + ", " + channel + ", " + parameterName
                        + " already exists</error>";
            }

            StringBuilder insert = new StringBuilder();
            insert.append("INSERT INTO ").append(ControlConstants.TABLE_CONTROLS).append(" (")
                    .append(ControlConstants.COLUMN_CRATE).append(", ").append(ControlConstants.COLUMN_MODULE)
                    .append(", ").append(ControlConstants.COLUMN_CHANNEL).append(", ")
                    .append(ControlConstants.COLUMN_CREATIONTIME).append(", ")
                    .append(ControlConstants.COLUMN_PARAMETER_NAME).append(", ")
                    .append(ControlConstants.COLUMN_PARAMETER_VALUE).append(") VALUES (").append(crate).append(", ")
                    .append(module).append(", ").append(channel).append(", '").append(creationTime).append("', '")
                    .append(parameterName).append("', '").append(parameterValue).append("');");

            m_mdb.insert(connection, insert.toString());

        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        m_log.info("Parameter set");
        return "<ok>Parameter set</ok>";
    }

    @Override
    public String updateParameter(String crateString, String moduleString, String channelString, String parameterName,
            String parameterValue) {
        m_log.debug("updateParameter(" + crateString + ", " + moduleString + ", " + channelString + ", "
                + parameterName + ", " + parameterValue + ")");
        // set the creationTime to now
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;

        try {
            int crate = m_validator.getInt(crateString);
            int module = m_validator.getInt(moduleString);
            int channel = m_validator.getInt(channelString);
            m_validator.validateStringNonBlank(parameterName);
            m_validator.validateStringNonBlank(parameterValue);

            if (parameterName.equalsIgnoreCase("name")) {
                return m_cabling.updateControl(crateString, moduleString, channelString, parameterValue);
            }

            connection = m_mdb.getConnect();

            if (newEntry(connection, crate, module, channel, parameterName)) {
                m_log.debug("Entry for " + crate + ", " + module + ", " + channel + ", " + parameterName
                        + " does not exist");
                return "<error>Entry for " + crate + ", " + module + ", " + channel + ", " + parameterName
                        + " does not exist</error>";
            }

            // update existing record
            StringBuilder sql = new StringBuilder();
            sql.append("UPDATE ").append(ControlConstants.TABLE_CONTROLS).append(" SET ")
                    .append(ControlConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                    .append("' WHERE ").append(ControlConstants.COLUMN_CRATE).append("=").append(crate).append(" AND ")
                    .append(ControlConstants.COLUMN_MODULE).append("=").append(module).append(" AND ")
                    .append(ControlConstants.COLUMN_CHANNEL).append("=").append(channel).append(" AND ")
                    .append(ControlConstants.COLUMN_PARAMETER_NAME).append("='").append(parameterName).append("' AND ")
                    .append(ControlConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
            m_mdb.insert(connection, sql.toString());

            sql = new StringBuilder();
            sql.append("INSERT INTO ").append(ControlConstants.TABLE_CONTROLS).append(" (")
                    .append(ControlConstants.COLUMN_CRATE).append(", ").append(ControlConstants.COLUMN_MODULE)
                    .append(", ").append(ControlConstants.COLUMN_CHANNEL).append(", ")
                    .append(ControlConstants.COLUMN_CREATIONTIME).append(", ")
                    .append(ControlConstants.COLUMN_PARAMETER_NAME).append(", ")
                    .append(ControlConstants.COLUMN_PARAMETER_VALUE).append(") VALUES (").append(crate).append(", ")
                    .append(module).append(", ").append(channel).append(", '").append(creationTime).append("', '")
                    .append(parameterName).append("', '").append(parameterValue).append("');");
            m_mdb.insert(connection, sql.toString());

        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        m_log.info("Parameter updated");
        return "<ok>Parameter updated</ok>";
    }

    /**
     * Check to see if a record for the crate/module/channel/parameterName
     * exists.
     * 
     * @param connection
     *            db connection
     * @param crate
     *            the number of the crate
     * @param module
     *            the number of the module
     * @param channel
     *            the number of the channel
     * @param parameterName
     *            the name of the parameter
     * @return <code>true</code> if crate/module/channel/parameterName does not
     *         exist
     * @throws CDBException
     */
    private boolean newEntry(Connection connection, int crate, int module, int channel, String parameterName)
            throws CDBException {
        m_log.debug("newEntry(" + crate + ", " + module + ", " + channel + ", " + parameterName + ")");
        // There will only be one row returned
        StringBuilder select = new StringBuilder();
        select.append("Select count(*) FROM ").append(ControlConstants.TABLE_CONTROLS).append(" WHERE ")
                .append(ControlConstants.COLUMN_CRATE).append(" = ").append(crate).append(" AND ")
                .append(ControlConstants.COLUMN_MODULE).append(" = ").append(module).append(" AND ")
                .append(ControlConstants.COLUMN_CHANNEL).append(" = ").append(channel).append(" AND ")
                .append(ControlConstants.COLUMN_PARAMETER_NAME).append(" = '").append(parameterName).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            rs.next();
            if (rs.getInt("count") > 0) {
                m_log.debug("newEntry - false");
                return false;
            } else {
                m_log.debug("newEntry - true");
                return true;
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }
}
