package mice.cdb.service.coolingchannel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.cabling.CablingConstants;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Coolingchannel</code>.
 * 
 * @see mice.cdb.service.coolingchannel.Coolingchannel
 */
@WebService(endpointInterface = "mice.cdb.service.coolingchannel.Coolingchannel")
public class CoolingchannelImpl extends CDB implements Coolingchannel {

    /**
     * Construct a Coolingchannel.
     */
    public CoolingchannelImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public String getAllCoolingchannels() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getAllCoolingchannels()");
        }
        String xml = null;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            xml = selectCoolingchannels(connection, "");
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCoolingchannelForRun(String runNo) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCoolingchannelForRun(" + runNo + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runStartTime = selectRunStartTime(connection, runNumber);
            xml = getCoolingchannelForDate(runStartTime.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCoolingchannelForDate(String timestamp) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCoolingchannelForDate(" + timestamp + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);

            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(CablingConstants.COLUMN_CREATIONTIME).append("<='").append(usersTimestamp)
                    .append("' AND (").append(CablingConstants.COLUMN_VALID_UNTIL_TIME).append(">'")
                    .append(usersTimestamp).append("' OR ").append(CablingConstants.COLUMN_VALID_UNTIL_TIME)
                    .append(" is NULL)");
            xml = selectCoolingchannels(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCoolingchannelForTag(String tag) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCoolingchannelForTag(" + tag + ")");
        }
        String xml = null;
        Connection connection = null;

        try {
            m_validator.validateNonBlank(tag);
            connection = m_mdb.getConnect();
            xml = selectTaggedCoolingchannel(connection, tag);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String listTags() {
        if (m_log.isDebugEnabled()) {
            m_log.debug("listTags()");
        }
        String xml = null;
        Connection connection = null;

        try {
            connection = m_mdb.getConnect();
            xml = selectTags(connection);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    /**
     * Select the values for all cooling channels matching the clause.
     * 
     * @param connection
     *            db connection
     * @param clause
     *            a SQL where clause
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectCoolingchannels(Connection connection, String clause) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCoolingchannels(" + clause + ")");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT ");
        select.append(CoolingchannelConstants.COLUMN_ID).append(", ");
        select.append(CoolingchannelConstants.COLUMN_CREATIONTIME).append(", ");
        select.append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME);
        select.append(" FROM ");
        select.append(CoolingchannelConstants.TABLE_COOLING_CHANNEL);
        if (!clause.isEmpty()) {
            select.append(clause);
        }
        select.append(" ORDER BY ").append(CoolingchannelConstants.COLUMN_ID).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + CoolingchannelConstants.COOLING_CHANNELS + ">");
        try {
            while (rs.next()) {
                // write cooling channel xml
                xml.append("<").append(CoolingchannelConstants.COOLING_CHANNEL).append(" ");
                xml.append(CoolingchannelConstants.VALID_FROM_TIME).append("='")
                        .append(rs.getTimestamp(CoolingchannelConstants.COLUMN_CREATIONTIME)).append("' ");
                xml.append(CoolingchannelConstants.VALID_UNTIL_TIME).append("='")
                        .append(rs.getTimestamp(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME)).append("'");
                xml.append(">");

                // for each cooling channel get the magnets
                xml.append("<" + CoolingchannelConstants.MAGNETS + ">");
                select = new StringBuilder();
                select.append("SELECT ").append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                select.append(CoolingchannelConstants.COLUMN_MODE).append(", ");
                select.append(CoolingchannelConstants.COLUMN_POLARITY);
                select.append(" FROM ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_MAGNET);
                select.append(" WHERE ");
                select.append(CoolingchannelConstants.COLUMN_COOLING_CHANNEL_ID).append(" = ")
                        .append(rs.getString(CoolingchannelConstants.COLUMN_ID)).append(";");
                ResultSet magnetRs = m_mdb.query(connection, select.toString());

                while (magnetRs.next()) {
                    // write magnet xml
                    xml.append("<").append(CoolingchannelConstants.MAGNET).append(" ");
                    xml.append(CoolingchannelConstants.NAME).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("' ");
                    xml.append(CoolingchannelConstants.MODE).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_MODE)).append("' ");
                    xml.append(CoolingchannelConstants.POLARITY).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_POLARITY)).append("'>");

                    // for each magnet get the coils
                    select = new StringBuilder();
                    select.append("SELECT ").append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_CALIBRATION).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_ILIM).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_ISET).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_RATE).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_STABILITY).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_VLIM);
                    select.append(" FROM ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_COIL);
                    select.append(" WHERE ");
                    select.append(CoolingchannelConstants.COLUMN_COOLING_CHANNEL_ID).append(" = ")
                            .append(rs.getString(CoolingchannelConstants.COLUMN_ID));
                    select.append(" AND ");
                    select.append(CoolingchannelConstants.COLUMN_MAGNET_NAME).append(" = '")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("';");
                    ResultSet coilRs = m_mdb.query(connection, select.toString());
                    while (coilRs.next()) {
                        // write coil xml
                        xml.append("<" + CoolingchannelConstants.COIL + " ");
                        xml.append(CoolingchannelConstants.NAME).append("='")
                                .append(coilRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("' ");
                        xml.append(CoolingchannelConstants.CALIBRATION).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_CALIBRATION)).append("' ");
                        xml.append(CoolingchannelConstants.ILIM).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_ILIM)).append("' ");
                        xml.append(CoolingchannelConstants.ISET).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_ISET)).append("' ");
                        xml.append(CoolingchannelConstants.RATE).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_RATE)).append("' ");
                        xml.append(CoolingchannelConstants.STABILITY).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_STABILITY)).append("' ");
                        xml.append(CoolingchannelConstants.VLIM).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_VLIM)).append("'/>");
                    }
                    xml.append("</").append(CoolingchannelConstants.MAGNET).append(">");
                }
                xml.append("</").append(CoolingchannelConstants.MAGNETS).append(">");
                xml.append("</").append(CoolingchannelConstants.COOLING_CHANNEL).append(">");

            }

        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        xml.append("</").append(CoolingchannelConstants.COOLING_CHANNELS).append(">");

        return xml.toString();
    }

    /**
     * Select the values for the cooling channel matching the tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of a tag
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTaggedCoolingchannel(Connection connection, String tag) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectTaggedCoolingchannel(" + tag + ")");
        }
        int id = getTagId(connection, tag);
        StringBuilder select = new StringBuilder();

        select.append("SELECT ");
        select.append(CoolingchannelConstants.COLUMN_ID).append(", ");
        select.append(CoolingchannelConstants.COLUMN_CREATIONTIME).append(", ");
        select.append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME);
        select.append(" FROM ");
        select.append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_TAG);
        select.append(" WHERE ").append(CoolingchannelConstants.COLUMN_ID).append(" = ").append(id).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + CoolingchannelConstants.COOLING_CHANNELS + ">");
        try {
            while (rs.next()) {
                // write cooling channel xml
                xml.append("<").append(CoolingchannelConstants.COOLING_CHANNEL).append(" ");
                xml.append(CoolingchannelConstants.VALID_FROM_TIME).append("='")
                        .append(rs.getTimestamp(CoolingchannelConstants.COLUMN_CREATIONTIME)).append("' ");
                xml.append(CoolingchannelConstants.VALID_UNTIL_TIME).append("='")
                        .append(rs.getTimestamp(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME)).append("'");
                xml.append(">");

                // for each cooling channel get the magnets
                xml.append("<" + CoolingchannelConstants.MAGNETS + ">");
                select = new StringBuilder();
                select.append("SELECT ").append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                select.append(CoolingchannelConstants.COLUMN_MODE).append(", ");
                select.append(CoolingchannelConstants.COLUMN_POLARITY);
                select.append(" FROM ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_MAGNET_TAGGED);
                select.append(" WHERE ");
                select.append(CoolingchannelConstants.COLUMN_TAG_ID).append(" = ")
                        .append(rs.getString(CoolingchannelConstants.COLUMN_ID)).append(";");
                ResultSet magnetRs = m_mdb.query(connection, select.toString());

                while (magnetRs.next()) {
                    // write magnet xml
                    xml.append("<").append(CoolingchannelConstants.MAGNET).append(" ");
                    xml.append(CoolingchannelConstants.NAME).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("' ");
                    xml.append(CoolingchannelConstants.MODE).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_MODE)).append("' ");
                    xml.append(CoolingchannelConstants.POLARITY).append("='")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_POLARITY)).append("'>");

                    // for each magnet get the coils
                    select = new StringBuilder();
                    select.append("SELECT ").append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_CALIBRATION).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_ILIM).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_ISET).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_RATE).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_STABILITY).append(", ");
                    select.append(CoolingchannelConstants.COLUMN_VLIM);
                    select.append(" FROM ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_COIL_TAGGED);
                    select.append(" WHERE ");
                    select.append(CoolingchannelConstants.COLUMN_TAG_ID).append(" = ")
                            .append(rs.getString(CoolingchannelConstants.COLUMN_ID));
                    select.append(" AND ");
                    select.append(CoolingchannelConstants.COLUMN_MAGNET_NAME).append(" = '")
                            .append(magnetRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("';");
                    ResultSet coilRs = m_mdb.query(connection, select.toString());
                    while (coilRs.next()) {
                        // write coil xml
                        xml.append("<" + CoolingchannelConstants.COIL + " ");
                        xml.append(CoolingchannelConstants.NAME).append("='")
                                .append(coilRs.getString(CoolingchannelConstants.COLUMN_NAME)).append("' ");
                        xml.append(CoolingchannelConstants.CALIBRATION).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_CALIBRATION)).append("' ");
                        xml.append(CoolingchannelConstants.ILIM).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_ILIM)).append("' ");
                        xml.append(CoolingchannelConstants.ISET).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_ISET)).append("' ");
                        xml.append(CoolingchannelConstants.RATE).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_RATE)).append("' ");
                        xml.append(CoolingchannelConstants.STABILITY).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_STABILITY)).append("' ");
                        xml.append(CoolingchannelConstants.VLIM).append("='")
                                .append(coilRs.getDouble(CoolingchannelConstants.COLUMN_VLIM)).append("'/>");
                    }
                    xml.append("</").append(CoolingchannelConstants.MAGNET).append(">");
                }
                xml.append("</").append(CoolingchannelConstants.MAGNETS).append(">");
                xml.append("</").append(CoolingchannelConstants.COOLING_CHANNEL).append(">");

            }

        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }

        xml.append("</").append(CoolingchannelConstants.COOLING_CHANNELS).append(">");

        return xml.toString();
    }

    /**
     * Select the list of available tag names.
     * 
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectTags(Connection connection) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectTags()");
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT DISTINCT ").append(CoolingchannelConstants.COLUMN_TAG).append(" FROM ")
                .append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_TAG).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<" + CoolingchannelConstants.TAG_NAMES + ">");
        try {
            while (rs.next()) {

                xml.append("<").append(CoolingchannelConstants.TAG).append(" ");
                xml.append(CoolingchannelConstants.NAME).append("='")
                        .append(rs.getString(CoolingchannelConstants.COLUMN_TAG)).append("' ");
                xml.append("/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml.append("</" + CoolingchannelConstants.TAG_NAMES + ">");
        return xml.toString();
    }

    /**
     * Get the id of the latest cooling channel.
     * 
     * @param connection
     *            db connection
     * @return id of the cooling channel
     * @throws CDBException
     */
    protected int getCCId(Connection connection) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCCId(connection)");
        }
        // There will only be one row returned
        StringBuilder select = new StringBuilder();
        select.append("SELECT MAX(").append(CoolingchannelConstants.COLUMN_ID).append(") FROM ");
        select.append(CoolingchannelConstants.TABLE_COOLING_CHANNEL).append(";");

        ResultSet rs = m_mdb.query(connection, select.toString());
        int id = 0;
        try {
            while (rs.next()) {
                id = rs.getInt("max");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        if (id == 0) {
            throw new CDBException("ID not found.");
        }
        return id;
    }

    /**
     * Get the id of the latest version of the given tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of a tag
     * @return id of the tag
     * @throws CDBException
     */
    protected int getTagId(Connection connection, String tag) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getTagId(connection" + tag + ")");
        }
        // There will only be one row returned
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(CoolingchannelConstants.COLUMN_ID).append(" FROM ")
                .append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_TAG).append(" WHERE ")
                .append(CoolingchannelConstants.COLUMN_TAG).append(" = '").append(tag).append("' AND ")
                .append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME).append(" is Null;");

        ResultSet rs = m_mdb.query(connection, select.toString());
        int id = 0;
        try {
            while (rs.next()) {
                id = rs.getInt(CoolingchannelConstants.COLUMN_ID);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        if (id == 0) {
            throw new CDBException("Tag " + tag + " not found.");
        }
        return id;
    }

}
