package mice.cdb.service.coolingchannel;

import java.sql.Connection;
import java.sql.Timestamp;
import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;
import mice.cdb.service.parser.CoolingchannelParser;
import mice.cdb.service.parser.ParseException;

/**
 * An implementation of the <code>CoolingchannelSuperMouse</code>.
 * 
 * @see mice.cdb.service.coolingchannel.CoolingchannelSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.coolingchannel.CoolingchannelSuperMouse")
public class CoolingchannelSuperMouseImpl extends CoolingchannelImpl implements CoolingchannelSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Constructs a Coolingchannel object with super mouse powers.
     */
    public CoolingchannelSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String setCoolingchannel(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setCoolingchannel(" + xml + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        CoolingchannelData ccData = null;
        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            CoolingchannelParser ccParser = new CoolingchannelParser();
            try {
                ccParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Coolingchannel XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            ccData = ccParser.getData();
            int id = updateCoolingchannel(connection, creationTime);
            writeCoolingchannel(connection, id, ccData);

        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return "<ok>Coolingchannel data added</ok>";
    }

    @Override
    public synchronized String setCoolingchannelTag(String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setCoolingchannelTag(" + xml + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        CoolingchannelData ccData = null;

        try {
            m_validator.validateNonBlank(xml);
            connection = m_mdb.getConnect();

            // parse the XML
            CoolingchannelParser ccParser = new CoolingchannelParser();
            try {
                ccParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Coolingchannel XML. Please check input. " + e.getMessage() + "</error>";
            }

            // insert the data
            ccData = ccParser.getData();
            int id = updateCoolingchannelTag(connection, ccData.getTag(), creationTime);
            writeTaggedCoolingchannel(connection, id, ccData);

        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        return "<ok>Coolingchannel tag " + ccData.getTag() + " set</ok>";
    }

    /**
     * Write out the magnet and coil data.
     * 
     * @param connection
     *            db connection
     * @param id
     *            the id of the cooling channel
     * @param ccData
     *            the cooling channel data
     * @throws CDBException
     */
    private void writeCoolingchannel(Connection connection, int id, CoolingchannelData ccData) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("writeCoolingchannel(" + id + ")");
        }

        StringBuilder sql = new StringBuilder();

        for (MagnetData magnet : ccData.getMagnets()) {
            sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_MAGNET).append(" (");
            sql.append(CoolingchannelConstants.COLUMN_COOLING_CHANNEL_ID).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_NAME).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_MODE).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_POLARITY).append(") VALUES (");
            sql.append(id).append(", '");
            sql.append(magnet.getName()).append("', '");
            sql.append(magnet.getMode()).append("', '");
            sql.append(magnet.getPolarity()).append("'); ");

            for (CoilData coil : magnet.getCoils()) {
                sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_COIL).append(" (");
                sql.append(CoolingchannelConstants.COLUMN_COOLING_CHANNEL_ID).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_MAGNET_NAME).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_CALIBRATION).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_ILIM).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_ISET).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_RATE).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_STABILITY).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_VLIM).append(") VALUES (");
                sql.append(id).append(", '");
                sql.append(magnet.getName()).append("', '");
                sql.append(coil.getName()).append("', ");
                sql.append(coil.getCalibration()).append(", ");
                sql.append(coil.getILim()).append(", ");
                sql.append(coil.getISet()).append(", ");
                sql.append(coil.getRate()).append(", ");
                sql.append(coil.getStability()).append(", ");
                sql.append(coil.getVLim()).append("); ");
            }
        }
        m_mdb.batchInsert(connection, sql.toString());
    }

    /**
     * Write a record to the cooling channel and get its id.
     * 
     * @param connection
     *            db connection
     * @param creationTime
     *            the creation time of the tag
     * @return the new id
     * @throws CDBException
     */
    private int updateCoolingchannel(Connection connection, Timestamp creationTime) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("updateCoolingchannel(" + creationTime + ")");
        }
        StringBuilder sql = new StringBuilder();
        sql.append("UPDATE ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL);
        sql.append(" SET ").append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime);
        sql.append("' WHERE ").append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        try {
            m_mdb.insert(connection, sql.toString());
        } catch (CDBException e) {
            // no record to update
        }
        sql = new StringBuilder();
        sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL).append(" (");
        sql.append(CoolingchannelConstants.COLUMN_CREATIONTIME);
        sql.append(") VALUES ('").append(creationTime).append("'); ");
        m_mdb.insert(connection, sql.toString());
        return getCCId(connection);
    }

    /**
     * Get a new id for the given tag.
     * 
     * @param connection
     *            db connection
     * @param tag
     *            the name of the tag
     * @param creationTime
     *            the creation time of the tag
     * @return the new id
     * @throws CDBException
     */
    private int updateCoolingchannelTag(Connection connection, String tag, Timestamp creationTime) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("updateCoolingchannelTag(" + tag + ", " + creationTime + ")");
        }
        StringBuilder sql = new StringBuilder();
        sql.append("UPDATE ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_TAG).append(" SET ")
                .append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                .append("' WHERE ").append(CoolingchannelConstants.COLUMN_TAG).append("='").append(tag)
                .append("' AND ").append(CoolingchannelConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        try {
            m_mdb.insert(connection, sql.toString());
        } catch (CDBException e) {
        }
        sql = new StringBuilder();
        sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_TAG).append(" (")
                .append(CoolingchannelConstants.COLUMN_TAG).append(", ")
                .append(CoolingchannelConstants.COLUMN_CREATIONTIME).append(") VALUES ('").append(tag).append("', '")
                .append(creationTime).append("'); ");
        m_mdb.insert(connection, sql.toString());
        return getTagId(connection, tag);
    }

    /**
     * Write the tagged data.
     * 
     * @param connection
     *            db connection
     * @param id
     *            the id of the tag
     * @param ccData
     *            the cooling channel data
     * @throws CDBException
     */
    private void writeTaggedCoolingchannel(Connection connection, int id, CoolingchannelData ccData)
            throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("writeTaggedCoolingchannel(" + id + ")");
        }
        StringBuilder sql = new StringBuilder();

        for (MagnetData magnet : ccData.getMagnets()) {
            sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_MAGNET_TAGGED).append(" (");
            sql.append(CoolingchannelConstants.COLUMN_TAG_ID).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_NAME).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_MODE).append(", ");
            sql.append(CoolingchannelConstants.COLUMN_POLARITY).append(") VALUES (");
            sql.append(id).append(", '");
            sql.append(magnet.getName()).append("', '");
            sql.append(magnet.getMode()).append("', '");
            sql.append(magnet.getPolarity()).append("'); ");

            for (CoilData coil : magnet.getCoils()) {
                sql.append("INSERT INTO ").append(CoolingchannelConstants.TABLE_COOLING_CHANNEL_COIL_TAGGED)
                        .append(" (");
                sql.append(CoolingchannelConstants.COLUMN_TAG_ID).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_MAGNET_NAME).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_NAME).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_CALIBRATION).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_ILIM).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_ISET).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_RATE).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_STABILITY).append(", ");
                sql.append(CoolingchannelConstants.COLUMN_VLIM).append(") VALUES (");
                sql.append(id).append(", '");
                sql.append(magnet.getName()).append("', '");
                sql.append(coil.getName()).append("', ");
                sql.append(coil.getCalibration()).append(", ");
                sql.append(coil.getILim()).append(", ");
                sql.append(coil.getISet()).append(", ");
                sql.append(coil.getRate()).append(", ");
                sql.append(coil.getStability()).append(", ");
                sql.append(coil.getVLim()).append("); ");
            }
        }
        m_mdb.batchInsert(connection, sql.toString());
    }

}
