package mice.cdb.service.geometry;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>Geometry</code> interface is used to retrieve a GDML data set. To
 * store GDML data sets use the {@link GeometrySuperMouse} interface. Previous
 * versions of the GDML data sets are also stored. The current GDML data set can
 * be retrieved using {@link #getCurrentGDML() getCurrentGDML}. The GDML data
 * set associated with a run can be retrieved using
 * {@link #getGDMLForRun(String) getGDMLForRun}. Each GDML data set has a unique
 * id. A list of ids for data sets that were valid at a given time or over a
 * time range can be retrieved using {@link #getIds(String, String) getIds} and
 * the GDML data set associated with an id can be retrieved using
 * {@link #getGDMLForId(String) getGDMLForId}. The time a GDML data set was
 * valid from is defined by the user when storing the GDML data set.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated byte array the returned
 * byte array will contain a XML message enclosed in one of two tags:
 * <code>warning</code> or <code>error</code>. A tag of <code>warning</code>
 * indicates that the problem maybe transient and retrying the request MAY
 * succeed. A tag of <code>error</code> maybe due to to bad data being passed in
 * or an unexpected internal error.
 * </p>
 */
@WebService
public interface Geometry {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the current GDML data set. The current GDML is the one with the
     * latest validFromTime. If there are more than one GDML with the the same
     * latest validFromTime then the one with the latest creation time is
     * returned.
     * 
     * @return a byte array containing the GDML data set
     * 
     * @see mice.cdb.service.geometry.GeometrySuperMouse#setGDML(byte[], String,
     *      String, String)
     */
    @WebMethod
    public abstract byte[] getCurrentGDML();

    /**
     * Get the GDML data set for the given id.
     * 
     * @param id
     *            the id of the GDML
     * @return a byte array containing the GDML data set
     * 
     * @see #getIds(String, String)
     * @see mice.cdb.service.geometry.GeometrySuperMouse#setGDML(byte[], String,
     *      String, String)
     */
    @WebMethod
    public abstract byte[] getGDMLForId(String id);

    /**
     * Get the GDML data set that was valid when the run represented by the run
     * number occurred. If more than one GDML data set was valid then the one
     * with the latest creation time is returned.
     * 
     * @param runNo
     *            the number of a run
     * @return a byte array containing the GDML data set
     */
    @WebMethod
    public abstract byte[] getGDMLForRun(String runNo);

    /**
     * Return a list of ids of the GDML data sets that were valid during the
     * given time period. <b>N.B.</b> if the stopTime is <code>null</code> then
     * only the ids that were valid at the startTime will be returned.
     * 
     * @param startTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * @param stopTime
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC.
     *            May be <code>null</code>.
     * @return a string containing XML representing the selected data sets with
     *         the format of the XML as
     *         <dl>
     *         <dd><code>&lt;ids&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='1' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' notes='xxx' technicalDrawingName='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='5' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' notes='xxx' technicalDrawingName='xxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;id name='3' validFrom='yyyy-mm-dd hh:mm:ss[.fffffffff]' created='yyyy-mm-dd hh:mm:ss[.fffffffff]' notes='xxx' technicalDrawingName='xxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/ids&gt;</code></dd>
     *         <dl>
     * 
     * @see mice.cdb.service.geometry.GeometrySuperMouse#setGDML(byte[], String,
     *      String, String)
     */
    @WebMethod
    public abstract String getIds(String startTime, String stopTime);

}
