package mice.cdb.service.geometry;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Geometry</code>.
 * 
 * @see mice.cdb.service.geometry.Geometry
 */
@WebService(endpointInterface = "mice.cdb.service.geometry.Geometry")
public class GeometryImpl extends CDB implements Geometry {

    /**
     * Construct a Geometry.
     */
    public GeometryImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public byte[] getCurrentGDML() {
        byte[] gdml = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            gdml = selectCurrentGdml(connection);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return gdml;
    }

    @Override
    public byte[] getGDMLForId(String id) {
        byte[] gdml = null;
        Connection connection = null;
        try {
            int numericId = m_validator.getInt(id);
            connection = m_mdb.getConnect();
            gdml = selectGdml(connection, numericId);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return gdml;
    }

    @Override
    public byte[] getGDMLForRun(String runNo) {
        byte[] gdml = null;
        Connection connection = null;
        try {
            m_validator.validateStringNonBlank(runNo);
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            gdml = selectRun(connection, runNumber);
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message.getBytes();
        } finally {
            m_mdb.close(connection);
        }
        return gdml;
    }

    @Override
    public String getIds(String startTime, String stopTime) {
        String gdml = null;
        Connection connection = null;
        try {
            Timestamp start = m_validator.getTimeStamp(startTime);
            connection = m_mdb.getConnect();
            if (stopTime == null) {
                gdml = selectIds(connection, start);
            } else {
                Timestamp stop = m_validator.getTimeStamp(stopTime);
                if (start.after(stop)) {
                    String message = "<error>Start time must be before stop time.</error>";
                    return message;
                }
                gdml = selectIds(connection, start, stop);
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return gdml;
    }

    /**
     * Select the current GDML entry.
     * 
     * @param connection
     *            db connection
     * @return a byte array containing GDML
     * @throws CDBException
     */
    private byte[] selectCurrentGdml(Connection connection) throws CDBException {
        m_log.debug("selectCurrentGdml");
        Timestamp now = new Timestamp(System.currentTimeMillis());
        // There will only be one row returned
        String maxValidFromTime = "SELECT max(" + GeometryConstants.COLUMN_VALIDFROMTIME + ") FROM "
                + GeometryConstants.TABLE_GEOMETRY + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " <= '" + now
                + "'";
        String maxCreationTime = "SELECT max(" + GeometryConstants.COLUMN_CREATIONTIME + ") FROM "
                + GeometryConstants.TABLE_GEOMETRY + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " = ("
                + maxValidFromTime + ")";
        String select = "SELECT " + GeometryConstants.COLUMN_GDML + " FROM " + GeometryConstants.TABLE_GEOMETRY
                + " WHERE " + GeometryConstants.COLUMN_CREATIONTIME + " = (" + maxCreationTime + ") AND "
                + GeometryConstants.COLUMN_VALIDFROMTIME + " = (" + maxValidFromTime + ");";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            if (rs.next()) {
                return rs.getBytes(GeometryConstants.COLUMN_GDML);
            } else {
                m_log.debug("No data returned from the database.\n\t" + select);
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select the GDML for the given id.
     * 
     * @param connection
     *            db connection
     * @param id
     *            the id of the GDML
     * @return a byte array containing GDML
     * @throws CDBException
     */
    private byte[] selectGdml(Connection connection, int id) throws CDBException {
        m_log.debug("selectGdml id:" + id);
        // There will only be one row returned
        String select = "SELECT " + GeometryConstants.COLUMN_GDML + " FROM " + GeometryConstants.TABLE_GEOMETRY
                + " WHERE " + GeometryConstants.COLUMN_ID + " = " + id + ";";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            if (rs.next()) {
                return rs.getBytes(GeometryConstants.COLUMN_GDML);
            } else {
                m_log.debug("No GDML found for id:" + id);
                throw new CDBException("No GDML found for id:" + id);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select the GDML for the given run number.
     * 
     * @param connection
     *            db connection
     * @param runNo
     *            the run number
     * @return a byte array containing GDML
     * @throws CDBException
     */
    private byte[] selectRun(Connection connection, int runNo) throws CDBException {
        m_log.debug("selectRun runNo:" + runNo);
        // There will only be one row returned
        Timestamp runTime = selectRunStartTime(connection, runNo);
        String maxCreationTime = "SELECT max(" + GeometryConstants.COLUMN_CREATIONTIME + ") FROM "
                + GeometryConstants.TABLE_GEOMETRY + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " <= ('"
                + runTime + "')";
        String select = "SELECT " + GeometryConstants.COLUMN_GDML + " FROM " + GeometryConstants.TABLE_GEOMETRY
                + " WHERE " + GeometryConstants.COLUMN_CREATIONTIME + " = (" + maxCreationTime + ") AND "
                + GeometryConstants.COLUMN_VALIDFROMTIME + " <= ('" + runTime + "');";

        ResultSet rs = m_mdb.query(connection, select);
        try {
            if (rs.next()) {
                return rs.getBytes(GeometryConstants.COLUMN_GDML);
            } else {
                m_log.debug("No GDML found for run:" + runNo);
                throw new CDBException("No GDML found for run:" + runNo);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select all ids that would have been valid at the given time.
     * 
     * @param connection
     *            db connection
     * @param time
     *            the time of interest
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp time) throws CDBException {
        m_log.debug("selectIds startTime:" + time);
        String maxValidFromTime = "SELECT max(" + GeometryConstants.COLUMN_VALIDFROMTIME + ") FROM "
                + GeometryConstants.TABLE_GEOMETRY + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " <= '"
                + time + "'";
        String select = "SELECT " + GeometryConstants.COLUMN_ID + ", " + GeometryConstants.COLUMN_VALIDFROMTIME + ", "
                + GeometryConstants.COLUMN_CREATIONTIME + ", " + GeometryConstants.COLUMN_NOTES + ", "
                + GeometryConstants.COLUMN_TECHNICAL_DRAWING_NAME + " FROM " + GeometryConstants.TABLE_GEOMETRY
                + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " = (" + maxValidFromTime + ") ORDER BY "
                + GeometryConstants.COLUMN_CREATIONTIME + ";";
        return selectIds(connection, select);
    }

    /**
     * Select all ids that would have been valid during the given time range.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @param stopTime
     *            the end of the time range
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, Timestamp startTime, Timestamp stopTime) throws CDBException {
        m_log.debug("selectIds startTime:" + startTime + " stopTime:" + stopTime);
        String startValidFromTime = selectStartValidFromTime(connection, startTime);
        String select = "SELECT " + GeometryConstants.COLUMN_ID + ", " + GeometryConstants.COLUMN_VALIDFROMTIME + ", "
                + GeometryConstants.COLUMN_CREATIONTIME + ", " + GeometryConstants.COLUMN_NOTES + ", "
                + GeometryConstants.COLUMN_TECHNICAL_DRAWING_NAME + " FROM " + GeometryConstants.TABLE_GEOMETRY
                + " WHERE " + startValidFromTime + GeometryConstants.COLUMN_VALIDFROMTIME + " <= '" + stopTime
                + "' ORDER BY " + GeometryConstants.COLUMN_VALIDFROMTIME + ", " + GeometryConstants.COLUMN_CREATIONTIME
                + ";";

        return selectIds(connection, select);
    }

    /**
     * Get an SQL expression that includes the ValidFromTime that immediately
     * precedes the given time. If the given time is before the earliest time
     * then use the earliest ValidFromTime.
     * 
     * @param connection
     *            db connection
     * @param startTime
     *            the start of the time range
     * @return a string containing an SQL snippet
     * @throws CDBException
     */
    private String selectStartValidFromTime(Connection connection, Timestamp startTime) throws CDBException {
        m_log.debug("selectStartValidFromTime startTime:" + startTime);
        String select = "SELECT max(" + GeometryConstants.COLUMN_VALIDFROMTIME + ") FROM "
                + GeometryConstants.TABLE_GEOMETRY + " WHERE " + GeometryConstants.COLUMN_VALIDFROMTIME + " <= '"
                + startTime + "'";
        ResultSet rs = m_mdb.query(connection, select);
        try {
            rs.next();
            Timestamp startValidFromTime = rs.getTimestamp("max");
            if (startValidFromTime == null) {
                return "";
            } else {
                return GeometryConstants.COLUMN_VALIDFROMTIME + " >= '" + startValidFromTime + "' AND ";
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Select ids
     * 
     * @param connection
     *            db connection
     * @param select
     *            a string containing the select statement.
     * @return a string containing xml
     * @throws CDBException
     */
    private String selectIds(Connection connection, String select) throws CDBException {
        m_log.debug("selectIds select:" + select);
        StringBuilder xml = new StringBuilder();
        xml = xml.append("<ids>");
        ResultSet rs = m_mdb.query(connection, select);
        try {
            while (rs.next()) {
                xml = xml.append("<id name='").append(rs.getString(GeometryConstants.COLUMN_ID))
                        .append("' validFrom='").append(rs.getString(GeometryConstants.COLUMN_VALIDFROMTIME))
                        .append("' created='").append(rs.getString(GeometryConstants.COLUMN_CREATIONTIME))
                        .append("' notes='").append(rs.getString(GeometryConstants.COLUMN_NOTES))
                        .append("' technicalDrawingName='")
                        .append(rs.getString(GeometryConstants.COLUMN_TECHNICAL_DRAWING_NAME)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml = xml.append("</ids>");
        return xml.toString();
    }

}
