package mice.cdb.service.parser;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import mice.cdb.service.beamline.BeamlineConstants;
import mice.cdb.service.beamline.BeamlineData;
import mice.cdb.service.beamline.IsisData;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

public class BeamlineParser extends Parser {

    /**
     * Encapsulates the properties extracted from the xml message.
     */
    private BeamlineData m_beamlineData = new BeamlineData();
    private List<Map<String, String>> m_magnets = new LinkedList<Map<String, String>>();

    /**
     * Default constructor
     */
    public BeamlineParser() {
    }

    /**
     * Get the parsed XML.
     * 
     * @return the parsed XML as a BeamlineData
     */
    public BeamlineData getData() {
        return m_beamlineData;
    }

    /**
     * Get the parsed XML.
     * 
     * @return the parsed XML as a map
     */
    public List<Map<String, String>> getMagnets() {
        return m_magnets;
    }

    /**
     * Method called by the sax parser when a new element is found.
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attr) throws SAXException {
        m_log.debug("Start of " + qName);
        if (qName.equalsIgnoreCase(BeamlineConstants.START_RUN)) {
            // START RUN
            m_beamlineData = new BeamlineData();
            m_beamlineData.setRunNumber(parseInt(attr.getValue(BeamlineConstants.RUNNUMBER)));
            m_beamlineData.setStartTime(parseTimeStamp(attr.getValue(BeamlineConstants.STARTTIME)));
            m_beamlineData.setNotes(validateString(attr.getValue(BeamlineConstants.START_NOTES)));
            m_beamlineData.setOptics(validateString(attr.getValue(BeamlineConstants.OPTICS)));
            m_beamlineData.setPulseStart(parseInt(attr.getValue(BeamlineConstants.START_PULSE)));
            m_beamlineData.setStep(parseDouble(attr.getValue(BeamlineConstants.STEP)));
            m_beamlineData.setRunType(validateString(attr.getValue(BeamlineConstants.RUN_TYPE)));
            m_beamlineData.setDaqTrigger(validateString(attr.getValue(BeamlineConstants.TRIGGER)));
            m_beamlineData.setDaqGateWidth(parseDouble(attr.getValue(BeamlineConstants.GATE_WIDTH)));
            m_beamlineData.setDaqVersion(attr.getValue(BeamlineConstants.VERSION));
            m_beamlineData.setGdcHostName(validateString(attr.getValue(BeamlineConstants.GDC_HOST_NAME)));

            if (attr.getValue(BeamlineConstants.DELAY) != null) {
                m_beamlineData.setTargetDelay(parseDouble(attr.getValue(BeamlineConstants.DELAY)));
            }
            if (attr.getValue(BeamlineConstants.DEPTH) != null) {
                m_beamlineData.setTargetDepth(parseDouble(attr.getValue(BeamlineConstants.DEPTH)));
            }
            if (attr.getValue(BeamlineConstants.DRIVE_VOLTAGE) != null) {
                m_beamlineData.setTargetDriveVoltage(parseDouble(attr.getValue(BeamlineConstants.DRIVE_VOLTAGE)));
            }
            // overwrite defaults to false
            m_beamlineData.setOverwrite(parseBoolean(attr.getValue(BeamlineConstants.OVERWRITE), false));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.LDC_HOST)) {
            // LDC HOST, part of start run
            m_beamlineData.addLdcHost(validateString(attr.getValue(BeamlineConstants.NAME)));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.END_RUN)) {
            // END RUN
            m_beamlineData = new BeamlineData();
            m_beamlineData.setRunNumber(parseInt(attr.getValue(BeamlineConstants.RUNNUMBER)));
            m_beamlineData.setEndTime(parseTimeStamp(attr.getValue(BeamlineConstants.ENDTIME)));
            m_beamlineData.setPulseEnd(parseInt(attr.getValue(BeamlineConstants.END_PULSE)));
            m_beamlineData.setStatus(parseBoolean(attr.getValue(BeamlineConstants.STATUS)));
            m_beamlineData.setEndNotes(validateString(attr.getValue(BeamlineConstants.END_NOTES)));
            // overwrite defaults to false
            m_beamlineData.setOverwrite(parseBoolean(attr.getValue(BeamlineConstants.OVERWRITE), false));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.SCALAR)) {
            // SCALAR
            // this is part of the end run
            m_beamlineData.addScalar(validateString(attr.getValue(BeamlineConstants.SCALAR_NAME)),
                    parseInt(attr.getValue(BeamlineConstants.SCALAR_VALUE)));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.ISIS_BEAM)) {
            // ISIS BEAM
            // this is part of the end run
            m_beamlineData.addIsisBeam(
                    validateString(attr.getValue(BeamlineConstants.ISIS_BEAM_NAME)),
                    new IsisData(parseDouble(attr.getValue(BeamlineConstants.MEAN)), parseDouble(attr
                            .getValue(BeamlineConstants.SIGMA))));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.BEAMLINE)) {
            // BEAMLINE
            m_beamlineData = new BeamlineData();
            m_beamlineData.setRunNumber(parseInt(attr.getValue(BeamlineConstants.RUNNUMBER)));
            m_beamlineData.setBeamStop(parseBoolean(attr.getValue(BeamlineConstants.BEAM_STOP)));
            m_beamlineData.setDiffuserThickness(parseInt(attr.getValue(BeamlineConstants.DIFFUSER_THICKNESS)));
            m_beamlineData.setProtonAbsorberThickness(parseInt(attr
                    .getValue(BeamlineConstants.PROTON_ABSORBOR_THICKNESS)));
            // overwrite defaults to false
            m_beamlineData.setOverwrite(parseBoolean(attr.getValue(BeamlineConstants.OVERWRITE), false));

        } else if (qName.equalsIgnoreCase(BeamlineConstants.MAGNET)) {
            // MAGNET
            Map<String, String> magnet = new HashMap<String, String>();
            magnet.put(BeamlineConstants.NAME, validateString(attr.getValue(BeamlineConstants.NAME)));
            magnet.put(BeamlineConstants.SET_CURRENT, parseDouble(attr.getValue(BeamlineConstants.SET_CURRENT))
                    .toString());
            magnet.put(BeamlineConstants.POLARITY, validateString(attr.getValue(BeamlineConstants.POLARITY)));
            m_magnets.add(magnet);

        } else if (qName.equalsIgnoreCase(BeamlineConstants.TAG)) {
            // TAG
            m_beamlineData = new BeamlineData();
            m_beamlineData.setTag(validateStringNonBlank(attr.getValue(BeamlineConstants.NAME)));
            m_beamlineData.setBeamStop(parseBoolean(attr.getValue(BeamlineConstants.BEAM_STOP)));
            m_beamlineData.setDiffuserThickness(parseInt(attr.getValue(BeamlineConstants.DIFFUSER_THICKNESS)));
            m_beamlineData.setProtonAbsorberThickness(parseInt(attr
                    .getValue(BeamlineConstants.PROTON_ABSORBOR_THICKNESS)));
        }
    }
}
