package mice.cdb.service.parser;

import java.io.IOException;
import java.io.StringReader;
import java.sql.Timestamp;

import mice.cdb.service.CDBException;
import mice.cdb.service.Validator;

import org.apache.log4j.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

public abstract class Parser extends DefaultHandler {

    /** Reference to logging tool. */
    protected Logger m_log;

    /** Reference to validator tool. */
    protected Validator m_validator = new Validator();

    public Parser() {
        m_log = Logger.getLogger(getClass().getName());
    }

    public void parse(String msg) throws ParseException {
        try {
            XMLReader xmlReader = XMLReaderFactory.createXMLReader();
            xmlReader.setContentHandler(this);
            xmlReader.setErrorHandler(this);

            try {
                xmlReader.parse(new InputSource(new StringReader(msg)));
            } catch (IOException e) {
                m_log.error("ERROR reading XLM in parser. " + e.getMessage());
                throw new ParseException(e.getMessage());
            }

        } catch (SAXException e) {
            m_log.info("SAX error. " + e.getMessage());
            throw new ParseException(e.getMessage());
        }

        m_log.info("XML successfully parsed");
    }

    /**
     * Called when the SAX parser encounters an error.
     */
    @Override
    public void error(SAXParseException e) {
        m_log.info("SAX error. " + e.getMessage());
    }

    /**
     * Called when the SAX parser encounters a 'serious' error.
     */
    @Override
    public void fatalError(SAXParseException e) {
        m_log.info("SAX error. " + e.getMessage());
    }

    /**
     * Use the Validator to validate the string.
     * 
     * @param s
     * @return
     * @throws SAXException
     */
    protected String validateString(String s) throws SAXException {
        try {
            m_validator.validateString(s);
        } catch (CDBException e) {
            throw new SAXException(e);
        }
        return s;
    }

    /**
     * Use the Validator to validate the string.
     * 
     * @param s
     * @return
     * @throws SAXException
     */
    protected String validateStringNonBlank(String s) throws SAXException {
        try {
            m_validator.validateStringNonBlank(s);
        } catch (CDBException e) {
            throw new SAXException(e);
        }
        return s;
    }

    /**
     * Use the Validator to return the time stamp value.
     * 
     * @param timestamp
     *            a String containing a timestamp
     * @return a Timestamp object
     * @throws SAXException
     *             String is not a valid timestamp.
     */
    public Timestamp parseTimeStamp(String timestamp) throws SAXException {
        try {
            return m_validator.getTimeStamp(timestamp);
        } catch (CDBException e) {
            throw new SAXException(e);
        }
    }

    /**
     * Convert a String into a double.
     * 
     * @param value
     * @return a double
     * @throws SAXException
     */
    protected Double parseDouble(String value) throws SAXException {
        if (value == null) {
            throw new SAXException("Could not parse Double:" + value);
        }
        try {
            return Double.parseDouble(value);
        } catch (NumberFormatException e) {
            throw new SAXException("Could not parse Double:" + value);
        }
    }

    /**
     * Convert a String into an int.
     * 
     * @param value
     * @return an int
     * @throws SAXException
     */
    protected int parseInt(String value) throws SAXException {
        try {
            return Integer.parseInt(value);
        } catch (NumberFormatException e) {
            throw new SAXException("Could not parse Int:" + value);
        }
    }

    /**
     * Convert a String into a boolean.
     * 
     * @param value
     * @return a boolean
     * @throws SAXException
     */
    protected boolean parseBoolean(String value) throws SAXException {
        if (value == null) {
            throw new SAXException("Could not parse boolean:" + value);
        }
        if (value.equals("0") || value.equalsIgnoreCase("false")) {
            return false;
        } else if (value.equals("1") || value.equalsIgnoreCase("true")) {
            return true;
        }
        throw new SAXException("Could not parse boolean:" + value);
    }

    /**
     * Convert a String into a boolean.
     * 
     * @param value
     * @param defaultValue
     *            return this value if the field is null
     * @return a boolean
     * @throws SAXException
     */
    protected boolean parseBoolean(String value, boolean defaultValue) throws SAXException {
        if (value == null) {
            return defaultValue;
        }
        if (value.equals("0") || value.equalsIgnoreCase("false")) {
            return false;
        } else if (value.equals("1") || value.equalsIgnoreCase("true")) {
            return true;
        }
        throw new SAXException("Could not parse boolean:" + value);
    }
}
