package mice.cdb.service.statemachine;

import java.sql.Timestamp;

/**
 * A representation of a single record of the alarm table.
 * 
 */
public class PV extends StateMachineConstants {
    /** The name of the alarm. */
    private String m_name = "";

    /** The upper limit for a critical state. */
    private double m_hihi;

    /** The upper limit for a warning state. */
    private double m_hi;

    /** The lower limit for a warning state. */
    private double m_lo;

    /** The lower limit for a critical state. */
    private double m_lolo;

    private String m_units = "";
    private boolean m_autoSms;
    private String m_mode = "";
    private double m_frequency;
    private double m_deadband;
    private double m_transition;

    /**
     * Constructs an Alarm object.
     */
    public PV() {
    }

    /**
     * Constructs an Alarm object.
     * 
     * @param name
     *            a string containing the name of the alarm
     * @param hihi
     *            the higher value which the alarm considers an error
     * @param hi
     *            the higher value which the alarm considers a warning
     * @param lo
     *            the lower value which the alarm considers a warning
     * @param lolo
     *            the lower value which the alarm considers an error
     */
    protected PV(String name, double hihi, double hi, double lo, double lolo) {
        m_name = name;
        m_hihi = hihi;
        m_hi = hi;
        m_lo = lo;
        m_lolo = lolo;
    }

    /**
     * Set the higher value which the alarm considers a warning.
     * 
     * @param hi
     *            the higher value which the alarm considers a warning
     */
    public void setHi(double hi) {
        m_hi = hi;
    }

    /**
     * Set the higher value which the alarm considers an error.
     * 
     * @param hihi
     *            the higher value which the alarm considers an error
     */
    public void setHiHi(double hihi) {
        m_hihi = hihi;
    }

    /**
     * Set the lower value which the alarm considers a warning.
     * 
     * @param lo
     *            the lower value which the alarm considers a warning
     */
    public void setLo(double lo) {
        m_lo = lo;
    }

    /**
     * Set the lower value which the alarm considers an error.
     * 
     * @param lolo
     *            the lower value which the alarm considers an error
     */
    public void setLoLo(double lolo) {
        m_lolo = lolo;
    }

    /**
     * Set the name of the Alarm.
     * 
     * @param name
     *            a string containing the name of the alarm
     * 
     * @see #getName
     */
    public void setName(String name) {
        m_name = name;
    }

    /**
     * @param units
     *            the units to set
     */
    public void setUnits(String units) {
        m_units = units;
    }

    /**
     * @param autoSms
     *            the autoSms to set
     */
    public void setAutoSms(boolean autoSms) {
        m_autoSms = autoSms;
    }

    /**
     * @param mode
     *            the mode to set
     */
    public void setMode(String mode) {
        m_mode = mode;
    }

    /**
     * @param frequency
     *            the frequency to set
     */
    public void setFrequency(double frequency) {
        m_frequency = frequency;
    }

    /**
     * @param deadband
     *            the deadband to set
     */
    public void setDeadband(double deadband) {
        m_deadband = deadband;
    }

    /**
     * @param transition
     *            the transition to set
     */
    public void setTransition(double transition) {
        m_transition = transition;
    }

    /**
     * Get the name of the Alarm.
     * 
     * @return the name of the alarm
     * 
     * @see #setName
     */
    public String getName() {
        return m_name;
    }

    /**
     * Validate that all values make sense.
     * 
     * @return <code>true</code> if OK
     */
    public boolean validate() {
        if (m_lolo > m_lo | m_lo > m_hi | m_hi > m_hihi) {
            return false;
        }
        return true;
    }

    /**
     * Return a string containing an SQL insert statement that encapsulates the
     * data in the Alarm object.
     * 
     * @param system
     *            the name of the system to update
     * @param state
     *            the state to update to
     * @param creationTime
     *            a timestamp of when the data was created
     * @return a string containing an SQL insert statement
     */
    protected String getInsertQuery(String system, String state, Timestamp creationTime) {
        StringBuilder sql = new StringBuilder();
        // update old record
        sql.append("UPDATE ").append(StateMachineConstants.TABLE_PV_DATA).append(" SET ");
        sql.append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime);
        sql.append("' WHERE ");
        sql.append(StateMachineConstants.COLUMN_SYSTEM).append("='").append(system);
        sql.append("' AND ").append(StateMachineConstants.COLUMN_STATE).append("='").append(state);
        sql.append("' AND ").append(StateMachineConstants.COLUMN_NAME).append("='").append(m_name);
        sql.append("' AND ").append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        // add new record
        sql.append("INSERT INTO ").append(TABLE_PV_DATA);
        sql.append(" (").append(COLUMN_NAME).append(", ").append(COLUMN_HIHI).append(", ").append(COLUMN_HI)
                .append(", ").append(COLUMN_LO).append(", ").append(COLUMN_LOLO).append(", ").append(COLUMN_UNITS)
                .append(", ").append(COLUMN_AUTO_SMS).append(", ").append(COLUMN_MODE).append(", ")
                .append(COLUMN_FREQUENCY).append(", ").append(COLUMN_DEADBAND).append(", ").append(COLUMN_TRANSITION)
                .append(", ").append(COLUMN_CREATIONTIME).append(", ").append(COLUMN_SYSTEM).append(", ")
                .append(COLUMN_STATE);
        sql.append(") VALUES ('").append(m_name).append("', ").append(m_hihi).append(", ").append(m_hi).append(", ")
                .append(m_lo).append(", ").append(m_lolo).append(", '").append(m_units).append("', ").append(m_autoSms)
                .append(", '").append(m_mode).append("', ").append(m_frequency).append(", ").append(m_deadband)
                .append(", ").append(m_transition).append(", '").append(creationTime).append("', '").append(system)
                .append("', '").append(state).append("');");
        return sql.toString();
    }

}
