package mice.cdb.service.statemachine;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;

import org.apache.log4j.Logger;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.parser.PVParser;
import mice.cdb.service.parser.ParseException;

public class PVHandler extends CDB {

    public PVHandler(DBAccess mdb) {
        m_mdb = mdb;
        m_log = Logger.getLogger(getClass().getName());
    }

    public String getPVs(String system, String state) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getPVs(" + system + ", " + state + ")");
        }
        String xml = null;
        Connection connection = null;
        try {
            m_validator.validateStringNonBlank(system);
            String upperSystem = system.toUpperCase().trim();
            m_validator.validateStringNonBlank(state);
            String upperState = state.toUpperCase().trim();
            connection = m_mdb.getConnect();
            xml = selectPVs(connection, upperSystem, upperState);
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    public synchronized String setPVData(String system, String state, String xml) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setPVData(" + system + ", " + state + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        List<PV> pvList = null;

        try {
            m_validator.validateStringNonBlank(system);
            String upperSystem = system.toUpperCase().trim();
            m_validator.validateStringNonBlank(state);
            String upperState = state.toUpperCase().trim();

            // parse the XML
            PVParser ahParser = new PVParser();
            try {
                ahParser.parse(xml);
            } catch (ParseException e) {
                return "<error>Could not parse Alarm XML. Please check input. " + e.getMessage() + "</error>";
            }
            pvList = ahParser.getPVList();
            connection = m_mdb.getConnect();
            setPvData(connection, upperSystem, upperState, creationTime, pvList);
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return "<ok>" + system + " " + state + " " + pvList.size() + " pvs updated</ok>";
    }

    /**
     * Get the list of PV data that have the given system and state.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the name of the system
     * @param state
     *            the name of the state
     * @return a string containing XML representing the pv data
     * @throws CDBException
     */
    private String selectPVs(Connection connection, String system, String state) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectPVs system: " + system + ", state: " + state);
        }
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(StateMachineConstants.COLUMN_NAME).append(", ")
                .append(StateMachineConstants.COLUMN_HIHI).append(", ").append(StateMachineConstants.COLUMN_HI)
                .append(", ").append(StateMachineConstants.COLUMN_LO).append(", ")
                .append(StateMachineConstants.COLUMN_LOLO).append(", ").append(StateMachineConstants.COLUMN_UNITS)
                .append(", ").append(StateMachineConstants.COLUMN_AUTO_SMS).append(", ")
                .append(StateMachineConstants.COLUMN_MODE).append(", ").append(StateMachineConstants.COLUMN_FREQUENCY)
                .append(", ").append(StateMachineConstants.COLUMN_DEADBAND).append(", ")
                .append(StateMachineConstants.COLUMN_TRANSITION);
        select.append(" FROM ").append(StateMachineConstants.TABLE_PV_DATA);
        select.append(" WHERE ");
        select.append(StateMachineConstants.COLUMN_SYSTEM).append(" = '").append(system);
        select.append("' AND ").append(StateMachineConstants.COLUMN_STATE).append(" = '").append(state);
        select.append("' AND ").append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        ResultSet rs = m_mdb.query(connection, select.toString());
        StringBuilder xml = new StringBuilder();
        try {
            xml.append("<").append(StateMachineConstants.PV_DATA).append(">");
            while (rs.next()) {
                xml.append("<").append(StateMachineConstants.PV).append(" ");
                xml.append(StateMachineConstants.NAME).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_NAME));
                xml.append("' ").append(StateMachineConstants.HIHI).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_HIHI));
                xml.append("' ").append(StateMachineConstants.HI).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_HI));
                xml.append("' ").append(StateMachineConstants.LO).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_LO));
                xml.append("' ").append(StateMachineConstants.LOLO).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_LOLO));
                xml.append("' ").append(StateMachineConstants.UNITS).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_UNITS));
                xml.append("' ").append(StateMachineConstants.AUTO_SMS).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_AUTO_SMS));
                xml.append("' ").append(StateMachineConstants.MODE).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_MODE));
                xml.append("' ").append(StateMachineConstants.FREQUENCY).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_FREQUENCY));
                xml.append("' ").append(StateMachineConstants.DEADBAND).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_DEADBAND));
                xml.append("' ").append(StateMachineConstants.TRANSITION).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_TRANSITION));
                xml.append("'/>");
            }
            xml.append("</").append(StateMachineConstants.PV_DATA).append(">");
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return xml.toString();
    }

    /**
     * Update the existing record in the database with a valid until time and
     * add a new record for the given system and state.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the name of the system to update
     * @param state
     *            the state to update to
     * @param creationTime
     *            the time of the change
     * @param pvList
     *            a list of PVs
     * @throws CDBException
     */
    private void setPvData(Connection connection, String system, String state, Timestamp creationTime, List<PV> pvList)
            throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setPvData system: " + system + ", state: " + state);
        }
        StringBuilder sql = new StringBuilder();
        for (PV pv : pvList) {
            sql.append(pv.getInsertQuery(system, state, creationTime));
        }
        m_mdb.insert(connection, sql.toString());
    }

}
