package mice.cdb.service.statemachine;

import javax.jws.WebMethod;
import javax.jws.WebService;

/**
 * <p>
 * The <code>StateMachine</code> interface is used to retrieve stateMachine
 * data. To store data use the {@link StateMachineSuperMouse} interface. The
 * state of an individual system can be retrieved using
 * {@link #getCurrentState(String) getCurrentState} and the state of the entire
 * system can be retrieved with {@link #getCurrentStateMachine()
 * getCurrentStateMachine}. Old versions of the data are stored for diagnostic
 * purposes. Data can be retrieved for a given time using
 * {@link #getStateMachineForDate(String) getStateMachineForDate} and for a
 * given run using {@link #getStateMachineForRun(String) getStateMachineForRun}.
 * </p>
 * 
 * <p>
 * Please note that all timestamps should be in the format of 'yyyy-mm-dd
 * hh:mm:ss[.fffffffff]' in UTC.
 * </p>
 * 
 * <p>
 * If there are any problems then instead of the stated XML the returned XML
 * will contain a message enclosed in one of two tags: <code>warning</code> or
 * <code>error</code>. A tag of <code>warning</code> indicates that the problem
 * maybe transient and retrying the request MAY succeed. A tag of
 * <code>error</code> maybe due to to bad data being passed in or an unexpected
 * internal error.
 * </p>
 * 
 */
@WebService
public interface StateMachine {

    /**
     * Get the status of the service.
     * 
     * @return a string containing XML representing the status of the service.
     */
    @WebMethod
    public abstract String getStatus();

    /**
     * Get the name of the server.
     * 
     * @return a string containing XML representing the name of the server.
     */
    @WebMethod
    public abstract String getName();

    /**
     * Get the current state of the given system.
     * 
     * @param system
     *            the name of the system
     * @return a string containing XML representing the state
     *         <dl>
     *         <dd><code>&lt;state&gt;xxxx&lt;/state&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getCurrentState(String system);

    /**
     * Get the current state of the entire system.
     * 
     * @return a string containing XML representing the state
     *         <dl>
     *         <dd><code>&lt;stateMachine&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/stateMachine&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getCurrentStateMachine();

    /**
     * Get the state machine data that was valid at the given timestamp.
     * 
     * @param timestamp
     *            a time in the format 'yyyy-mm-dd hh:mm:ss[.fffffffff]' in UTC
     * @return a string containing XML representing the state
     *         <dl>
     *         <dd><code>&lt;stateMachine&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/stateMachine&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getStateMachineForDate(String timestamp);

    /**
     * Get the state machine data that was valid for the given run number.
     * 
     * @param runNo
     *            the number of a run
     * @return a string containing XML representing the state
     *         <dl>
     *         <dd><code>&lt;stateMachine&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/stateMachine&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getStateMachineForRun(String runNo);

    /**
     * Get a list of the allowed state transition relative to the current state
     * of the state machine.
     * 
     * @return a string containing XML representing the allowed state
     *         transitions
     *         <dl>
     *         <dd><code>&lt;stateMachine&gt;</code></dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd>
     *         <code>&nbsp;&nbsp;&nbsp;&nbsp;&lt;system name='xxxx' state='xxxx'/&gt;</code>
     *         </dd>
     *         <dd><code>&lt;/stateMachine&gt;</code></dd>
     *         </dl>
     */
    @WebMethod
    public abstract String getAllowedTransitions();

    @WebMethod
    public abstract String getPVData(String system, String state);

}
