package mice.cdb.service.statemachine;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>StateMachine</code>.
 * 
 * @see mice.cdb.service.statemachine.StateMachine
 */
@WebService(endpointInterface = "mice.cdb.service.statemachine.StateMachine")
public class StateMachineImpl extends CDB implements StateMachine {
    private PVHandler m_pvHandler;

    /**
     * Construct a StateMachine.
     */
    public StateMachineImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
        m_pvHandler = new PVHandler(m_mdb);
    }

    @Override
    public String getCurrentState(String system) {
        String xml = null;
        Connection connection = null;
        try {
            m_validator.validateStringNonBlank(system);
            String upperSystem = system.toUpperCase().trim();
            connection = m_mdb.getConnect();
            xml = selectCurrentState(connection, upperSystem);
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getCurrentStateMachine() {
        String xml = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append(" WHERE ").append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL");
            xml = selectStateMachine(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getStateMachineForDate(String timestamp) {
        String xml = null;
        Connection connection = null;
        try {
            Timestamp usersTimestamp = m_validator.getTimeStamp(timestamp);
            connection = m_mdb.getConnect();
            StringBuilder clause = new StringBuilder();
            clause.append("WHERE ").append(StateMachineConstants.COLUMN_CREATIONTIME).append("<='")
                    .append(usersTimestamp).append("' AND (").append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME)
                    .append(">'").append(usersTimestamp).append("' OR ")
                    .append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL)");
            xml = selectStateMachine(connection, clause.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getStateMachineForRun(String runNo) {
        String xml = null;
        Connection connection = null;
        try {
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runStartTime = selectRunStartTime(connection, runNumber);
            xml = getStateMachineForDate(runStartTime.toString());
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getAllowedTransitions() {
        String xml = null;
        Connection connection = null;
        try {
            connection = m_mdb.getConnect();
            xml = selectAllowedTransitions(connection, selectCurrentStates(connection));
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return xml;
    }

    @Override
    public String getPVData(String system, String state) {
        return m_pvHandler.getPVs(system, state);
    }

    /**
     * Test if the given system is allowed to change state to the given state.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the system whose state is to change
     * @param state
     *            the state to change to
     * @return <code>true</code> if the change is allowed
     * @throws CDBException
     *             if there is no entry for the system and or state
     */
    protected boolean changeAllowed(Connection connection, String system, String state) throws CDBException {
        Map<String, List<String>> allowedStates = allowedStates(connection, system, state);
        if (allowedStates.isEmpty()) {
            throw new CDBException("No entries found for system " + system + " state " + state);
        }
        Map<String, String> currentStates = selectCurrentStates(connection);
        for (String key : allowedStates.keySet()) {
            String currentState = currentStates.get(key);
            if (!allowedStates.get(key).contains(currentState)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Get the allowed states of related systems for the given system and state.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the system
     * @param state
     *            the state
     * @return the keys of the map are the related system names, the values are
     *         a list of allowed states for that related system
     * @throws CDBException
     */
    private Map<String, List<String>> allowedStates(Connection connection, String system, String state)
            throws CDBException {
        StringBuilder sql = new StringBuilder();
        sql.append("SELECT ").append(StateMachineConstants.COLUMN_RELATED_SYSTEM).append(", ")
                .append(StateMachineConstants.COLUMN_RELATED_SYSTEM_STATE).append(" FROM ")
                .append(StateMachineConstants.TABLE_STATE_MACHINE).append(" WHERE ")
                .append(StateMachineConstants.COLUMN_SYSTEM).append("='").append(system).append("' AND ")
                .append(StateMachineConstants.COLUMN_STATE).append("='").append(state).append("';");
        ResultSet rs = m_mdb.query(connection, sql.toString());
        Map<String, List<String>> allowedStates = new HashMap<String, List<String>>();
        List<String> stateList = new LinkedList<String>();
        try {
            while (rs.next()) {
                String subsystem = rs.getString(StateMachineConstants.COLUMN_RELATED_SYSTEM);
                String allowedState = rs.getString(StateMachineConstants.COLUMN_RELATED_SYSTEM_STATE);
                if (allowedStates.containsKey(subsystem)) {
                    stateList = allowedStates.get(subsystem);
                } else {
                    stateList = new LinkedList<String>();
                }
                stateList.add(allowedState);
                allowedStates.put(subsystem, stateList);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return allowedStates;
    }

    /**
     * Select the allowed transitions for the given states
     * 
     * @param connection
     *            db connection
     * @param currentStates
     *            a map containing system and state
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectAllowedTransitions(Connection connection, Map<String, String> currentStates)
            throws CDBException {
        StringBuilder sql;
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(StateMachineConstants.STATE_MACHINE).append(">");
        for (String system : currentStates.keySet()) {
            sql = new StringBuilder();
            sql.append("SELECT ").append(StateMachineConstants.COLUMN_STATE).append(" FROM ")
                    .append(StateMachineConstants.TABLE_STATE_MACHINE).append(" WHERE ")
                    .append(StateMachineConstants.COLUMN_SYSTEM).append("='").append(system).append("' AND ")
                    .append(StateMachineConstants.COLUMN_RELATED_SYSTEM).append("='").append(system).append("' AND ")
                    .append(StateMachineConstants.COLUMN_RELATED_SYSTEM_STATE).append("='")
                    .append(currentStates.get(system)).append("';");
            ResultSet rs = m_mdb.query(connection, sql.toString());
            try {
                while (rs.next()) {
                    if (changeAllowed(connection, system, rs.getString(StateMachineConstants.COLUMN_STATE))) {
                        xml.append("<").append(StateMachineConstants.SYSTEM).append(" ")
                                .append(StateMachineConstants.NAME).append("='").append(system).append("' ")
                                .append(StateMachineConstants.STATE).append("='")
                                .append(rs.getString(StateMachineConstants.COLUMN_STATE)).append("'/>");
                    }
                }
            } catch (SQLException e) {
                m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
                throw new CDBException("Internal error querying the database.");
            }
        }
        xml.append("</").append(StateMachineConstants.STATE_MACHINE).append(">");
        return xml.toString();
    }

    /**
     * Get the current state of the given system from the database.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the system
     * @return a string containing XML representing the current state of the
     *         given system
     * @throws CDBException
     */
    private String selectCurrentState(Connection connection, String system) throws CDBException {
        StringBuilder sql = new StringBuilder();
        sql.append("SELECT ").append(StateMachineConstants.COLUMN_STATE).append(" FROM ")
                .append(StateMachineConstants.TABLE_CURRENT_STATE).append(" WHERE ")
                .append(StateMachineConstants.COLUMN_SYSTEM).append("='").append(system).append("' AND ")
                .append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        ResultSet rs = m_mdb.query(connection, sql.toString());
        try {
            if (rs.next()) {
                StringBuilder xml = new StringBuilder();
                xml.append("<state>").append(rs.getString(StateMachineConstants.COLUMN_STATE)).append("</state>");
                return xml.toString();
            } else {
                throw new CDBException("No entry found for system " + system);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Get the current state of the entire system from the database.
     * 
     * @param connection
     *            db connection
     * @return a string containing XML representing the current state of the
     *         entire system
     * @throws CDBException
     */
    private Map<String, String> selectCurrentStates(Connection connection) throws CDBException {
        StringBuilder sql = new StringBuilder();
        sql.append("SELECT ").append(StateMachineConstants.COLUMN_SYSTEM).append(", ")
                .append(StateMachineConstants.COLUMN_STATE).append(" FROM ")
                .append(StateMachineConstants.TABLE_CURRENT_STATE).append(" WHERE ")
                .append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        ResultSet rs = m_mdb.query(connection, sql.toString());
        Map<String, String> currentStates = new HashMap<String, String>();
        try {
            while (rs.next()) {
                String system = rs.getString(StateMachineConstants.COLUMN_SYSTEM);
                String currentState = rs.getString(StateMachineConstants.COLUMN_STATE);
                currentStates.put(system, currentState);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return currentStates;
    }

    /**
     * Select the stateMachine using the given where clause.
     * 
     * @param connection
     *            db connection
     * @param clause
     *            an SQL WHERE statement
     * @return a string containing XML
     * @throws CDBException
     */
    private String selectStateMachine(Connection connection, String clause) throws CDBException {
        m_log.debug("selectStateMachine(" + clause + ")");
        StringBuilder sql = new StringBuilder();
        sql.append("SELECT ").append(StateMachineConstants.COLUMN_SYSTEM).append(", ")
                .append(StateMachineConstants.COLUMN_STATE).append(" FROM ")
                .append(StateMachineConstants.TABLE_CURRENT_STATE).append(" ").append(clause).append(";");
        ResultSet rs = m_mdb.query(connection, sql.toString());
        StringBuilder xml = new StringBuilder();
        xml.append("<").append(StateMachineConstants.STATE_MACHINE).append(">");
        try {
            while (rs.next()) {
                xml.append("<").append(StateMachineConstants.SYSTEM).append(" ").append(StateMachineConstants.NAME)
                        .append("='").append(rs.getString(StateMachineConstants.COLUMN_SYSTEM)).append("' ")
                        .append(StateMachineConstants.STATE).append("='")
                        .append(rs.getString(StateMachineConstants.COLUMN_STATE)).append("'/>");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        xml.append("</").append(StateMachineConstants.STATE_MACHINE).append(">");
        return xml.toString();
    }

}
