package mice.cdb.service.statemachine;

import java.sql.Connection;
import java.sql.Timestamp;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;

/**
 * An implementation of the <code>StateMachineSuperMouse</code>.
 * 
 * @see mice.cdb.service.statemachine.StateMachineSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.statemachine.StateMachineSuperMouse")
public class StateMachineSuperMouseImpl extends StateMachineImpl implements StateMachineSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;
    private PVHandler m_pvHandler;

    /**
     * Construct a StateMachine with super mouse powers.
     */
    public StateMachineSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
        m_pvHandler = new PVHandler(m_mdb);
    }

    @Override
    public synchronized String setState(String system, String state) {
        m_log.debug("setState(" + system + ", " + state + ")");
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        try {
            m_validator.validateStringNonBlank(system);
            String upperSystem = system.toUpperCase().trim();
            m_validator.validateStringNonBlank(state);
            String upperState = state.toUpperCase().trim();
            connection = m_mdb.getConnect();
            if (changeAllowed(connection, upperSystem, upperState)) {
                setState(connection, upperSystem, upperState, creationTime);
            } else {
                return "<error>" + "Cannot change state of " + system + " to " + state + "</error>";
            }
        } catch (CDBException e) {
            m_log.debug(e.getMessage());
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return "<ok>" + system + " state set to " + state + "</ok>";
    }

    /**
     * Update the existing record in the database with a valid until time and
     * add a new record for the given system with the given state.
     * 
     * @param connection
     *            db connection
     * @param system
     *            the name of the system to update
     * @param state
     *            the state to update to
     * @param creationTime
     *            the time of the change
     * @throws CDBException
     */
    private void setState(Connection connection, String system, String state, Timestamp creationTime)
            throws CDBException {
        StringBuilder sql = new StringBuilder();
        sql.append("UPDATE ").append(StateMachineConstants.TABLE_CURRENT_STATE).append(" SET ")
                .append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append("='").append(creationTime)
                .append("' WHERE ").append(StateMachineConstants.COLUMN_SYSTEM).append("='").append(system)
                .append("' AND ").append(StateMachineConstants.COLUMN_VALID_UNTIL_TIME).append(" is NULL;");
        m_mdb.insert(connection, sql.toString());

        sql = new StringBuilder();
        sql.append("INSERT into ").append(StateMachineConstants.TABLE_CURRENT_STATE).append(" (")
                .append(StateMachineConstants.COLUMN_SYSTEM).append(", ").append(StateMachineConstants.COLUMN_STATE)
                .append(", ").append(StateMachineConstants.COLUMN_CREATIONTIME).append(") VALUES ('").append(system)
                .append("', '").append(state).append("', '").append(creationTime).append("');");
        m_mdb.insert(connection, sql.toString());
    }

    @Override
    public String setPVData(String system, String state, String xml) {
        return m_pvHandler.setPVData(system, state, xml);
    }

}
