package mice.cdb.service.target;

import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.jws.WebService;

import mice.cdb.service.CDB;
import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;

import org.apache.log4j.Logger;

/**
 * An implementation of the <code>Target</code>.
 * 
 * @see mice.cdb.service.target.Target
 */
@WebService(endpointInterface = "mice.cdb.service.target.Target")
public class TargetImpl extends CDB implements Target {

    /**
     * Construct a Target.
     */
    public TargetImpl() {
        m_log = Logger.getLogger(getClass().getName());
        m_mdb = new DBAccess();
    }

    @Override
    public String getCurrentBlob(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentBlob(" + name + ")");
        }
        String data = "";
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            try {
                data = new String(selectCurrentBlob(connection, name.toString()), "UTF-8");
            } catch (UnsupportedEncodingException e) {
                m_log.error("Internal error decoding byte array.\n\t" + e.getLocalizedMessage());
                String message = "<error>Problem decoding data from the database</error>";
                return message;
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public String getBlobForRun(String name, String runNo) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBlobForRun(" + name + ", " + runNo + ")");
        }
        String data = "";
        Connection connection = null;
        try {
            validateTargetName(name);
            m_validator.validateStringNonBlank(runNo);
            int runNumber = m_validator.getInt(runNo);
            connection = m_mdb.getConnect();
            Timestamp runTime = selectRunStartTime(connection, runNumber);
            try {
                data = new String(selectByTimestamp(connection, name.toString(), runTime), "UTF-8");
            } catch (UnsupportedEncodingException e) {
                m_log.error("Internal error decoding byte array.\n\t" + e.getLocalizedMessage());
                String message = "<error>Problem decoding data from the database</error>";
                return message;
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public String getBlobForDate(String name, String dateTime) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getBlobForDate(" + name + ", " + dateTime + ")");
        }
        String data = "";
        Connection connection = null;
        try {
            validateTargetName(name);
            m_validator.validateStringNonBlank(name.toString());
            Timestamp timeStamp = m_validator.getTimeStamp(dateTime);
            connection = m_mdb.getConnect();
            try {
                data = new String(selectByTimestamp(connection, name.toString(), timeStamp), "UTF-8");
            } catch (UnsupportedEncodingException e) {
                m_log.error("Internal error decoding byte array.\n\t" + e.getLocalizedMessage());
                String message = "<error>Problem decoding data from the database</error>";
                return message;
            }
        } catch (CDBException e) {
            String message = "<error>" + e.getMessage() + "</error>";
            return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public Integer getCurrentUserDelay(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentUserDelay(" + name + ")");
        }
        Integer data = -1;
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            data = selectCurrentUserDelay(connection, name);
        } catch (CDBException e) {
            // String message = "<error>" + e.getMessage() + "</error>";
            // return message;
            return -1;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public Integer getCurrentSetBCD(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentSetBCD(" + name + ")");
        }
        Integer data = -1;
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            data = selectCurrentSetBCD(connection, name);
        } catch (CDBException e) {
            // String message = "<error>" + e.getMessage() + "</error>";
            // return message;
            return -1;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    // @Override
    public HashMap<String, Integer> getCurrentTargetIntegers(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentTargetIntegers(" + name + ")");
        }
        HashMap<String, Integer> data = null;
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            data = selectCurrentIntegers(connection, name);
        } catch (CDBException e) {
            // String message = "<error>" + e.getMessage() + "</error>";
            // return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    // @Override
    public HashMap<String, Float> getCurrentTargetFloats(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentTargetFloats(" + name + ")");
        }
        HashMap<String, Float> data = null;
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            data = selectCurrentFloats(connection, name);
        } catch (CDBException e) {
            // String message = "<error>" + e.getMessage() + "</error>";
            // return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    // @Override
    public HashMap<String, String> getCurrentTargetStrings(String name) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("getCurrentTargetStrings(" + name + ")");
        }
        HashMap<String, String> data = null;
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            data = selectCurrentStrings(connection, name);
        } catch (CDBException e) {
            // String message = "<error>" + e.getMessage() + "</error>";
            // return message;
        } finally {
            m_mdb.close(connection);
        }
        return data;
    }

    @Override
    public List<String> getTargetNames() {
        List<String> names = new ArrayList<String>();
        for (TargetName targetName : TargetName.values()) {
            names.add(targetName.name());
        }
        return names;
    }

    /**
     * Select the current data entry.
     * 
     * @param connection
     *            db connection
     * @return a byte array containing the target data
     * @throws CDBException
     */
    private byte[] selectCurrentBlob(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentBlob(" + name + ")");
        }
        // There will only be one row returned
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_BLOB);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_DATA);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_BLOB);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getBytes(TargetConstants.COLUMN_DATA);
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug("No data returned from the database.\n\t" + select);
                }
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    private Integer selectCurrentUserDelay(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentUserDelay(" + name + ")");
        }
        // There will only be one row returned
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_USER_DELAY);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_TIME);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_USER_DELAY);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getInt(TargetConstants.COLUMN_TIME);
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug("No data returned from the database.\n\t" + select);
                }
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    private Integer selectCurrentSetBCD(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentSetBCD(" + name + ")");
        }
        // There will only be one row returned
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_SET_BCD);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_DISTANCE);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_SET_BCD);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getInt(TargetConstants.COLUMN_DISTANCE);
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug("No data returned from the database.\n\t" + select);
                }
                throw new CDBException("No data returned from the database.");
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    private HashMap<String, Integer> selectCurrentIntegers(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentIntegers(" + name + ")");
        }
        HashMap<String, Integer> results = new HashMap<String, Integer>();
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_KEY);
        select.append(", ").append(TargetConstants.COLUMN_VALUE);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            while (rs.next()) {
                results.put(rs.getString(TargetConstants.COLUMN_KEY), rs.getInt(TargetConstants.COLUMN_VALUE));
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return results;
    }

    private HashMap<String, Float> selectCurrentFloats(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentFloats(" + name + ")");
        }
        HashMap<String, Float> results = new HashMap<String, Float>();
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_KEY);
        select.append(", ").append(TargetConstants.COLUMN_VALUE);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            while (rs.next()) {
                results.put(rs.getString(TargetConstants.COLUMN_KEY), rs.getFloat(TargetConstants.COLUMN_VALUE));
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return results;
    }

    private HashMap<String, String> selectCurrentStrings(Connection connection, String name) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectCurrentStrings(" + name + ")");
        }
        HashMap<String, String> results = new HashMap<String, String>();
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_KEY);
        select.append(", ").append(TargetConstants.COLUMN_VALUE);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_INTEGER);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");
        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            while (rs.next()) {
                results.put(rs.getString(TargetConstants.COLUMN_KEY), rs.getString(TargetConstants.COLUMN_VALUE));
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
        return results;
    }

    /**
     * Select the target data for the given timestamp.
     * 
     * @param connection
     *            db connection
     * @param timestamp
     *            date of interest
     * @return a byte array containing the target data
     * @throws CDBException
     */
    private byte[] selectByTimestamp(Connection connection, String name, Timestamp timeStamp) throws CDBException {
        if (m_log.isDebugEnabled()) {
            m_log.debug("selectByTimestamp(" + timeStamp + ")");
        }
        StringBuilder maxCreationTime = new StringBuilder();
        maxCreationTime.append("SELECT max(").append(TargetConstants.COLUMN_CREATIONTIME);
        maxCreationTime.append(") FROM ").append(TargetConstants.TABLE_TARGET_BLOB);
        maxCreationTime.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" <= '").append(timeStamp)
                .append("' AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("'");
        StringBuilder select = new StringBuilder();
        select.append("SELECT ").append(TargetConstants.COLUMN_DATA);
        select.append(" FROM ").append(TargetConstants.TABLE_TARGET_BLOB);
        select.append(" WHERE ").append(TargetConstants.COLUMN_CREATIONTIME).append(" = (").append(maxCreationTime)
                .append(") AND ").append(TargetConstants.COLUMN_NAME).append(" = '").append(name).append("';");

        ResultSet rs = m_mdb.query(connection, select.toString());
        try {
            if (rs.next()) {
                return rs.getBytes(TargetConstants.COLUMN_DATA);
            } else {
                if (m_log.isDebugEnabled()) {
                    m_log.debug("No target data found for " + timeStamp);
                }
                throw new CDBException("No target data found for " + timeStamp);
            }
        } catch (SQLException e) {
            m_log.error("Internal error querying the database.\n\t" + e.getLocalizedMessage());
            throw new CDBException("Internal error querying the database.");
        }
    }

    /**
     * Check that the name is known.
     * 
     * @param name
     *            a string containing the target name to validate
     * @throws CDBException
     */
    protected void validateTargetName(String name) throws CDBException {
        try {
            TargetName.valueOf(TargetName.class, name);
        } catch (IllegalArgumentException e) {
            throw new CDBException("Invalid target name");
        } catch (NullPointerException e) {
            throw new CDBException("Invalid target name");
        }
    }

}
