package mice.cdb.service.target;

import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.jws.WebService;

import mice.cdb.service.CDBException;
import mice.cdb.service.DBAccess;
import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;

/**
 * An implementation of the <code>TargetSuperMouse</code>.
 * 
 * @see mice.cdb.service.target.TargetSuperMouse
 */
@WebService(endpointInterface = "mice.cdb.service.target.TargetSuperMouse")
public class TargetSuperMouseImpl extends TargetImpl implements TargetSuperMouse {

    /** CDB properties object. */
    private static ServerConfig m_config;

    /**
     * Constructs a Target object with super mouse powers.
     */
    public TargetSuperMouseImpl() {
        try {
            m_config = ServerConfig.getInstance();
            String dbUser = m_config.getString(ServerConstants.DB_SUPER_USER);
            String dbPwd = m_config.getString(ServerConstants.DB_SUPER_PWD);
            m_mdb = new DBAccess(dbUser, dbPwd);
        } catch (CDBException e) {
            m_log.error(e.getMessage());
        }
    }

    @Override
    public synchronized String addBlob(String name, String data) {
        if (m_log.isDebugEnabled()) {
            if (data.length() > 200) {
                m_log.debug("addBlob(" + name + ")");
            } else {
                m_log.debug("addBlob(" + name + ", " + data + ")");

            }
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        int inserted = 0;
        Connection connection = null;

        try {
            validateTargetName(name);
            m_validator.validateNonBlank(data);

            connection = m_mdb.getConnect();
            String insert = "INSERT INTO " + TargetConstants.TABLE_TARGET_BLOB + " (" + TargetConstants.COLUMN_DATA
                    + ", " + TargetConstants.COLUMN_CREATIONTIME + ", " + TargetConstants.COLUMN_NAME
                    + ") VALUES (?, '" + creationTime + "', '" + name.toString() + "');";
            if (m_log.isDebugEnabled()) {
                m_log.debug("insert:" + insert);
            }
            PreparedStatement ps = m_mdb.prepareStatement(connection, insert);
            try {
                ps.setBytes(1, data.getBytes("UTF-8"));
            } catch (UnsupportedEncodingException e) {
                return "<error>Unsupported character found when encoding string</error>";
            }
            inserted = ps.executeUpdate();
            ps.close();
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } catch (SQLException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }

        if (inserted == 1) {
            if (m_log.isInfoEnabled()) {
                m_log.info("Inserted data, " + data.length() + " characters");
            }
            return "<ok>Inserted data, " + data.length() + " characters</ok>";
        } else {
            if (m_log.isInfoEnabled()) {
                m_log.info("Data not inserted");
            }
            return "<error>Data not inserted</error>";
        }
    }

    @Override
    public String setUserDelay(String name, Integer time) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("(setUserDelay" + name + ", " + time + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            String insert = "INSERT INTO " + TargetConstants.TABLE_TARGET_USER_DELAY + " ("
                    + TargetConstants.COLUMN_NAME + ", " + TargetConstants.COLUMN_TIME + ", "
                    + TargetConstants.COLUMN_CREATIONTIME + ") VALUES ('" + name + "', " + time + ", '" + creationTime
                    + "');";
            m_mdb.insert(connection, insert.toString());
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return "<ok>User delay data added</ok>";
    }

    @Override
    public String setSetBCD(String name, Integer distance) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setSetBCD(" + name + ", " + distance + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            String insert = "INSERT INTO " + TargetConstants.TABLE_TARGET_SET_BCD + " (" + TargetConstants.COLUMN_NAME
                    + ", " + TargetConstants.COLUMN_DISTANCE + ", " + TargetConstants.COLUMN_CREATIONTIME
                    + ") VALUES ('" + name + "', " + distance + ", '" + creationTime + "');";
            m_mdb.insert(connection, insert.toString());
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return "<ok>Set BCD data added</ok>";
    }

    // @Override
    public String setTarget(String name, HashMap<String, Integer> mapIntegers, HashMap<String, Float> mapFloats,
            HashMap<String, String> mapStrings) {
        if (m_log.isDebugEnabled()) {
            m_log.debug("setTarget(" + name + ")");
        }
        Timestamp creationTime = new Timestamp(System.currentTimeMillis());
        Connection connection = null;
        try {
            validateTargetName(name);
            connection = m_mdb.getConnect();
            StringBuilder insert = new StringBuilder();
            insert.append(addIntegers(name, mapIntegers, creationTime));
            insert.append(addFloats(name, mapFloats, creationTime));
            insert.append(addStrings(name, mapStrings, creationTime));
            m_mdb.batchInsert(connection, insert.toString());
        } catch (CDBException e) {
            if (m_log.isDebugEnabled()) {
                m_log.debug(e.getMessage());
            }
            return "<error>" + e.getMessage() + "</error>";
        } finally {
            m_mdb.close(connection);
        }
        return "<ok>Target data added</ok>";
    }

    private String addFloats(String name, Map<String, Float> mapFloats, Timestamp creationTime) {
        StringBuilder insert = new StringBuilder();
        Set<String> keys = mapFloats.keySet();
        for (String key : keys) {
            insert.append("INSERT INTO ").append(TargetConstants.TABLE_TARGET_FLOAT).append("(");
            insert.append(TargetConstants.COLUMN_NAME).append(", ");
            insert.append(TargetConstants.COLUMN_KEY).append(", ");
            insert.append(TargetConstants.COLUMN_VALUE).append(", ");
            insert.append(TargetConstants.COLUMN_CREATIONTIME).append(")");
            insert.append(" VALUES ('");
            insert.append(name).append("', '");
            insert.append(key).append("', ");
            insert.append(mapFloats.get(key)).append(", '");
            insert.append(creationTime).append("');");
        }
        return insert.toString();
    }

    private String addStrings(String name, Map<String, String> mapStrings, Timestamp creationTime) {
        StringBuilder insert = new StringBuilder();
        Set<String> keys = mapStrings.keySet();
        for (String key : keys) {
            insert.append("INSERT INTO ").append(TargetConstants.TABLE_TARGET_STRING).append("(");
            insert.append(TargetConstants.COLUMN_NAME).append(", ");
            insert.append(TargetConstants.COLUMN_KEY).append(", ");
            insert.append(TargetConstants.COLUMN_VALUE).append(", ");
            insert.append(TargetConstants.COLUMN_CREATIONTIME).append(")");
            insert.append(" VALUES ('");
            insert.append(name).append("', '");
            insert.append(key).append("', '");
            insert.append(mapStrings.get(key)).append("', '");
            insert.append(creationTime).append("');");
        }
        return insert.toString();
    }

    private String addIntegers(String name, Map<String, Integer> mapIntegers, Timestamp creationTime) {
        StringBuilder insert = new StringBuilder();
        Set<String> keys = mapIntegers.keySet();
        for (String key : keys) {
            insert.append("INSERT INTO ").append(TargetConstants.TABLE_TARGET_INTEGER).append("(");
            insert.append(TargetConstants.COLUMN_NAME).append(", ");
            insert.append(TargetConstants.COLUMN_KEY).append(", ");
            insert.append(TargetConstants.COLUMN_VALUE).append(", ");
            insert.append(TargetConstants.COLUMN_CREATIONTIME).append(")");
            insert.append(" VALUES ('");
            insert.append(name).append("', '");
            insert.append(key).append("', ");
            insert.append(mapIntegers.get(key)).append(", '");
            insert.append(creationTime).append("');");
        }
        return insert.toString();
    }
}