package mice.cdb.service;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;

import java.sql.Connection;
import java.sql.ResultSet;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import utils.RTest;
import utils.TestSetup;

public class DBAccessTest extends RTest {

    private static String m_smUser;
    private static String m_smPassword;
    private static DBAccess m_mdb;

    @BeforeClass
    public static void createTables() throws Exception {
        startOfTestClass(DBAccessTest.class);
        TestSetup.initializeServer();
        ServerConfig serverConfig = ServerConfig.getInstance();
        m_smUser = serverConfig.getProperty(ServerConstants.DB_SUPER_USER);
        m_smPassword = serverConfig.getProperty(ServerConstants.DB_SUPER_PWD);
        testClassReady();
    }

    @After
    public void a1_After() {
        endOfTest();
    }

    @AfterClass
    public static void a1_AfterClass() {
        endOfTestClass();
    }

    @Test
    public final void testConnect() throws Exception {
        m_mdb = new DBAccess("not", "me");
        String message = "";
        Connection connection = null;
        try {
            try {
                connection = m_mdb.getConnect();
            } catch (CDBException e) {
                message = e.getMessage();
            }
            assertEquals(
                    "Querry - error",
                    "Could not get a connection with database test at jdbc:postgresql://localhost:5432/. FATAL: password authentication failed for user \"not\"",
                    message);
        } finally {
            m_mdb.close(connection);
        }

        m_mdb = new DBAccess();
        boolean error = false;
        try {
            try {
                m_mdb.getConnect();
            } catch (CDBException e) {
                error = true;
            }
            assertFalse("Connect should be true", error);

        } finally {
            m_mdb.close(connection);
        }

        m_mdb = new DBAccess(m_smUser, m_smPassword);
        try {
            try {
                m_mdb.getConnect();
            } catch (CDBException e) {
                error = true;
            }
            assertFalse("Connect should be true", error);
        } finally {
            m_mdb.close(connection);
        }
    }

    @Test
    public final void testQuery() throws Exception {
        m_mdb = new DBAccess();
        Connection connection = null;
        try {
            String error = "";
            try {
                m_mdb.query(connection, "select * from junk;");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Querry - error", "Internal error, query called before database connect", error);

            connection = m_mdb.getConnect();
            error = "";
            ResultSet rs = m_mdb.query(connection, "select 1;");
            rs.next();
            int result = rs.getInt(1);
            assertEquals("Querry", 1, result);

            error = "";
            try {
                m_mdb.query(connection, "select * from junk;");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Querry - error", "Problem issuing query select * from junk;", error);
        } finally {
            m_mdb.close(connection);
        }
    }

    @Test
    public final void testInsert() throws Exception {
        m_mdb = new DBAccess();
        Connection connection = null;
        String error = "";
        try {
            connection = m_mdb.getConnect();
            try {
                m_mdb.insert(connection, "insert into test (astring) VALUES ('testInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Insert - error", "Problem inserting data into the database.", error);
        } finally {
            m_mdb.close(connection);
        }

        try {
            connection = null;
            m_mdb = new DBAccess(m_smUser, m_smPassword);
            error = "";
            try {
                m_mdb.insert(connection, "insert into test (astring) VALUES ('testInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Insert - error", "Internal error, insert called before database connect", error);

            connection = m_mdb.getConnect();
            error = "";
            try {
                m_mdb.insert(connection, "insert into test (astring) VALUES ('testInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Insert", "", error);

            error = "";
            try {
                m_mdb.insert(connection, "insert into test (astring) VALUES ('testInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Insert - error", "Problem inserting data into the database - duplicate record.", error);

            error = "";
            try {
                m_mdb.insert(connection, "insert into test (astrin) VALUES ('testInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("Insert - error", "Problem inserting data into the database.", error);
        } finally {
            m_mdb.close(connection);
        }
    }

    @Test
    public final void testBatchInsert() throws Exception {
        m_mdb = new DBAccess();
        Connection connection = null;
        String error = "";
        try {
            try {
                connection = m_mdb.getConnect();
                m_mdb.batchInsert(connection, "insert into test (astring) VALUES ('testBatchInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("BatchInsert - error",
                    "Problem with batch insert into the database. No data has been inserted.", error);
        } finally {
            m_mdb.close(connection);
        }

        try {
            connection = null;
            m_mdb = new DBAccess(m_smUser, m_smPassword);
            error = "";
            try {
                m_mdb.batchInsert(connection, "insert into test (astring) VALUES ('testBatchInsert');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("BatchInsert - error", "Internal error, batch insert called before database connect", error);

            connection = m_mdb.getConnect();
            int rows = m_mdb.batchInsert(connection, "insert into test (astring) VALUES ('testBatchInsert');");
            assertEquals("BatchInsert", 1, rows);

            rows = 0;
            rows = m_mdb
                    .batchInsert(connection,
                            "insert into test (astring) VALUES ('testBatchInsert1');insert into test (astring) VALUES ('testBatchInsert2');");
            assertEquals("BatchInsert", 2, rows);

            error = "";
            try {
                m_mdb.batchInsert(connection,
                        "insert into test (astring) VALUES ('testBatchInsert3');insert into test (astring) VALUES ('testBatchInsert3');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("BatchInsert - error",
                    "Problem inserting data into the database - duplicate record. No data has been inserted.", error);

            error = "";
            try {
                m_mdb.batchInsert(connection,
                        "insert into test (astring) VALUES ('testBatchInsert4');insert into test (astrin) VALUES ('testBatchInsert5');");
            } catch (CDBException e) {
                error = e.getMessage();
            }
            assertEquals("BatchInsert - error",
                    "Problem with batch insert into the database. No data has been inserted.", error);
        } finally {
            m_mdb.close(connection);
        }
    }

}
