package utils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.TimeZone;

import mice.cdb.service.ServerConfig;
import mice.cdb.service.ServerConstants;

public class TestSetup {

    public static final String TEST_LOGGER = "tests";

    /**
     * Initialize all of the server constants to default values Drop the
     * Consumer, PrimaryProducer, Schema, Registry and TaskManager instances
     * Drop and recreate the database
     */
    public static void initializeServer() throws Exception {

        /*
         * This TimeZone setting is normally done by the service class - but not
         * all tests use that class
         */
        TimeZone.setDefault(TimeZone.getTimeZone("GMT"));

        /* clean up any old services that may be hanging around */
        // dropServiceInstances();

        File tempFile = getTempFile("test", "");
        tempFile.delete();
        tempFile.mkdir();

        String miceHome = tempFile.getPath();
        System.setProperty(ServerConstants.MICE_HOME_PROPERTY, miceHome);

        File svfl = getTempFile("version", "props");
        FileOutputStream svflst = new FileOutputStream(svfl);
        PrintWriter writer = new PrintWriter(svflst);
        writer.write("1.2.3");
        writer.close();
        svflst.close();

        File dir = new File(miceHome, ServerConstants.SERVER_CONFIG_LOCATION);
        dir.mkdirs();
        dir.delete();

        Properties serverConfig = new Properties();

        serverConfig.store(new FileOutputStream(dir), "Test props file");

        serverConfig = ServerConfig.getInstance(); // This may be an old
        // instance

        serverConfig.clear(); // Clear it and fill it

        // Database
        serverConfig.setProperty(ServerConstants.DB_JDBC_DRIVER, "org.postgresql.Driver");
        serverConfig.setProperty(ServerConstants.DB_URL, "jdbc:postgresql://localhost:5432/");
        serverConfig.setProperty(ServerConstants.DB_PWD, "pw1");
        serverConfig.setProperty(ServerConstants.DB_USER, "tester");
        serverConfig.setProperty(ServerConstants.DB_SUPER_PWD, "pw2");
        serverConfig.setProperty(ServerConstants.DB_SUPER_USER, "supertester");
        serverConfig.setProperty(ServerConstants.DB_NAME, "test");

        // clean up any old databases that may be hanging around
        destroyDatabase();

    }

    /**
     * Drop and recreate the database
     * 
     * @throws Exception
     *             DOCUMENT ME!
     */
    private static void destroyDatabase() throws Exception {
        Statement statement = null;
        Connection connection = null;
        ServerConfig serverConfig = ServerConfig.getInstance();
        String JDBC_DRIVER = serverConfig.getProperty(ServerConstants.DB_JDBC_DRIVER);

        try {
            Class.forName(JDBC_DRIVER);

            String url = serverConfig.getProperty(ServerConstants.DB_URL);
            String db = serverConfig.getProperty(ServerConstants.DB_NAME);

            String adminUser = System.getenv("DB_ADMIN_USER");
            if (adminUser == null) {
                System.out.println("Property DB_ADMIN_USER not found");
                System.exit(0);
            }
            String adminPassword = System.getenv("DB_ADMIN_PWD");
            if (adminPassword == null) {
                System.out.println("Property DB_ADMIN_PWD not found");
                System.exit(0);
            }

            boolean dropped = true;
            try {
                connection = DriverManager.getConnection(url, adminUser, adminPassword);
                statement = connection.createStatement();
                try {
                    statement.execute("DROP DATABASE " + db);
                } catch (SQLException se) {
                    System.out.println("Database not dropped " + se.getMessage());
                    dropped = false;
                }
                if (dropped) {
                    statement.execute("CREATE DATABASE " + db);
                }
                statement.close();
            } finally {
                if (connection != null) {
                    connection.close();
                }
            }
            String smUser = "";
            String smPassword = "";
            String user = "";
            String password = "";

            try {
                connection = DriverManager.getConnection(url + db, adminUser, adminPassword);
                statement = connection.createStatement();

                smUser = serverConfig.getProperty(ServerConstants.DB_SUPER_USER);
                smPassword = serverConfig.getProperty(ServerConstants.DB_SUPER_PWD);
                user = serverConfig.getProperty(ServerConstants.DB_USER);
                password = serverConfig.getProperty(ServerConstants.DB_PWD);

                try {
                    statement.execute("DROP USER " + smUser + ";");
                } catch (SQLException se) {
                }
                try {
                    statement.execute("DROP USER " + user + ";");
                } catch (SQLException se) {
                }
                try {
                    statement.execute("CREATE USER " + smUser + " WITH PASSWORD '" + smPassword + "';");
                    statement.execute("CREATE USER " + user + " WITH PASSWORD '" + password + "';");
                } catch (SQLException se) {
                }
                statement.close();
            } finally {
                connection.close();
            }

            try {
                connection = DriverManager.getConnection(url + db, smUser, smPassword);
                statement = connection.createStatement();
                if (dropped) {
                    statement.execute("CREATE table test (astring varchar(50) PRIMARY KEY, anint int);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON  test to " + smUser + ";");
                    statement.execute("GRANT SELECT ON test to " + user + ";");

                    statement
                            .execute("CREATE table alarm (creationtime timestamp, tag varchar(40), name varchar (40), hihi double precision, hi double precision, lo double precision, lolo double precision, CONSTRAINT creationtime_tag_name PRIMARY KEY(creationtime,tag,name));");
                    statement
                            .execute("CREATE table alarmhistory (starttime timestamp PRIMARY KEY, alarmcreationtime timestamp, alarmtag varchar(40));");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON alarm, alarmhistory to " + smUser + ";");
                    statement.execute("GRANT SELECT ON alarm, alarmhistory to " + user + ";");

                    statement
                            .execute("CREATE TABLE beamline (runnumber bigint PRIMARY KEY, beamStop boolean, diffuserThickness int, protonAbsorberThickness int);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON beamline to " + smUser + ";");
                    statement.execute("GRANT SELECT ON beamline to " + user + ";");

                    statement
                            .execute("CREATE table cablingcontrol (crate integer, module integer, channel integer, name varchar(50), creationtime timestamp, validuntiltime timestamp);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON cablingcontrol to " + smUser + ";");
                    statement.execute("GRANT SELECT ON cablingcontrol to " + user + ";");

                    statement
                            .execute("CREATE table cablingtracker (device varchar(50), vlsbcomputerid integer, vlsbgeonumber integer, vlsbchannel integer, trackernumber integer, station integer, plane integer, channel integer, creationtime timestamp, validuntiltime timestamp);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON cablingtracker to " + smUser + ";");
                    statement.execute("GRANT SELECT ON cablingtracker to " + user + ";");

                    statement
                            .execute("CREATE table calibrationtracker (device varchar(50), board integer, bank integer, channel integer, adcpedestal integer, adcgain integer, tdcpedestal integer, tdcslope integer, creationtime timestamp, validuntiltime timestamp);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON calibrationtracker to " + smUser + ";");
                    statement.execute("GRANT SELECT ON calibrationtracker to " + user + ";");

                    statement
                            .execute("CREATE table controls (crate integer, module integer, channel integer, parametername varchar(100), parametervalue varchar (100), creationtime timestamp, validuntiltime timestamp);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON controls to " + smUser + ";");
                    statement.execute("GRANT SELECT ON controls to " + user + ";");

                    statement
                            .execute("CREATE table geometry (id SERIAL PRIMARY KEY, validfromtime timestamp, gdml bytea, notes text, technicaldrawingnname varchar(100), creationtime timestamp);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON geometry to " + smUser + ";");
                    statement.execute("GRANT SELECT ON geometry_id_seq to " + smUser + ";");
                    statement.execute("GRANT SELECT ON geometry, geometry_id_seq to " + user + ";");

                    statement
                            .execute("CREATE TABLE isisbeam (runnumber bigint, name varchar(30), mean double precision, sigma double precision);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON isisbeam to " + smUser + ";");
                    statement.execute("GRANT SELECT ON isisbeam to " + user + ";");

                    statement.execute("CREATE TABLE ldchosts (runnumber bigint, name varchar(50));");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON ldchosts to " + smUser + ";");
                    statement.execute("GRANT SELECT ON ldchosts to " + user + ";");

                    statement
                            .execute("CREATE TABLE run (runnumber bigint PRIMARY KEY, starttime timestamp, endtime timestamp, notes text, endnotes text, optics varchar, pulsestart bigint, pulseend bigint, step double precision, runtype varchar(50), status boolean, gdchostname varchar(50));");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON run to " + smUser + ";");
                    statement.execute("GRANT SELECT ON run to " + user + ";");

                    statement
                            .execute("CREATE TABLE isis (runnumber bigint PRIMARY KEY, targetfrequency bigint, reprate double precision, nominalbeamcurrent double precision, nominalbeamloss double precision, beampositionx double precision, beampositiony double precision);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON isis to " + smUser + ";");
                    statement.execute("GRANT SELECT ON isis to " + user + ";");

                    statement
                            .execute("CREATE TABLE daq (runnumber bigint PRIMARY KEY, trigger text, gatewidth double precision, version varchar);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON daq to " + smUser + ";");
                    statement.execute("GRANT SELECT ON daq to " + user + ";");

                    statement
                            .execute("CREATE TABLE target (runnumber bigint PRIMARY KEY, depth double precision, delay double precision, drivevoltage double precision);");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON target to " + smUser + ";");
                    statement.execute("GRANT SELECT ON target to " + user + ";");

                    statement
                            .execute("CREATE TABLE  magnet (runnumber bigint, name varchar, setcurrent double precision, polarity varchar, CONSTRAINT runnumber_name PRIMARY KEY(runnumber, name));");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON magnet to " + smUser + ";");
                    statement.execute("GRANT SELECT ON magnet to " + user + ";");

                    statement
                            .execute("CREATE TABLE scalar (runnumber bigint, name varchar(30), value int, CONSTRAINT runno_name PRIMARY KEY(runnumber, name));");
                    statement.execute("GRANT SELECT, INSERT, UPDATE ON scalar to " + smUser + ";");
                    statement.execute("GRANT SELECT ON scalar to " + user + ";");

                } else {
                    statement.execute("DELETE FROM test;");
                    statement.execute("DELETE FROM alarm;");
                    statement.execute("DELETE FROM alarmhistory;");
                    statement.execute("DELETE FROM cablingcontrol;");
                    statement.execute("DELETE FROM cablingtracker;");
                    statement.execute("DELETE FROM calibrationtracker;");
                    statement.execute("DELETE FROM controls;");
                    statement.execute("DELETE FROM geometry;");
                    statement.execute("DELETE FROM run;");
                    statement.execute("DELETE FROM isis;");
                    statement.execute("DELETE FROM daq;");
                    statement.execute("DELETE FROM target;");
                    statement.execute("DELETE FROM magnet;");
                    statement.execute("DELETE FROM scalar;");
                }

                statement.close();
            } finally {
                connection.close();
            }
        } catch (Exception e) {
            System.out.println("Problem with database " + e.getMessage());
            throw e;
        } finally {
            if (statement != null) {
                statement.close();
            }

            if (connection != null) {
                connection.close();
            }
        }
    }

    private static File getTempFile(String prefix, String suffix) throws IOException {
        File tempFile = null;
        for (int i = 1; i < 10; i++) {
            try {
                tempFile = File.createTempFile(prefix, suffix);
                break;
            } catch (IOException e) {
                // Do nothing - just hope for better luck
            }
        }
        if (tempFile == null) {
            tempFile = File.createTempFile(prefix, suffix); // One more go but
            // this time it can
            // throw the exception back
        }
        return tempFile;
    }
}
