package mice.cdb.service;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Properties object used to configure a CDB Server.
 * 
 */
@SuppressWarnings("serial")
public class ServerConfig extends Properties {

    static ServerConfig s_config;

    /**
     * Constructs a ServerConfig object.
     */
    private ServerConfig() throws CDBException {
        InputStream is = null;
        String miceHome = System.getProperty(ServerConstants.MICE_HOME_PROPERTY);
        if (miceHome == null) {
            // TODO throw new CDBException("System property " +
            // ServerConstants.MICE_HOME_PROPERTY + " is not set");
            miceHome = "/opt/mice";
        }
        try {
            is = new FileInputStream(miceHome + ServerConstants.SERVER_CONFIG_LOCATION);
            load(is);
        } catch (IOException e) {
            throw new CDBException("ServerConfiguration could not be loaded from: " + miceHome
                    + ServerConstants.SERVER_CONFIG_LOCATION + "\n\t" + e.getMessage());
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    // No action
                }
            }
        }
    }

    /**
     * Get an instance of the server configuration.
     * 
     * @return the current server configuration
     * @throws CDBException
     *             exception if the loading of the server configuration file
     *             fails
     */
    public static synchronized ServerConfig getInstance() throws CDBException {
        if (s_config == null) {
            s_config = new ServerConfig();
        }
        return s_config;
    }

    /**
     * Returns the string value.
     * 
     * @param name
     *            a String containing the name of the property
     * @return the value of the property
     * @throws CDBException
     */
    public String getString(String name) throws CDBException {
        String value = getProperty(name);
        if (value != null) {
            return value.trim();
        } else {
            throw new CDBException("Required parameter not found: " + name);
        }
    }

    /**
     * Returns the boolean value.
     * 
     * @param name
     *            a String containing the name of the property
     * @return the value of the property
     * @throws CDBException
     */
    public boolean getBoolean(String name) throws CDBException {
        String str = getProperty(name);
        if (str == null) {
            throw new CDBException("Required parameter " + name + " not found");
        } else {
            str = str.trim();
            if (str.equalsIgnoreCase("TRUE")) {
                return true;
            } else if (str.equalsIgnoreCase("FALSE")) {
                return false;
            } else {
                throw new CDBException("Boolean parameter " + name + " had invalid value: " + str);
            }
        }
    }

    /**
     * Returns the long value.
     * 
     * @param name
     *            a String containing the name of the property
     * @return the value of the property
     * @throws CDBException
     */
    public long getLong(String name) throws CDBException {
        String str = getProperty(name);
        if (str == null) {
            throw new CDBException("Required parameter " + name + " not found");
        } else {
            try {
                return Long.parseLong(str.trim());
            } catch (NumberFormatException e) {
                throw new CDBException("Long parameter " + name + " had invalid value: " + str);
            }
        }
    }

    /**
     * Returns the int value.
     * 
     * @param name
     *            a String containing the name of the property
     * @return the value of the property
     * @throws CDBException
     */
    public int getInt(String name) throws CDBException {
        String str = getProperty(name);
        if (str == null) {
            throw new CDBException("Required parameter " + name + " not found");
        } else {
            try {
                return Integer.parseInt(str.trim());
            } catch (NumberFormatException e) {
                throw new CDBException("Integer parameter " + name + " had invalid value: " + str);
            }
        }
    }
}
